@@ -1,15 +1,15 @@
 use std::collections::HashSet;
+
 use tantivy::collector::TopDocs;
-use tantivy::doc;
 use tantivy::query::BooleanQuery;
 use tantivy::schema::*;
-use tantivy::{DocId, Index, Score, SegmentReader};
+use tantivy::{doc, DocId, Index, Score, SegmentReader};
 
 fn main() -> tantivy::Result<()> {
     let mut schema_builder = Schema::builder();
 
     let title = schema_builder.add_text_field("title", STORED);
-    let ingredient = schema_builder.add_facet_field("ingredient", INDEXED);
+    let ingredient = schema_builder.add_facet_field("ingredient", FacetOptions::default());
 
     let schema = schema_builder.build();
     let index = Index::create_in_ram(schema);
@@ -20,14 +20,14 @@ fn main() -> tantivy::Result<()> {
         title => "Fried egg",
         ingredient => Facet::from("/ingredient/egg"),
         ingredient => Facet::from("/ingredient/oil"),
-    ));
+    ))?;
     index_writer.add_document(doc!(
         title => "Scrambled egg",
         ingredient => Facet::from("/ingredient/egg"),
         ingredient => Facet::from("/ingredient/butter"),
         ingredient => Facet::from("/ingredient/milk"),
         ingredient => Facet::from("/ingredient/salt"),
-    ));
+    ))?;
     index_writer.add_document(doc!(
         title => "Egg rolls",
         ingredient => Facet::from("/ingredient/egg"),
@@ -36,7 +36,7 @@ fn main() -> tantivy::Result<()> {
         ingredient => Facet::from("/ingredient/oil"),
         ingredient => Facet::from("/ingredient/tortilla-wrap"),
         ingredient => Facet::from("/ingredient/mushroom"),
-    ));
+    ))?;
     index_writer.commit()?;
 
     let reader = index.reader()?;
@@ -87,7 +87,7 @@ fn main() -> tantivy::Result<()> {
                     .unwrap()
                     .get_first(title)
                     .unwrap()
-                    .text()
+                    .as_text()
                     .unwrap()
                     .to_owned()
             })
