@@ -1,21 +1,14 @@
-use crate::collector::Collector;
-use crate::collector::SegmentCollector;
-use crate::fastfield::FacetReader;
-use crate::schema::Facet;
-use crate::schema::Field;
-use crate::DocId;
-use crate::Score;
-use crate::SegmentOrdinal;
-use crate::SegmentReader;
 use std::cmp::Ordering;
-use std::collections::btree_map;
-use std::collections::BTreeMap;
-use std::collections::BTreeSet;
-use std::collections::BinaryHeap;
+use std::collections::{btree_map, BTreeMap, BTreeSet, BinaryHeap};
 use std::iter::Peekable;
 use std::ops::Bound;
 use std::{u64, usize};
 
+use crate::collector::{Collector, SegmentCollector};
+use crate::fastfield::FacetReader;
+use crate::schema::{Facet, Field};
+use crate::{DocId, Score, SegmentOrdinal, SegmentReader};
+
 struct Hit<'a> {
     count: u64,
     facet: &'a Facet,
@@ -83,7 +76,7 @@ fn facet_depth(facet_bytes: &[u8]) -> usize {
 /// ```rust
 /// use tantivy::collector::FacetCollector;
 /// use tantivy::query::AllQuery;
-/// use tantivy::schema::{Facet, Schema, INDEXED, TEXT};
+/// use tantivy::schema::{Facet, Schema, FacetOptions, TEXT};
 /// use tantivy::{doc, Index};
 ///
 /// fn example() -> tantivy::Result<()> {
@@ -92,7 +85,7 @@ fn facet_depth(facet_bytes: &[u8]) -> usize {
 ///     // Facet have their own specific type.
 ///     // It is not a bad practise to put all of your
 ///     // facet information in the same field.
-///     let facet = schema_builder.add_facet_field("facet", INDEXED);
+///     let facet = schema_builder.add_facet_field("facet", FacetOptions::default());
 ///     let title = schema_builder.add_text_field("title", TEXT);
 ///     let schema = schema_builder.build();
 ///     let index = Index::create_in_ram(schema);
@@ -103,23 +96,23 @@ fn facet_depth(facet_bytes: &[u8]) -> usize {
 ///             title => "The Name of the Wind",
 ///             facet => Facet::from("/lang/en"),
 ///             facet => Facet::from("/category/fiction/fantasy")
-///         ));
+///         ))?;
 ///         index_writer.add_document(doc!(
 ///             title => "Dune",
 ///             facet => Facet::from("/lang/en"),
 ///             facet => Facet::from("/category/fiction/sci-fi")
-///         ));
+///         ))?;
 ///         index_writer.add_document(doc!(
 ///             title => "La Vénus d'Ille",
 ///             facet => Facet::from("/lang/fr"),
 ///             facet => Facet::from("/category/fiction/fantasy"),
 ///             facet => Facet::from("/category/fiction/horror")
-///         ));
+///         ))?;
 ///         index_writer.add_document(doc!(
 ///             title => "The Diary of a Young Girl",
 ///             facet => Facet::from("/lang/en"),
 ///             facet => Facet::from("/category/biography")
-///         ));
+///         ))?;
 ///         index_writer.commit()?;
 ///     }
 ///     let reader = index.reader()?;
@@ -240,9 +233,7 @@ impl FacetCollector {
     /// If you need the correct number of unique documents for two such facets,
     /// just add them in separate `FacetCollector`.
     pub fn add_facet<T>(&mut self, facet_from: T)
-    where
-        Facet: From<T>,
-    {
+    where Facet: From<T> {
         let facet = Facet::from(facet_from);
         for old_facet in &self.facets {
             assert!(
@@ -400,11 +391,9 @@ impl<'a> Iterator for FacetChildIterator<'a> {
 
 impl FacetCounts {
     /// Returns an iterator over all of the facet count pairs inside this result.
-    /// See the documentation for `FacetCollector` for a usage example.
+    /// See the documentation for [FacetCollector] for a usage example.
     pub fn get<T>(&self, facet_from: T) -> FacetChildIterator<'_>
-    where
-        Facet: From<T>,
-    {
+    where Facet: From<T> {
         let facet = Facet::from(facet_from);
         let left_bound = Bound::Excluded(facet.clone());
         let right_bound = if facet.is_root() {
@@ -421,11 +410,9 @@ impl FacetCounts {
     }
 
     /// Returns a vector of top `k` facets with their counts, sorted highest-to-lowest by counts.
-    /// See the documentation for `FacetCollector` for a usage example.
+    /// See the documentation for [FacetCollector] for a usage example.
     pub fn top_k<T>(&self, facet: T, k: usize) -> Vec<(&Facet, u64)>
-    where
-        Facet: From<T>,
-    {
+    where Facet: From<T> {
         let mut heap = BinaryHeap::with_capacity(k);
         let mut it = self.get(facet);
 
@@ -458,25 +445,27 @@ impl FacetCounts {
 
 #[cfg(test)]
 mod tests {
+    use std::iter;
+
+    use rand::distributions::Uniform;
+    use rand::prelude::SliceRandom;
+    use rand::{thread_rng, Rng};
+
     use super::{FacetCollector, FacetCounts};
     use crate::collector::Count;
     use crate::core::Index;
     use crate::query::{AllQuery, QueryParser, TermQuery};
-    use crate::schema::{Document, Facet, Field, IndexRecordOption, Schema, INDEXED};
+    use crate::schema::{Document, Facet, FacetOptions, Field, IndexRecordOption, Schema};
     use crate::Term;
-    use rand::distributions::Uniform;
-    use rand::prelude::SliceRandom;
-    use rand::{thread_rng, Rng};
-    use std::iter;
 
     #[test]
-    fn test_facet_collector_drilldown() {
+    fn test_facet_collector_drilldown() -> crate::Result<()> {
         let mut schema_builder = Schema::builder();
-        let facet_field = schema_builder.add_facet_field("facet", INDEXED);
+        let facet_field = schema_builder.add_facet_field("facet", FacetOptions::default());
         let schema = schema_builder.build();
         let index = Index::create_in_ram(schema);
 
-        let mut index_writer = index.writer_for_tests().unwrap();
+        let mut index_writer = index.writer_for_tests()?;
         let num_facets: usize = 3 * 4 * 5;
         let facets: Vec<Facet> = (0..num_facets)
             .map(|mut n| {
@@ -491,14 +480,14 @@ mod tests {
         for i in 0..num_facets * 10 {
             let mut doc = Document::new();
             doc.add_facet(facet_field, facets[i % num_facets].clone());
-            index_writer.add_document(doc);
+            index_writer.add_document(doc)?;
         }
-        index_writer.commit().unwrap();
-        let reader = index.reader().unwrap();
+        index_writer.commit()?;
+        let reader = index.reader()?;
         let searcher = reader.searcher();
         let mut facet_collector = FacetCollector::for_field(facet_field);
         facet_collector.add_facet(Facet::from("/top1"));
-        let counts = searcher.search(&AllQuery, &facet_collector).unwrap();
+        let counts = searcher.search(&AllQuery, &facet_collector)?;
 
         {
             let facets: Vec<(String, u64)> = counts
@@ -518,11 +507,13 @@ mod tests {
                 .collect::<Vec<_>>()
             );
         }
+        Ok(())
     }
 
     #[test]
-    #[should_panic(expected = "Tried to add a facet which is a descendant of \
-                               an already added facet.")]
+    #[should_panic(
+        expected = "Tried to add a facet which is a descendant of an already added facet."
+    )]
     fn test_misused_facet_collector() {
         let mut facet_collector = FacetCollector::for_field(Field::from_field_id(0));
         facet_collector.add_facet(Facet::from("/country"));
@@ -530,48 +521,49 @@ mod tests {
     }
 
     #[test]
-    fn test_doc_unsorted_multifacet() {
+    fn test_doc_unsorted_multifacet() -> crate::Result<()> {
         let mut schema_builder = Schema::builder();
-        let facet_field = schema_builder.add_facet_field("facets", INDEXED);
+        let facet_field = schema_builder.add_facet_field("facets", FacetOptions::default());
         let schema = schema_builder.build();
         let index = Index::create_in_ram(schema);
-        let mut index_writer = index.writer_for_tests().unwrap();
+        let mut index_writer = index.writer_for_tests()?;
         index_writer.add_document(doc!(
             facet_field => Facet::from_text(&"/subjects/A/a").unwrap(),
             facet_field => Facet::from_text(&"/subjects/B/a").unwrap(),
             facet_field => Facet::from_text(&"/subjects/A/b").unwrap(),
             facet_field => Facet::from_text(&"/subjects/B/b").unwrap(),
-        ));
-        index_writer.commit().unwrap();
-        let reader = index.reader().unwrap();
+        ))?;
+        index_writer.commit()?;
+        let reader = index.reader()?;
         let searcher = reader.searcher();
         assert_eq!(searcher.num_docs(), 1);
         let mut facet_collector = FacetCollector::for_field(facet_field);
         facet_collector.add_facet("/subjects");
-        let counts = searcher.search(&AllQuery, &facet_collector).unwrap();
+        let counts = searcher.search(&AllQuery, &facet_collector)?;
         let facets: Vec<(&Facet, u64)> = counts.get("/subjects").collect();
         assert_eq!(facets[0].1, 1);
+        Ok(())
     }
 
     #[test]
     fn test_doc_search_by_facet() -> crate::Result<()> {
         let mut schema_builder = Schema::builder();
-        let facet_field = schema_builder.add_facet_field("facet", INDEXED);
+        let facet_field = schema_builder.add_facet_field("facet", FacetOptions::default());
         let schema = schema_builder.build();
         let index = Index::create_in_ram(schema);
         let mut index_writer = index.writer_for_tests()?;
         index_writer.add_document(doc!(
             facet_field => Facet::from_text(&"/A/A").unwrap(),
-        ));
+        ))?;
         index_writer.add_document(doc!(
             facet_field => Facet::from_text(&"/A/B").unwrap(),
-        ));
+        ))?;
         index_writer.add_document(doc!(
             facet_field => Facet::from_text(&"/A/C/A").unwrap(),
-        ));
+        ))?;
         index_writer.add_document(doc!(
             facet_field => Facet::from_text(&"/D/C/A").unwrap(),
-        ));
+        ))?;
         index_writer.commit()?;
         let reader = index.reader()?;
         let searcher = reader.searcher();
@@ -613,7 +605,7 @@ mod tests {
     #[test]
     fn test_facet_collector_topk() {
         let mut schema_builder = Schema::builder();
-        let facet_field = schema_builder.add_facet_field("facet", INDEXED);
+        let facet_field = schema_builder.add_facet_field("facet", FacetOptions::default());
         let schema = schema_builder.build();
         let index = Index::create_in_ram(schema);
 
@@ -637,7 +629,7 @@ mod tests {
 
         let mut index_writer = index.writer_for_tests().unwrap();
         for doc in docs {
-            index_writer.add_document(doc);
+            index_writer.add_document(doc).unwrap();
         }
         index_writer.commit().unwrap();
         let searcher = index.reader().unwrap().searcher();
@@ -662,7 +654,7 @@ mod tests {
     #[test]
     fn test_facet_collector_topk_tie_break() -> crate::Result<()> {
         let mut schema_builder = Schema::builder();
-        let facet_field = schema_builder.add_facet_field("facet", INDEXED);
+        let facet_field = schema_builder.add_facet_field("facet", FacetOptions::default());
         let schema = schema_builder.build();
         let index = Index::create_in_ram(schema);
 
@@ -677,7 +669,7 @@ mod tests {
 
         let mut index_writer = index.writer_for_tests()?;
         for doc in docs {
-            index_writer.add_document(doc);
+            index_writer.add_document(doc)?;
         }
         index_writer.commit()?;
 
@@ -698,13 +690,14 @@ mod tests {
 #[cfg(all(test, feature = "unstable"))]
 mod bench {
 
+    use rand::seq::SliceRandom;
+    use rand::thread_rng;
+    use test::Bencher;
+
     use crate::collector::FacetCollector;
     use crate::query::AllQuery;
     use crate::schema::{Facet, Schema, INDEXED};
     use crate::Index;
-    use rand::seq::SliceRandom;
-    use rand::thread_rng;
-    use test::Bencher;
 
     #[bench]
     fn bench_facet_collector(b: &mut Bencher) {
@@ -725,7 +718,7 @@ mod bench {
 
         let mut index_writer = index.writer_for_tests().unwrap();
         for doc in docs {
-            index_writer.add_document(doc);
+            index_writer.add_document(doc).unwrap();
         }
         index_writer.commit().unwrap();
         let reader = index.reader().unwrap();
