@@ -1,14 +1,10 @@
-use super::Collector;
-use super::SegmentCollector;
-use crate::collector::Fruit;
-use crate::DocId;
-use crate::Score;
-use crate::SegmentOrdinal;
-use crate::SegmentReader;
-use crate::TantivyError;
 use std::marker::PhantomData;
 use std::ops::Deref;
 
+use super::{Collector, SegmentCollector};
+use crate::collector::Fruit;
+use crate::{DocId, Score, SegmentOrdinal, SegmentReader, TantivyError};
+
 pub struct MultiFruit {
     sub_fruits: Vec<Option<Box<dyn Fruit>>>,
 }
@@ -104,7 +100,8 @@ impl<TFruit: Fruit> FruitHandle<TFruit> {
 ///
 /// If the type of the collectors is known, you can just group yours collectors
 /// in a tuple. See the
-/// [Combining several collectors section of the collector documentation](./index.html#combining-several-collectors).
+/// [Combining several collectors section of the collector
+/// documentation](./index.html#combining-several-collectors).
 ///
 /// ```rust
 /// use tantivy::collector::{Count, TopDocs, MultiCollector};
@@ -112,19 +109,19 @@ impl<TFruit: Fruit> FruitHandle<TFruit> {
 /// use tantivy::schema::{Schema, TEXT};
 /// use tantivy::{doc, Index};
 ///
+/// # fn main() -> tantivy::Result<()> {
 /// let mut schema_builder = Schema::builder();
 /// let title = schema_builder.add_text_field("title", TEXT);
 /// let schema = schema_builder.build();
 /// let index = Index::create_in_ram(schema);
+/// let mut index_writer = index.writer(3_000_000)?;
+/// index_writer.add_document(doc!(title => "The Name of the Wind"))?;
+/// index_writer.add_document(doc!(title => "The Diary of Muadib"))?;
+/// index_writer.add_document(doc!(title => "A Dairy Cow"))?;
+/// index_writer.add_document(doc!(title => "The Diary of a Young Girl"))?;
+/// index_writer.commit()?;
 ///
-/// let mut index_writer = index.writer(3_000_000).unwrap();
-/// index_writer.add_document(doc!(title => "The Name of the Wind"));
-/// index_writer.add_document(doc!(title => "The Diary of Muadib"));
-/// index_writer.add_document(doc!(title => "A Dairy Cow"));
-/// index_writer.add_document(doc!(title => "The Diary of a Young Girl"));
-/// assert!(index_writer.commit().is_ok());
-///
-/// let reader = index.reader().unwrap();
+/// let reader = index.reader()?;
 /// let searcher = reader.searcher();
 ///
 /// let mut collectors = MultiCollector::new();
@@ -139,6 +136,8 @@ impl<TFruit: Fruit> FruitHandle<TFruit> {
 ///
 /// assert_eq!(count, 2);
 /// assert_eq!(top_docs.len(), 2);
+/// # Ok(())
+/// # }
 /// ```
 #[allow(clippy::type_complexity)]
 #[derive(Default)]
@@ -246,30 +245,28 @@ mod tests {
     use super::*;
     use crate::collector::{Count, TopDocs};
     use crate::query::TermQuery;
-    use crate::schema::IndexRecordOption;
-    use crate::schema::{Schema, TEXT};
-    use crate::Index;
-    use crate::Term;
+    use crate::schema::{IndexRecordOption, Schema, TEXT};
+    use crate::{Index, Term};
 
     #[test]
-    fn test_multi_collector() {
+    fn test_multi_collector() -> crate::Result<()> {
         let mut schema_builder = Schema::builder();
         let text = schema_builder.add_text_field("text", TEXT);
         let schema = schema_builder.build();
 
         let index = Index::create_in_ram(schema);
         {
-            let mut index_writer = index.writer_for_tests().unwrap();
-            index_writer.add_document(doc!(text=>"abc"));
-            index_writer.add_document(doc!(text=>"abc abc abc"));
-            index_writer.add_document(doc!(text=>"abc abc"));
-            index_writer.commit().unwrap();
-            index_writer.add_document(doc!(text=>""));
-            index_writer.add_document(doc!(text=>"abc abc abc abc"));
-            index_writer.add_document(doc!(text=>"abc"));
-            index_writer.commit().unwrap();
+            let mut index_writer = index.writer_for_tests()?;
+            index_writer.add_document(doc!(text=>"abc"))?;
+            index_writer.add_document(doc!(text=>"abc abc abc"))?;
+            index_writer.add_document(doc!(text=>"abc abc"))?;
+            index_writer.commit()?;
+            index_writer.add_document(doc!(text=>""))?;
+            index_writer.add_document(doc!(text=>"abc abc abc abc"))?;
+            index_writer.add_document(doc!(text=>"abc"))?;
+            index_writer.commit()?;
         }
-        let searcher = index.reader().unwrap().searcher();
+        let searcher = index.reader()?.searcher();
         let term = Term::from_field_text(text, "abc");
         let query = TermQuery::new(term, IndexRecordOption::Basic);
 
@@ -280,5 +277,6 @@ mod tests {
 
         assert_eq!(count_handler.extract(&mut multifruits), 5);
         assert_eq!(topdocs_handler.extract(&mut multifruits).len(), 2);
+        Ok(())
     }
 }
