@@ -1,21 +1,18 @@
+use std::collections::BinaryHeap;
+use std::fmt;
+use std::marker::PhantomData;
+
 use super::Collector;
-use crate::collector::top_collector::{ComparableDoc, TopCollector};
+use crate::collector::custom_score_top_collector::CustomScoreTopCollector;
+use crate::collector::top_collector::{ComparableDoc, TopCollector, TopSegmentCollector};
 use crate::collector::tweak_score_top_collector::TweakedScoreTopCollector;
 use crate::collector::{
     CustomScorer, CustomSegmentScorer, ScoreSegmentTweaker, ScoreTweaker, SegmentCollector,
 };
-use crate::fastfield::{DynamicFastFieldReader, FastFieldReader};
+use crate::fastfield::{DynamicFastFieldReader, FastFieldReader, FastValue};
 use crate::query::Weight;
 use crate::schema::Field;
-use crate::DocAddress;
-use crate::DocId;
-use crate::Score;
-use crate::SegmentOrdinal;
-use crate::SegmentReader;
-use crate::{collector::custom_score_top_collector::CustomScoreTopCollector, fastfield::FastValue};
-use crate::{collector::top_collector::TopSegmentCollector, TantivyError};
-use std::fmt;
-use std::{collections::BinaryHeap, marker::PhantomData};
+use crate::{DocAddress, DocId, Score, SegmentOrdinal, SegmentReader, TantivyError};
 
 struct FastFieldConvertCollector<
     TCollector: Collector<Fruit = Vec<(u64, DocAddress)>>,
@@ -94,27 +91,30 @@ where
 /// use tantivy::schema::{Schema, TEXT};
 /// use tantivy::{doc, DocAddress, Index};
 ///
+/// # fn main() -> tantivy::Result<()> {
 /// let mut schema_builder = Schema::builder();
 /// let title = schema_builder.add_text_field("title", TEXT);
 /// let schema = schema_builder.build();
 /// let index = Index::create_in_ram(schema);
 ///
-/// let mut index_writer = index.writer_with_num_threads(1, 10_000_000).unwrap();
-/// index_writer.add_document(doc!(title => "The Name of the Wind"));
-/// index_writer.add_document(doc!(title => "The Diary of Muadib"));
-/// index_writer.add_document(doc!(title => "A Dairy Cow"));
-/// index_writer.add_document(doc!(title => "The Diary of a Young Girl"));
-/// assert!(index_writer.commit().is_ok());
+/// let mut index_writer = index.writer_with_num_threads(1, 10_000_000)?;
+/// index_writer.add_document(doc!(title => "The Name of the Wind"))?;
+/// index_writer.add_document(doc!(title => "The Diary of Muadib"))?;
+/// index_writer.add_document(doc!(title => "A Dairy Cow"))?;
+/// index_writer.add_document(doc!(title => "The Diary of a Young Girl"))?;
+/// index_writer.commit()?;
 ///
-/// let reader = index.reader().unwrap();
+/// let reader = index.reader()?;
 /// let searcher = reader.searcher();
 ///
 /// let query_parser = QueryParser::for_index(&index, vec![title]);
-/// let query = query_parser.parse_query("diary").unwrap();
-/// let top_docs = searcher.search(&query, &TopDocs::with_limit(2)).unwrap();
+/// let query = query_parser.parse_query("diary")?;
+/// let top_docs = searcher.search(&query, &TopDocs::with_limit(2))?;
 ///
 /// assert_eq!(top_docs[0].1, DocAddress::new(0, 1));
 /// assert_eq!(top_docs[1].1, DocAddress::new(0, 3));
+/// # Ok(())
+/// # }
 /// ```
 pub struct TopDocs(TopCollector<Score>);
 
@@ -180,41 +180,46 @@ impl TopDocs {
     /// use tantivy::schema::{Schema, TEXT};
     /// use tantivy::{doc, DocAddress, Index};
     ///
+    /// # fn main() -> tantivy::Result<()> {
     /// let mut schema_builder = Schema::builder();
     /// let title = schema_builder.add_text_field("title", TEXT);
     /// let schema = schema_builder.build();
     /// let index = Index::create_in_ram(schema);
     ///
-    /// let mut index_writer = index.writer_with_num_threads(1, 10_000_000).unwrap();
-    /// index_writer.add_document(doc!(title => "The Name of the Wind"));
-    /// index_writer.add_document(doc!(title => "The Diary of Muadib"));
-    /// index_writer.add_document(doc!(title => "A Dairy Cow"));
-    /// index_writer.add_document(doc!(title => "The Diary of a Young Girl"));
-    /// index_writer.add_document(doc!(title => "The Diary of Lena Mukhina"));
-    /// assert!(index_writer.commit().is_ok());
+    /// let mut index_writer = index.writer_with_num_threads(1, 10_000_000)?;
+    /// index_writer.add_document(doc!(title => "The Name of the Wind"))?;
+    /// index_writer.add_document(doc!(title => "The Diary of Muadib"))?;
+    /// index_writer.add_document(doc!(title => "A Dairy Cow"))?;
+    /// index_writer.add_document(doc!(title => "The Diary of a Young Girl"))?;
+    /// index_writer.add_document(doc!(title => "The Diary of Lena Mukhina"))?;
+    /// index_writer.commit()?;
     ///
-    /// let reader = index.reader().unwrap();
+    /// let reader = index.reader()?;
     /// let searcher = reader.searcher();
     ///
     /// let query_parser = QueryParser::for_index(&index, vec![title]);
-    /// let query = query_parser.parse_query("diary").unwrap();
-    /// let top_docs = searcher.search(&query, &TopDocs::with_limit(2).and_offset(1)).unwrap();
+    /// let query = query_parser.parse_query("diary")?;
+    /// let top_docs = searcher.search(&query, &TopDocs::with_limit(2).and_offset(1))?;
     ///
     /// assert_eq!(top_docs.len(), 2);
     /// assert_eq!(top_docs[0].1, DocAddress::new(0, 4));
     /// assert_eq!(top_docs[1].1, DocAddress::new(0, 3));
+    /// Ok(())
+    /// # }
     /// ```
+    #[must_use]
     pub fn and_offset(self, offset: usize) -> TopDocs {
         TopDocs(self.0.and_offset(offset))
     }
 
     /// Set top-K to rank documents by a given fast field.
     ///
-    /// If the field is not a fast or does not exist, this method returns successfully (it is not aware of any schema).
-    /// An error will be returned at the moment of search.
+    /// If the field is not a fast or does not exist, this method returns successfully (it is not
+    /// aware of any schema). An error will be returned at the moment of search.
     ///
-    /// If the field is a FAST field but not a u64 field, search will return successfully but it will return
-    /// returns a monotonic u64-representation (ie. the order is still correct) of the requested field type.
+    /// If the field is a FAST field but not a u64 field, search will return successfully but it
+    /// will return returns a monotonic u64-representation (ie. the order is still correct) of
+    /// the requested field type.
     ///
     /// # Example
     ///
@@ -234,11 +239,11 @@ impl TopDocs {
     /// #
     /// #   let index = Index::create_in_ram(schema);
     /// #   let mut index_writer = index.writer_with_num_threads(1, 10_000_000)?;
-    /// #   index_writer.add_document(doc!(title => "The Name of the Wind", rating => 92u64));
-    /// #   index_writer.add_document(doc!(title => "The Diary of Muadib", rating => 97u64));
-    /// #   index_writer.add_document(doc!(title => "A Dairy Cow", rating => 63u64));
-    /// #   index_writer.add_document(doc!(title => "The Diary of a Young Girl", rating => 80u64));
-    /// #   assert!(index_writer.commit().is_ok());
+    /// #   index_writer.add_document(doc!(title => "The Name of the Wind", rating => 92u64))?;
+    /// #   index_writer.add_document(doc!(title => "The Diary of Muadib", rating => 97u64))?;
+    /// #   index_writer.add_document(doc!(title => "A Dairy Cow", rating => 63u64))?;
+    /// #   index_writer.add_document(doc!(title => "The Diary of a Young Girl", rating => 80u64))?;
+    /// #   index_writer.commit()?;
     /// #   let reader = index.reader()?;
     /// #   let query = QueryParser::for_index(&index, vec![title]).parse_query("diary")?;
     /// #   let top_docs = docs_sorted_by_rating(&reader.searcher(), &query, rating)?;
@@ -289,14 +294,15 @@ impl TopDocs {
 
     /// Set top-K to rank documents by a given fast field.
     ///
-    /// If the field is not a fast field, or its field type does not match the generic type, this method does not panic,
-    /// but an explicit error will be returned at the moment of collection.
+    /// If the field is not a fast field, or its field type does not match the generic type, this
+    /// method does not panic, but an explicit error will be returned at the moment of
+    /// collection.
     ///
     /// Note that this method is a generic. The requested fast field type will be often
     /// inferred in your code by the rust compiler.
     ///
-    /// Implementation-wise, for performance reason, tantivy will manipulate the u64 representation of your fast
-    /// field until the last moment.
+    /// Implementation-wise, for performance reason, tantivy will manipulate the u64 representation
+    /// of your fast field until the last moment.
     ///
     /// # Example
     ///
@@ -316,9 +322,9 @@ impl TopDocs {
     /// #
     /// #   let index = Index::create_in_ram(schema);
     /// #   let mut index_writer = index.writer_with_num_threads(1, 10_000_000)?;
-    /// #   index_writer.add_document(doc!(title => "MadCow Inc.", rating => 92_000_000i64));
-    /// #   index_writer.add_document(doc!(title => "Zozo Cow KKK", rating => 119_000_000i64));
-    /// #   index_writer.add_document(doc!(title => "Declining Cow", rating => -63_000_000i64));
+    /// #   index_writer.add_document(doc!(title => "MadCow Inc.", rating => 92_000_000i64))?;
+    /// #   index_writer.add_document(doc!(title => "Zozo Cow KKK", rating => 119_000_000i64))?;
+    /// #   index_writer.add_document(doc!(title => "Declining Cow", rating => -63_000_000i64))?;
     /// #   assert!(index_writer.commit().is_ok());
     /// #   let reader = index.reader()?;
     /// #   let top_docs = docs_sorted_by_revenue(&reader.searcher(), &AllQuery, rating)?;
@@ -417,9 +423,9 @@ impl TopDocs {
     ///   let mut index_writer = index.writer_with_num_threads(1, 10_000_000)?;
     ///   let product_name = index.schema().get_field("product_name").unwrap();
     ///   let popularity: Field = index.schema().get_field("popularity").unwrap();
-    ///   index_writer.add_document(doc!(product_name => "The Diary of Muadib", popularity => 1u64));
-    ///   index_writer.add_document(doc!(product_name => "A Dairy Cow", popularity => 10u64));
-    ///   index_writer.add_document(doc!(product_name => "The Diary of a Young Girl", popularity => 15u64));
+    ///   index_writer.add_document(doc!(product_name => "The Diary of Muadib", popularity => 1u64))?;
+    ///   index_writer.add_document(doc!(product_name => "A Dairy Cow", popularity => 10u64))?;
+    ///   index_writer.add_document(doc!(product_name => "The Diary of a Young Girl", popularity => 15u64))?;
     ///   index_writer.commit()?;
     ///   Ok(index)
     /// }
@@ -527,9 +533,9 @@ impl TopDocs {
     /// #
     /// let popularity: Field = index.schema().get_field("popularity").unwrap();
     /// let boosted: Field = index.schema().get_field("boosted").unwrap();
-    /// #   index_writer.add_document(doc!(boosted=>1u64, product_name => "The Diary of Muadib", popularity => 1u64));
-    /// #   index_writer.add_document(doc!(boosted=>0u64, product_name => "A Dairy Cow", popularity => 10u64));
-    /// #   index_writer.add_document(doc!(boosted=>0u64, product_name => "The Diary of a Young Girl", popularity => 15u64));
+    /// #   index_writer.add_document(doc!(boosted=>1u64, product_name => "The Diary of Muadib", popularity => 1u64))?;
+    /// #   index_writer.add_document(doc!(boosted=>0u64, product_name => "A Dairy Cow", popularity => 10u64))?;
+    /// #   index_writer.add_document(doc!(boosted=>0u64, product_name => "The Diary of a Young Girl", popularity => 15u64))?;
     /// #   index_writer.commit()?;
     /// // ...
     /// # let user_query = "diary";
@@ -629,10 +635,10 @@ impl Collector for TopDocs {
         let heap_len = self.0.limit + self.0.offset;
         let mut heap: BinaryHeap<ComparableDoc<Score, DocId>> = BinaryHeap::with_capacity(heap_len);
 
-        if let Some(delete_bitset) = reader.delete_bitset() {
+        if let Some(alive_bitset) = reader.alive_bitset() {
             let mut threshold = Score::MIN;
             weight.for_each_pruning(threshold, reader, &mut |doc, score| {
-                if delete_bitset.is_deleted(doc) {
+                if alive_bitset.is_deleted(doc) {
                     return threshold;
                 }
                 let heap_item = ComparableDoc {
@@ -708,25 +714,20 @@ mod tests {
     use crate::collector::Collector;
     use crate::query::{AllQuery, Query, QueryParser};
     use crate::schema::{Field, Schema, FAST, STORED, TEXT};
-    use crate::Index;
-    use crate::IndexWriter;
-    use crate::Score;
-    use crate::{DocAddress, DocId, SegmentReader};
+    use crate::{DocAddress, DocId, Index, IndexWriter, Score, SegmentReader};
 
-    fn make_index() -> Index {
+    fn make_index() -> crate::Result<Index> {
         let mut schema_builder = Schema::builder();
         let text_field = schema_builder.add_text_field("text", TEXT);
         let schema = schema_builder.build();
         let index = Index::create_in_ram(schema);
-        {
-            // writing the segment
-            let mut index_writer = index.writer_with_num_threads(1, 10_000_000).unwrap();
-            index_writer.add_document(doc!(text_field=>"Hello happy tax payer."));
-            index_writer.add_document(doc!(text_field=>"Droopy says hello happy tax payer"));
-            index_writer.add_document(doc!(text_field=>"I like Droopy"));
-            assert!(index_writer.commit().is_ok());
-        }
-        index
+        // writing the segment
+        let mut index_writer = index.writer_with_num_threads(1, 10_000_000)?;
+        index_writer.add_document(doc!(text_field=>"Hello happy tax payer."))?;
+        index_writer.add_document(doc!(text_field=>"Droopy says hello happy tax payer"))?;
+        index_writer.add_document(doc!(text_field=>"I like Droopy"))?;
+        index_writer.commit()?;
+        Ok(index)
     }
 
     fn assert_results_equals(results: &[(Score, DocAddress)], expected: &[(Score, DocAddress)]) {
@@ -737,17 +738,15 @@ mod tests {
     }
 
     #[test]
-    fn test_top_collector_not_at_capacity_without_offset() {
-        let index = make_index();
+    fn test_top_collector_not_at_capacity_without_offset() -> crate::Result<()> {
+        let index = make_index()?;
         let field = index.schema().get_field("text").unwrap();
         let query_parser = QueryParser::for_index(&index, vec![field]);
-        let text_query = query_parser.parse_query("droopy tax").unwrap();
+        let text_query = query_parser.parse_query("droopy tax")?;
         let score_docs: Vec<(Score, DocAddress)> = index
-            .reader()
-            .unwrap()
+            .reader()?
             .searcher()
-            .search(&text_query, &TopDocs::with_limit(4))
-            .unwrap();
+            .search(&text_query, &TopDocs::with_limit(4))?;
         assert_results_equals(
             &score_docs,
             &[
@@ -756,11 +755,12 @@ mod tests {
                 (0.48527452, DocAddress::new(0, 0)),
             ],
         );
+        Ok(())
     }
 
     #[test]
     fn test_top_collector_not_at_capacity_with_offset() {
-        let index = make_index();
+        let index = make_index().unwrap();
         let field = index.schema().get_field("text").unwrap();
         let query_parser = QueryParser::for_index(&index, vec![field]);
         let text_query = query_parser.parse_query("droopy tax").unwrap();
@@ -775,7 +775,7 @@ mod tests {
 
     #[test]
     fn test_top_collector_at_capacity() {
-        let index = make_index();
+        let index = make_index().unwrap();
         let field = index.schema().get_field("text").unwrap();
         let query_parser = QueryParser::for_index(&index, vec![field]);
         let text_query = query_parser.parse_query("droopy tax").unwrap();
@@ -796,7 +796,7 @@ mod tests {
 
     #[test]
     fn test_top_collector_at_capacity_with_offset() {
-        let index = make_index();
+        let index = make_index().unwrap();
         let field = index.schema().get_field("text").unwrap();
         let query_parser = QueryParser::for_index(&index, vec![field]);
         let text_query = query_parser.parse_query("droopy tax").unwrap();
@@ -817,7 +817,7 @@ mod tests {
 
     #[test]
     fn test_top_collector_stable_sorting() {
-        let index = make_index();
+        let index = make_index().unwrap();
 
         // using AllQuery to get a constant score
         let searcher = index.reader().unwrap().searcher();
@@ -848,29 +848,35 @@ mod tests {
     const SIZE: &str = "size";
 
     #[test]
-    fn test_top_field_collector_not_at_capacity() {
+    fn test_top_field_collector_not_at_capacity() -> crate::Result<()> {
         let mut schema_builder = Schema::builder();
         let title = schema_builder.add_text_field(TITLE, TEXT);
         let size = schema_builder.add_u64_field(SIZE, FAST);
         let schema = schema_builder.build();
         let (index, query) = index("beer", title, schema, |index_writer| {
-            index_writer.add_document(doc!(
-                title => "bottle of beer",
-                size => 12u64,
-            ));
-            index_writer.add_document(doc!(
-                title => "growler of beer",
-                size => 64u64,
-            ));
-            index_writer.add_document(doc!(
-                title => "pint of beer",
-                size => 16u64,
-            ));
+            index_writer
+                .add_document(doc!(
+                    title => "bottle of beer",
+                    size => 12u64,
+                ))
+                .unwrap();
+            index_writer
+                .add_document(doc!(
+                    title => "growler of beer",
+                    size => 64u64,
+                ))
+                .unwrap();
+            index_writer
+                .add_document(doc!(
+                    title => "pint of beer",
+                    size => 16u64,
+                ))
+                .unwrap();
         });
-        let searcher = index.reader().unwrap().searcher();
+        let searcher = index.reader()?.searcher();
 
         let top_collector = TopDocs::with_limit(4).order_by_u64_field(size);
-        let top_docs: Vec<(u64, DocAddress)> = searcher.search(&query, &top_collector).unwrap();
+        let top_docs: Vec<(u64, DocAddress)> = searcher.search(&query, &top_collector)?;
         assert_eq!(
             &top_docs[..],
             &[
@@ -879,6 +885,7 @@ mod tests {
                 (12, DocAddress::new(0, 0))
             ]
         );
+        Ok(())
     }
 
     #[test]
@@ -894,12 +901,12 @@ mod tests {
         index_writer.add_document(doc!(
             name => "Paul Robeson",
             birthday => pr_birthday
-        ));
+        ))?;
         let mr_birthday = crate::DateTime::from_str("1947-11-08T00:00:00+00:00")?;
         index_writer.add_document(doc!(
             name => "Minnie Riperton",
             birthday => mr_birthday
-        ));
+        ))?;
         index_writer.commit()?;
         let searcher = index.reader()?.searcher();
         let top_collector = TopDocs::with_limit(3).order_by_fast_field(birthday);
@@ -926,11 +933,11 @@ mod tests {
         index_writer.add_document(doc!(
                 city => "georgetown",
                 altitude =>  -1i64,
-        ));
+        ))?;
         index_writer.add_document(doc!(
             city => "tokyo",
             altitude =>  40i64,
-        ));
+        ))?;
         index_writer.commit()?;
         let searcher = index.reader()?.searcher();
         let top_collector = TopDocs::with_limit(3).order_by_fast_field(altitude);
@@ -956,11 +963,11 @@ mod tests {
         index_writer.add_document(doc!(
                 city => "georgetown",
                 altitude =>  -1.0f64,
-        ));
+        ))?;
         index_writer.add_document(doc!(
             city => "tokyo",
             altitude =>  40f64,
-        ));
+        ))?;
         index_writer.commit()?;
         let searcher = index.reader()?.searcher();
         let top_collector = TopDocs::with_limit(3).order_by_fast_field(altitude);
@@ -983,10 +990,12 @@ mod tests {
         let size = schema_builder.add_u64_field(SIZE, FAST);
         let schema = schema_builder.build();
         let (index, _) = index("beer", title, schema, |index_writer| {
-            index_writer.add_document(doc!(
-                title => "bottle of beer",
-                size => 12u64,
-            ));
+            index_writer
+                .add_document(doc!(
+                    title => "bottle of beer",
+                    size => 12u64,
+                ))
+                .unwrap();
         });
         let searcher = index.reader().unwrap().searcher();
         let top_collector = TopDocs::with_limit(4).order_by_u64_field(Field::from_field_id(2));
@@ -1003,7 +1012,7 @@ mod tests {
         let schema = schema_builder.build();
         let index = Index::create_in_ram(schema);
         let mut index_writer = index.writer_for_tests()?;
-        index_writer.add_document(doc!(size=>1u64));
+        index_writer.add_document(doc!(size=>1u64))?;
         index_writer.commit()?;
         let searcher = index.reader()?.searcher();
         let segment = searcher.segment_reader(0);
@@ -1020,7 +1029,7 @@ mod tests {
         let schema = schema_builder.build();
         let index = Index::create_in_ram(schema);
         let mut index_writer = index.writer_for_tests()?;
-        index_writer.add_document(doc!(size=>1u64));
+        index_writer.add_document(doc!(size=>1u64))?;
         index_writer.commit()?;
         let searcher = index.reader()?.searcher();
         let segment = searcher.segment_reader(0);
@@ -1033,30 +1042,26 @@ mod tests {
     }
 
     #[test]
-    fn test_tweak_score_top_collector_with_offset() {
-        let index = make_index();
+    fn test_tweak_score_top_collector_with_offset() -> crate::Result<()> {
+        let index = make_index()?;
         let field = index.schema().get_field("text").unwrap();
         let query_parser = QueryParser::for_index(&index, vec![field]);
-        let text_query = query_parser.parse_query("droopy tax").unwrap();
+        let text_query = query_parser.parse_query("droopy tax")?;
         let collector = TopDocs::with_limit(2).and_offset(1).tweak_score(
             move |_segment_reader: &SegmentReader| move |doc: DocId, _original_score: Score| doc,
         );
-        let score_docs: Vec<(u32, DocAddress)> = index
-            .reader()
-            .unwrap()
-            .searcher()
-            .search(&text_query, &collector)
-            .unwrap();
-
+        let score_docs: Vec<(u32, DocAddress)> =
+            index.reader()?.searcher().search(&text_query, &collector)?;
         assert_eq!(
             score_docs,
             vec![(1, DocAddress::new(0, 1)), (0, DocAddress::new(0, 0)),]
         );
+        Ok(())
     }
 
     #[test]
     fn test_custom_score_top_collector_with_offset() {
-        let index = make_index();
+        let index = make_index().unwrap();
         let field = index.schema().get_field("text").unwrap();
         let query_parser = QueryParser::for_index(&index, vec![field]);
         let text_query = query_parser.parse_query("droopy tax").unwrap();
