@@ -1,35 +1,27 @@
-use super::{segment::Segment, IndexSettings};
-use crate::core::Executor;
-use crate::core::IndexMeta;
-use crate::core::SegmentId;
-use crate::core::SegmentMeta;
-use crate::core::SegmentMetaInventory;
-use crate::core::META_FILEPATH;
-use crate::directory::error::OpenReadError;
-use crate::directory::ManagedDirectory;
-#[cfg(feature = "mmap")]
-use crate::directory::MmapDirectory;
-use crate::directory::INDEX_WRITER_LOCK;
-use crate::directory::{Directory, RamDirectory};
-use crate::error::DataCorruption;
-use crate::error::TantivyError;
-use crate::indexer::index_writer::{HEAP_SIZE_MIN, MAX_NUM_THREAD};
-use crate::indexer::segment_updater::save_new_metas;
-use crate::reader::IndexReader;
-use crate::reader::IndexReaderBuilder;
-use crate::schema::Field;
-use crate::schema::FieldType;
-use crate::schema::Schema;
-use crate::tokenizer::{TextAnalyzer, TokenizerManager};
-use crate::IndexWriter;
 use std::collections::HashSet;
 use std::fmt;
-
 #[cfg(feature = "mmap")]
 use std::path::Path;
 use std::path::PathBuf;
 use std::sync::Arc;
 
+use super::segment::Segment;
+use super::IndexSettings;
+use crate::core::{
+    Executor, IndexMeta, SegmentId, SegmentMeta, SegmentMetaInventory, META_FILEPATH,
+};
+use crate::directory::error::OpenReadError;
+#[cfg(feature = "mmap")]
+use crate::directory::MmapDirectory;
+use crate::directory::{Directory, ManagedDirectory, RamDirectory, INDEX_WRITER_LOCK};
+use crate::error::{DataCorruption, TantivyError};
+use crate::indexer::index_writer::{MAX_NUM_THREAD, MEMORY_ARENA_NUM_BYTES_MIN};
+use crate::indexer::segment_updater::save_new_metas;
+use crate::reader::{IndexReader, IndexReaderBuilder};
+use crate::schema::{Field, FieldType, Schema};
+use crate::tokenizer::{TextAnalyzer, TokenizerManager};
+use crate::IndexWriter;
+
 fn load_metas(
     directory: &dyn Directory,
     inventory: &SegmentMetaInventory,
@@ -72,13 +64,12 @@ fn load_metas(
 /// let body_field = schema_builder.add_text_field("body", TEXT);
 /// let number_field = schema_builder.add_u64_field(
 ///     "number",
-///     IntOptions::default().set_fast(Cardinality::SingleValue),
+///     NumericOptions::default().set_fast(Cardinality::SingleValue),
 /// );
 ///
 /// let schema = schema_builder.build();
 /// let settings = IndexSettings{sort_by_field: Some(IndexSortByField{field:"number".to_string(), order:Order::Asc}), ..Default::default()};
 /// let index = Index::builder().schema(schema).settings(settings).create_in_ram();
-///
 /// ```
 pub struct IndexBuilder {
     schema: Option<Schema>,
@@ -97,16 +88,21 @@ impl IndexBuilder {
             index_settings: IndexSettings::default(),
         }
     }
+
     /// Set the settings
+    #[must_use]
     pub fn settings(mut self, settings: IndexSettings) -> Self {
         self.index_settings = settings;
         self
     }
+
     /// Set the schema
+    #[must_use]
     pub fn schema(mut self, schema: Schema) -> Self {
         self.schema = Some(schema);
         self
     }
+
     /// Creates a new index using the `RAMDirectory`.
     ///
     /// The index will be allocated in anonymous memory.
@@ -117,18 +113,20 @@ impl IndexBuilder {
             .create(ram_directory)
             .expect("Creating a RAMDirectory should never fail"))
     }
+
     /// Creates a new index in a given filepath.
     /// The index will use the `MMapDirectory`.
     ///
     /// If a previous index was in this directory, it returns an `IndexAlreadyExists` error.
     #[cfg(feature = "mmap")]
     pub fn create_in_dir<P: AsRef<Path>>(self, directory_path: P) -> crate::Result<Index> {
-        let mmap_directory = MmapDirectory::open(directory_path)?;
-        if Index::exists(&mmap_directory)? {
+        let mmap_directory: Box<dyn Directory> = Box::new(MmapDirectory::open(directory_path)?);
+        if Index::exists(&*mmap_directory)? {
             return Err(TantivyError::IndexAlreadyExists);
         }
         self.create(mmap_directory)
     }
+
     /// Creates a new index in a temp directory.
     ///
     /// The index will use the `MMapDirectory` in a newly created directory.
@@ -139,18 +137,21 @@ impl IndexBuilder {
     /// For other unit tests, prefer the `RAMDirectory`, see: `create_in_ram`.
     #[cfg(feature = "mmap")]
     pub fn create_from_tempdir(self) -> crate::Result<Index> {
-        let mmap_directory = MmapDirectory::create_from_tempdir()?;
+        let mmap_directory: Box<dyn Directory> = Box::new(MmapDirectory::create_from_tempdir()?);
         self.create(mmap_directory)
     }
+
     fn get_expect_schema(&self) -> crate::Result<Schema> {
         self.schema
             .as_ref()
             .cloned()
             .ok_or(TantivyError::IndexBuilderMissingArgument("schema"))
     }
+
     /// Opens or creates a new index in the provided directory
-    pub fn open_or_create<Dir: Directory>(self, dir: Dir) -> crate::Result<Index> {
-        if !Index::exists(&dir)? {
+    pub fn open_or_create<T: Into<Box<dyn Directory>>>(self, dir: T) -> crate::Result<Index> {
+        let dir = dir.into();
+        if !Index::exists(&*dir)? {
             return self.create(dir);
         }
         let index = Index::open(dir)?;
@@ -165,7 +166,8 @@ impl IndexBuilder {
     /// Creates a new index given an implementation of the trait `Directory`.
     ///
     /// If a directory previously existed, it will be erased.
-    fn create<Dir: Directory>(self, dir: Dir) -> crate::Result<Index> {
+    fn create<T: Into<Box<dyn Directory>>>(self, dir: T) -> crate::Result<Index> {
+        let dir = dir.into();
         let directory = ManagedDirectory::wrap(dir)?;
         save_new_metas(
             self.get_expect_schema()?,
@@ -198,7 +200,7 @@ impl Index {
     /// Examines the directory to see if it contains an index.
     ///
     /// Effectively, it only checks for the presence of the `meta.json` file.
-    pub fn exists<Dir: Directory>(dir: &Dir) -> Result<bool, OpenReadError> {
+    pub fn exists(dir: &dyn Directory) -> Result<bool, OpenReadError> {
         dir.exists(&META_FILEPATH)
     }
 
@@ -215,7 +217,7 @@ impl Index {
     /// Replace the default single thread search executor pool
     /// by a thread pool with a given number of threads.
     pub fn set_multithread_executor(&mut self, num_threads: usize) -> crate::Result<()> {
-        self.executor = Arc::new(Executor::multi_thread(num_threads, "thrd-tantivy-search-")?);
+        self.executor = Arc::new(Executor::multi_thread(num_threads, "tantivy-search-")?);
         Ok(())
     }
 
@@ -250,7 +252,11 @@ impl Index {
     }
 
     /// Opens or creates a new index in the provided directory
-    pub fn open_or_create<Dir: Directory>(dir: Dir, schema: Schema) -> crate::Result<Index> {
+    pub fn open_or_create<T: Into<Box<dyn Directory>>>(
+        dir: T,
+        schema: Schema,
+    ) -> crate::Result<Index> {
+        let dir = dir.into();
         IndexBuilder::new().schema(schema).open_or_create(dir)
     }
 
@@ -270,11 +276,12 @@ impl Index {
     /// Creates a new index given an implementation of the trait `Directory`.
     ///
     /// If a directory previously existed, it will be erased.
-    pub fn create<Dir: Directory>(
-        dir: Dir,
+    pub fn create<T: Into<Box<dyn Directory>>>(
+        dir: T,
         schema: Schema,
         settings: IndexSettings,
     ) -> crate::Result<Index> {
+        let dir: Box<dyn Directory> = dir.into();
         let mut builder = IndexBuilder::new().schema(schema);
         builder = builder.settings(settings);
         builder.create(dir)
@@ -365,7 +372,8 @@ impl Index {
     }
 
     /// Open the index using the provided directory
-    pub fn open<D: Directory>(directory: D) -> crate::Result<Index> {
+    pub fn open<T: Into<Box<dyn Directory>>>(directory: T) -> crate::Result<Index> {
+        let directory = directory.into();
         let directory = ManagedDirectory::wrap(directory)?;
         let inventory = SegmentMetaInventory::default();
         let metas = load_metas(&directory, &inventory)?;
@@ -389,19 +397,18 @@ impl Index {
     /// - `num_threads` defines the number of indexing workers that
     /// should work at the same time.
     ///
-    /// - `overall_heap_size_in_bytes` sets the amount of memory
+    /// - `overall_memory_arena_in_bytes` sets the amount of memory
     /// allocated for all indexing thread.
-    /// Each thread will receive a budget of  `overall_heap_size_in_bytes / num_threads`.
+    /// Each thread will receive a budget of  `overall_memory_arena_in_bytes / num_threads`.
     ///
     /// # Errors
     /// If the lockfile already exists, returns `Error::DirectoryLockBusy` or an `Error::IoError`.
-    ///
-    /// # Panics
-    /// If the heap size per thread is too small, panics.
+    /// If the memory arena per thread is too small or too big, returns
+    /// `TantivyError::InvalidArgument`
     pub fn writer_with_num_threads(
         &self,
         num_threads: usize,
-        overall_heap_size_in_bytes: usize,
+        overall_memory_arena_in_bytes: usize,
     ) -> crate::Result<IndexWriter> {
         let directory_lock = self
             .directory
@@ -410,26 +417,25 @@ impl Index {
                 TantivyError::LockFailure(
                     err,
                     Some(
-                        "Failed to acquire index lock. If you are using \
-                         a regular directory, this means there is already an \
-                         `IndexWriter` working on this `Directory`, in this process \
-                         or in a different process."
+                        "Failed to acquire index lock. If you are using a regular directory, this \
+                         means there is already an `IndexWriter` working on this `Directory`, in \
+                         this process or in a different process."
                             .to_string(),
                     ),
                 )
             })?;
-        let heap_size_in_bytes_per_thread = overall_heap_size_in_bytes / num_threads;
+        let memory_arena_in_bytes_per_thread = overall_memory_arena_in_bytes / num_threads;
         IndexWriter::new(
             self,
             num_threads,
-            heap_size_in_bytes_per_thread,
+            memory_arena_in_bytes_per_thread,
             directory_lock,
         )
     }
 
     /// Helper to create an index writer for tests.
     ///
-    /// That index writer only simply has a single thread and a heap of 10 MB.
+    /// That index writer only simply has a single thread and a memory arena of 10 MB.
     /// Using a single thread gives us a deterministic allocation of DocId.
     #[cfg(test)]
     pub fn writer_for_tests(&self) -> crate::Result<IndexWriter> {
@@ -439,31 +445,29 @@ impl Index {
     /// Creates a multithreaded writer
     ///
     /// Tantivy will automatically define the number of threads to use, but
-    /// no more than [`MAX_NUM_THREAD`] threads.
-    /// `overall_heap_size_in_bytes` is the total target memory usage that will be split
+    /// no more than 8 threads.
+    /// `overall_memory_arena_in_bytes` is the total target memory usage that will be split
     /// between a given number of threads.
     ///
     /// # Errors
     /// If the lockfile already exists, returns `Error::FileAlreadyExists`.
-    /// # Panics
-    /// If the heap size per thread is too small, panics.
-    pub fn writer(&self, overall_heap_size_in_bytes: usize) -> crate::Result<IndexWriter> {
+    /// If the memory arena per thread is too small or too big, returns
+    /// `TantivyError::InvalidArgument`
+    pub fn writer(&self, memory_arena_num_bytes: usize) -> crate::Result<IndexWriter> {
         let mut num_threads = std::cmp::min(num_cpus::get(), MAX_NUM_THREAD);
-        let heap_size_in_bytes_per_thread = overall_heap_size_in_bytes / num_threads;
-        if heap_size_in_bytes_per_thread < HEAP_SIZE_MIN {
-            num_threads = (overall_heap_size_in_bytes / HEAP_SIZE_MIN).max(1);
+        let memory_arena_num_bytes_per_thread = memory_arena_num_bytes / num_threads;
+        if memory_arena_num_bytes_per_thread < MEMORY_ARENA_NUM_BYTES_MIN {
+            num_threads = (memory_arena_num_bytes / MEMORY_ARENA_NUM_BYTES_MIN).max(1);
         }
-        self.writer_with_num_threads(num_threads, overall_heap_size_in_bytes)
+        self.writer_with_num_threads(num_threads, memory_arena_num_bytes)
     }
 
     /// Accessor to the index settings
-    ///
     pub fn settings(&self) -> &IndexSettings {
         &self.settings
     }
 
     /// Accessor to the index settings
-    ///
     pub fn settings_mut(&mut self) -> &mut IndexSettings {
         &mut self.settings
     }
@@ -551,15 +555,9 @@ impl fmt::Debug for Index {
 
 #[cfg(test)]
 mod tests {
-    use crate::schema::Field;
-    use crate::schema::{Schema, INDEXED, TEXT};
-    use crate::IndexReader;
-    use crate::ReloadPolicy;
-    use crate::{
-        directory::{RamDirectory, WatchCallback},
-        IndexSettings,
-    };
-    use crate::{Directory, Index};
+    use crate::directory::{RamDirectory, WatchCallback};
+    use crate::schema::{Field, Schema, INDEXED, TEXT};
+    use crate::{Directory, Index, IndexReader, IndexSettings, ReloadPolicy};
 
     #[test]
     fn test_indexer_for_field() {
@@ -577,15 +575,15 @@ mod tests {
 
     #[test]
     fn test_index_exists() {
-        let directory = RamDirectory::create();
-        assert!(!Index::exists(&directory).unwrap());
+        let directory: Box<dyn Directory> = Box::new(RamDirectory::create());
+        assert!(!Index::exists(directory.as_ref()).unwrap());
         assert!(Index::create(
             directory.clone(),
             throw_away_schema(),
             IndexSettings::default()
         )
         .is_ok());
-        assert!(Index::exists(&directory).unwrap());
+        assert!(Index::exists(directory.as_ref()).unwrap());
     }
 
     #[test]
@@ -598,27 +596,27 @@ mod tests {
 
     #[test]
     fn open_or_create_should_open() {
-        let directory = RamDirectory::create();
+        let directory: Box<dyn Directory> = Box::new(RamDirectory::create());
         assert!(Index::create(
             directory.clone(),
             throw_away_schema(),
             IndexSettings::default()
         )
         .is_ok());
-        assert!(Index::exists(&directory).unwrap());
+        assert!(Index::exists(directory.as_ref()).unwrap());
         assert!(Index::open_or_create(directory, throw_away_schema()).is_ok());
     }
 
     #[test]
     fn create_should_wipeoff_existing() {
-        let directory = RamDirectory::create();
+        let directory: Box<dyn Directory> = Box::new(RamDirectory::create());
         assert!(Index::create(
             directory.clone(),
             throw_away_schema(),
             IndexSettings::default()
         )
         .is_ok());
-        assert!(Index::exists(&directory).unwrap());
+        assert!(Index::exists(directory.as_ref()).unwrap());
         assert!(Index::create(
             directory,
             Schema::builder().build(),
@@ -652,7 +650,7 @@ mod tests {
     }
 
     #[test]
-    fn test_index_on_commit_reload_policy() {
+    fn test_index_on_commit_reload_policy() -> crate::Result<()> {
         let schema = throw_away_schema();
         let field = schema.get_field("num_likes").unwrap();
         let index = Index::create_in_ram(schema);
@@ -662,19 +660,21 @@ mod tests {
             .try_into()
             .unwrap();
         assert_eq!(reader.searcher().num_docs(), 0);
-        test_index_on_commit_reload_policy_aux(field, &index, &reader);
+        test_index_on_commit_reload_policy_aux(field, &index, &reader)
     }
 
     #[cfg(feature = "mmap")]
     mod mmap_specific {
 
-        use super::*;
-        use crate::Directory;
         use std::path::PathBuf;
+
         use tempfile::TempDir;
 
+        use super::*;
+        use crate::Directory;
+
         #[test]
-        fn test_index_on_commit_reload_policy_mmap() {
+        fn test_index_on_commit_reload_policy_mmap() -> crate::Result<()> {
             let schema = throw_away_schema();
             let field = schema.get_field("num_likes").unwrap();
             let tempdir = TempDir::new().unwrap();
@@ -686,7 +686,7 @@ mod tests {
                 .try_into()
                 .unwrap();
             assert_eq!(reader.searcher().num_docs(), 0);
-            test_index_on_commit_reload_policy_aux(field, &index, &reader);
+            test_index_on_commit_reload_policy_aux(field, &index, &reader)
         }
 
         #[test]
@@ -701,7 +701,7 @@ mod tests {
                 .reload_policy(ReloadPolicy::Manual)
                 .try_into()?;
             assert_eq!(reader.searcher().num_docs(), 0);
-            writer.add_document(doc!(field=>1u64));
+            writer.add_document(doc!(field=>1u64))?;
             let (sender, receiver) = crossbeam::channel::unbounded();
             let _handle = index.directory_mut().watch(WatchCallback::new(move || {
                 let _ = sender.send(());
@@ -715,7 +715,7 @@ mod tests {
         }
 
         #[test]
-        fn test_index_on_commit_reload_policy_different_directories() {
+        fn test_index_on_commit_reload_policy_different_directories() -> crate::Result<()> {
             let schema = throw_away_schema();
             let field = schema.get_field("num_likes").unwrap();
             let tempdir = TempDir::new().unwrap();
@@ -728,10 +728,14 @@ mod tests {
                 .try_into()
                 .unwrap();
             assert_eq!(reader.searcher().num_docs(), 0);
-            test_index_on_commit_reload_policy_aux(field, &write_index, &reader);
+            test_index_on_commit_reload_policy_aux(field, &write_index, &reader)
         }
     }
-    fn test_index_on_commit_reload_policy_aux(field: Field, index: &Index, reader: &IndexReader) {
+    fn test_index_on_commit_reload_policy_aux(
+        field: Field,
+        index: &Index,
+        reader: &IndexReader,
+    ) -> crate::Result<()> {
         let mut reader_index = reader.index();
         let (sender, receiver) = crossbeam::channel::unbounded();
         let _watch_handle = reader_index
@@ -739,9 +743,9 @@ mod tests {
             .watch(WatchCallback::new(move || {
                 let _ = sender.send(());
             }));
-        let mut writer = index.writer_for_tests().unwrap();
+        let mut writer = index.writer_for_tests()?;
         assert_eq!(reader.searcher().num_docs(), 0);
-        writer.add_document(doc!(field=>1u64));
+        writer.add_document(doc!(field=>1u64))?;
         writer.commit().unwrap();
         // We need a loop here because it is possible for notify to send more than
         // one modify event. It was observed on CI on MacOS.
@@ -751,7 +755,7 @@ mod tests {
                 break;
             }
         }
-        writer.add_document(doc!(field=>2u64));
+        writer.add_document(doc!(field=>2u64))?;
         writer.commit().unwrap();
         // ... Same as above
         loop {
@@ -760,37 +764,37 @@ mod tests {
                 break;
             }
         }
+        Ok(())
     }
 
     // This test will not pass on windows, because windows
     // prevent deleting files that are MMapped.
     #[cfg(not(target_os = "windows"))]
     #[test]
-    fn garbage_collect_works_as_intended() {
+    fn garbage_collect_works_as_intended() -> crate::Result<()> {
         let directory = RamDirectory::create();
         let schema = throw_away_schema();
         let field = schema.get_field("num_likes").unwrap();
-        let index = Index::create(directory.clone(), schema, IndexSettings::default()).unwrap();
+        let index = Index::create(directory.clone(), schema, IndexSettings::default())?;
 
         let mut writer = index.writer_with_num_threads(8, 24_000_000).unwrap();
         for i in 0u64..8_000u64 {
-            writer.add_document(doc!(field => i));
+            writer.add_document(doc!(field => i))?;
         }
         let (sender, receiver) = crossbeam::channel::unbounded();
         let _handle = directory.watch(WatchCallback::new(move || {
             let _ = sender.send(());
         }));
-        writer.commit().unwrap();
+        writer.commit()?;
         let mem_right_after_commit = directory.total_mem_usage();
         assert!(receiver.recv().is_ok());
         let reader = index
             .reader_builder()
             .reload_policy(ReloadPolicy::Manual)
-            .try_into()
-            .unwrap();
+            .try_into()?;
 
         assert_eq!(reader.searcher().num_docs(), 8_000);
-        writer.wait_merging_threads().unwrap();
+        writer.wait_merging_threads()?;
         let mem_right_after_merge_finished = directory.total_mem_usage();
 
         reader.reload().unwrap();
@@ -802,5 +806,6 @@ mod tests {
             mem_right_after_merge_finished,
             mem_right_after_commit
         );
+        Ok(())
     }
 }
