@@ -1,12 +1,16 @@
-use super::SegmentComponent;
-use crate::schema::Schema;
-use crate::Opstamp;
-use crate::{core::SegmentId, store::Compressor};
-use census::{Inventory, TrackedObject};
-use serde::{Deserialize, Serialize};
+use std::collections::HashSet;
+use std::fmt;
 use std::path::PathBuf;
-use std::{collections::HashSet, sync::atomic::AtomicBool};
-use std::{fmt, sync::Arc};
+use std::sync::atomic::AtomicBool;
+use std::sync::Arc;
+
+use serde::{Deserialize, Serialize};
+
+use super::SegmentComponent;
+use crate::core::SegmentId;
+use crate::schema::Schema;
+use crate::store::Compressor;
+use crate::{Inventory, Opstamp, TrackedObject};
 
 #[derive(Clone, Debug, Serialize, Deserialize)]
 struct DeleteMeta {
@@ -188,7 +192,12 @@ impl SegmentMeta {
     }
 
     #[doc(hidden)]
+    #[must_use]
     pub fn with_delete_meta(self, num_deleted_docs: u32, opstamp: Opstamp) -> SegmentMeta {
+        assert!(
+            num_deleted_docs <= self.max_doc(),
+            "There cannot be more deleted docs than there are docs."
+        );
         let delete_meta = DeleteMeta {
             num_deleted_docs,
             opstamp,
@@ -278,7 +287,6 @@ impl Order {
 /// * the searchable segments,
 /// * the index `docstamp`
 /// * the schema
-///
 #[derive(Clone, Serialize)]
 pub struct IndexMeta {
     /// `IndexSettings` to configure index options.
@@ -366,10 +374,8 @@ impl fmt::Debug for IndexMeta {
 mod tests {
 
     use super::IndexMeta;
-    use crate::{
-        schema::{Schema, TEXT},
-        IndexSettings, IndexSortByField, Order,
-    };
+    use crate::schema::{Schema, TEXT};
+    use crate::{IndexSettings, IndexSortByField, Order};
 
     #[test]
     fn test_serialize_metas() {
@@ -394,7 +400,7 @@ mod tests {
         let json = serde_json::ser::to_string(&index_metas).expect("serialization failed");
         assert_eq!(
             json,
-            r#"{"index_settings":{"sort_by_field":{"field":"text","order":"Asc"},"docstore_compression":"lz4"},"segments":[],"schema":[{"name":"text","type":"text","options":{"indexing":{"record":"position","tokenizer":"default"},"stored":false}}],"opstamp":0}"#
+            r#"{"index_settings":{"sort_by_field":{"field":"text","order":"Asc"},"docstore_compression":"lz4"},"segments":[],"schema":[{"name":"text","type":"text","options":{"indexing":{"record":"position","fieldnorms":true,"tokenizer":"default"},"stored":false}}],"opstamp":0}"#
         );
     }
 }
