@@ -1,15 +1,13 @@
-use super::SegmentComponent;
-use crate::core::Index;
-use crate::core::SegmentId;
-use crate::core::SegmentMeta;
-use crate::directory::error::{OpenReadError, OpenWriteError};
-use crate::directory::Directory;
-use crate::directory::{FileSlice, WritePtr};
-use crate::schema::Schema;
-use crate::Opstamp;
 use std::fmt;
 use std::path::PathBuf;
 
+use super::SegmentComponent;
+use crate::core::{Index, SegmentId, SegmentMeta};
+use crate::directory::error::{OpenReadError, OpenWriteError};
+use crate::directory::{Directory, FileSlice, WritePtr};
+use crate::schema::Schema;
+use crate::Opstamp;
+
 /// A segment is a piece of the index.
 #[derive(Clone)]
 pub struct Segment {
@@ -56,6 +54,7 @@ impl Segment {
     }
 
     #[doc(hidden)]
+    #[must_use]
     pub fn with_delete_meta(self, num_deleted_docs: u32, opstamp: Opstamp) -> Segment {
         Segment {
             index: self.index,
