@@ -1,11 +1,12 @@
+use std::ops::{Deref, Range};
+use std::sync::{Arc, Weak};
+use std::{fmt, io};
+
+use async_trait::async_trait;
+use common::HasLen;
 use stable_deref_trait::StableDeref;
 
 use crate::directory::OwnedBytes;
-use common::HasLen;
-use std::fmt;
-use std::ops::Range;
-use std::sync::{Arc, Weak};
-use std::{io, ops::Deref};
 
 pub type ArcBytes = Arc<dyn Deref<Target = [u8]> + Send + Sync + 'static>;
 pub type WeakArcBytes = Weak<dyn Deref<Target = [u8]> + Send + Sync + 'static>;
@@ -18,23 +19,39 @@ pub type WeakArcBytes = Weak<dyn Deref<Target = [u8]> + Send + Sync + 'static>;
 /// The underlying behavior is therefore specific to the `Directory` that created it.
 /// Despite its name, a `FileSlice` may or may not directly map to an actual file
 /// on the filesystem.
+
+#[async_trait]
 pub trait FileHandle: 'static + Send + Sync + HasLen + fmt::Debug {
     /// Reads a slice of bytes.
     ///
     /// This method may panic if the range requested is invalid.
     fn read_bytes(&self, range: Range<usize>) -> io::Result<OwnedBytes>;
+
+    #[cfg(feature = "quickwit")]
+    #[doc(hidden)]
+    async fn read_bytes_async(
+        &self,
+        _byte_range: Range<usize>,
+    ) -> crate::AsyncIoResult<OwnedBytes> {
+        Err(crate::error::AsyncIoError::AsyncUnsupported)
+    }
 }
 
+#[async_trait]
 impl FileHandle for &'static [u8] {
     fn read_bytes(&self, range: Range<usize>) -> io::Result<OwnedBytes> {
         let bytes = &self[range];
         Ok(OwnedBytes::new(bytes))
     }
+
+    #[cfg(feature = "quickwit")]
+    async fn read_bytes_async(&self, byte_range: Range<usize>) -> crate::AsyncIoResult<OwnedBytes> {
+        Ok(self.read_bytes(byte_range)?)
+    }
 }
 
 impl<B> From<B> for FileSlice
-where
-    B: StableDeref + Deref<Target = [u8]> + 'static + Send + Sync,
+where B: StableDeref + Deref<Target = [u8]> + 'static + Send + Sync
 {
     fn from(bytes: B) -> FileSlice {
         FileSlice::new(Box::new(OwnedBytes::new(bytes)))
@@ -44,7 +61,6 @@ where
 /// Logical slice of read only file in tantivy.
 ///
 /// It can be cloned and sliced cheaply.
-///
 #[derive(Clone)]
 pub struct FileSlice {
     data: Arc<dyn FileHandle>,
@@ -66,6 +82,7 @@ impl FileSlice {
 
     /// Wraps a FileHandle.
     #[doc(hidden)]
+    #[must_use]
     pub fn new_with_num_bytes(file_handle: Box<dyn FileHandle>, num_bytes: usize) -> Self {
         FileSlice {
             data: Arc::from(file_handle),
@@ -78,6 +95,7 @@ impl FileSlice {
     /// # Panics
     ///
     /// Panics if `byte_range.end` exceeds the filesize.
+    #[must_use]
     pub fn slice(&self, byte_range: Range<usize>) -> FileSlice {
         assert!(byte_range.end <= self.len());
         FileSlice {
@@ -102,6 +120,12 @@ impl FileSlice {
         self.data.read_bytes(self.range.clone())
     }
 
+    #[cfg(feature = "quickwit")]
+    #[doc(hidden)]
+    pub async fn read_bytes_async(&self) -> crate::AsyncIoResult<OwnedBytes> {
+        self.data.read_bytes_async(self.range.clone()).await
+    }
+
     /// Reads a specific slice of data.
     ///
     /// This is equivalent to running `file_slice.slice(from, to).read_bytes()`.
@@ -116,6 +140,23 @@ impl FileSlice {
             .read_bytes(self.range.start + range.start..self.range.start + range.end)
     }
 
+    #[cfg(feature = "quickwit")]
+    #[doc(hidden)]
+    pub async fn read_bytes_slice_async(
+        &self,
+        byte_range: Range<usize>,
+    ) -> crate::AsyncIoResult<OwnedBytes> {
+        assert!(
+            self.range.start + byte_range.end <= self.range.end,
+            "`to` exceeds the fileslice length"
+        );
+        self.data
+            .read_bytes_async(
+                self.range.start + byte_range.start..self.range.start + byte_range.end,
+            )
+            .await
+    }
+
     /// Splits the FileSlice at the given offset and return two file slices.
     /// `file_slice[..split_offset]` and `file_slice[split_offset..]`.
     ///
@@ -137,6 +178,7 @@ impl FileSlice {
     /// boundary.
     ///
     /// Equivalent to `.slice(from_offset, self.len())`
+    #[must_use]
     pub fn slice_from(&self, from_offset: usize) -> FileSlice {
         self.slice(from_offset..self.len())
     }
@@ -144,6 +186,7 @@ impl FileSlice {
     /// Returns a slice from the end.
     ///
     /// Equivalent to `.slice(self.len() - from_offset, self.len())`
+    #[must_use]
     pub fn slice_from_end(&self, from_offset: usize) -> FileSlice {
         self.slice(self.len() - from_offset..self.len())
     }
@@ -152,15 +195,22 @@ impl FileSlice {
     /// boundary.
     ///
     /// Equivalent to `.slice(0, to_offset)`
+    #[must_use]
     pub fn slice_to(&self, to_offset: usize) -> FileSlice {
         self.slice(0..to_offset)
     }
 }
 
+#[async_trait]
 impl FileHandle for FileSlice {
     fn read_bytes(&self, range: Range<usize>) -> io::Result<OwnedBytes> {
         self.read_bytes_slice(range)
     }
+
+    #[cfg(feature = "quickwit")]
+    async fn read_bytes_async(&self, byte_range: Range<usize>) -> crate::AsyncIoResult<OwnedBytes> {
+        self.read_bytes_slice_async(byte_range).await
+    }
 }
 
 impl HasLen for FileSlice {
@@ -169,12 +219,27 @@ impl HasLen for FileSlice {
     }
 }
 
+#[async_trait]
+impl FileHandle for OwnedBytes {
+    fn read_bytes(&self, range: Range<usize>) -> io::Result<OwnedBytes> {
+        Ok(self.slice(range))
+    }
+
+    #[cfg(feature = "quickwit")]
+    async fn read_bytes_async(&self, range: Range<usize>) -> crate::AsyncIoResult<OwnedBytes> {
+        let bytes = self.read_bytes(range)?;
+        Ok(bytes)
+    }
+}
+
 #[cfg(test)]
 mod tests {
-    use super::{FileHandle, FileSlice};
-    use common::HasLen;
     use std::io;
 
+    use common::HasLen;
+
+    use super::{FileHandle, FileSlice};
+
     #[test]
     fn test_file_slice() -> io::Result<()> {
         let file_slice = FileSlice::new(Box::new(b"abcdef".as_ref()));
