@@ -1,30 +1,23 @@
-use crate::core::META_FILEPATH;
-use crate::directory::error::{DeleteError, OpenReadError, OpenWriteError};
-use crate::directory::AntiCallToken;
-use crate::directory::WatchCallbackList;
-use crate::directory::{Directory, FileSlice, WatchCallback, WatchHandle};
-use crate::directory::{TerminatingWrite, WritePtr};
-use common::HasLen;
-use fail::fail_point;
 use std::collections::HashMap;
-use std::fmt;
 use std::io::{self, BufWriter, Cursor, Seek, SeekFrom, Write};
 use std::path::{Path, PathBuf};
-use std::result;
 use std::sync::{Arc, RwLock};
+use std::{fmt, result};
+
+use common::HasLen;
+use fail::fail_point;
 
 use super::FileHandle;
+use crate::core::META_FILEPATH;
+use crate::directory::error::{DeleteError, OpenReadError, OpenWriteError};
+use crate::directory::{
+    AntiCallToken, Directory, FileSlice, TerminatingWrite, WatchCallback, WatchCallbackList,
+    WatchHandle, WritePtr,
+};
 
 /// Writer associated with the `RamDirectory`
 ///
 /// The Writer just writes a buffer.
-///
-/// # Panics
-///
-/// On drop, if the writer was left in a *dirty* state.
-/// That is, if flush was not called after the last call
-/// to write.
-///
 struct VecWriter {
     path: PathBuf,
     shared_directory: RamDirectory,
@@ -46,8 +39,10 @@ impl VecWriter {
 impl Drop for VecWriter {
     fn drop(&mut self) {
         if !self.is_flushed {
-            panic!(
-                "You forgot to flush {:?} before its writter got Drop. Do not rely on drop. This also occurs when the indexer crashed, so you may want to check the logs for the root cause.",
+            warn!(
+                "You forgot to flush {:?} before its writter got Drop. Do not rely on drop. This \
+                 also occurs when the indexer crashed, so you may want to check the logs for the \
+                 root cause.",
                 self.path
             )
         }
@@ -130,7 +125,6 @@ impl fmt::Debug for RamDirectory {
 ///
 /// It is mainly meant for unit testing.
 /// Writes are only made visible upon flushing.
-///
 #[derive(Clone, Default)]
 pub struct RamDirectory {
     fs: Arc<RwLock<InnerDirectory>>,
@@ -221,14 +215,8 @@ impl Directory for RamDirectory {
     }
 
     fn atomic_write(&self, path: &Path, data: &[u8]) -> io::Result<()> {
-        fail_point!("RamDirectory::atomic_write", |msg| Err(io::Error::new(
-            io::ErrorKind::Other,
-            msg.unwrap_or_else(|| "Undefined".to_string())
-        )));
         let path_buf = PathBuf::from(path);
-
         self.fs.write().unwrap().write(path_buf, data);
-
         if path == *META_FILEPATH {
             let _ = self.fs.write().unwrap().watch_router.broadcast();
         }
@@ -238,15 +226,20 @@ impl Directory for RamDirectory {
     fn watch(&self, watch_callback: WatchCallback) -> crate::Result<WatchHandle> {
         Ok(self.fs.write().unwrap().watch(watch_callback))
     }
+
+    fn sync_directory(&self) -> io::Result<()> {
+        Ok(())
+    }
 }
 
 #[cfg(test)]
 mod tests {
-    use super::RamDirectory;
-    use crate::Directory;
     use std::io::Write;
     use std::path::Path;
 
+    use super::RamDirectory;
+    use crate::Directory;
+
     #[test]
     fn test_persist() {
         let msg_atomic: &'static [u8] = b"atomic is the way";
