@@ -1,6 +1,3 @@
-use super::*;
-use futures::channel::oneshot;
-use futures::executor::block_on;
 use std::io::Write;
 use std::mem;
 use std::path::{Path, PathBuf};
@@ -9,6 +6,11 @@ use std::sync::atomic::{AtomicBool, AtomicUsize};
 use std::sync::Arc;
 use std::time::Duration;
 
+use futures::channel::oneshot;
+use futures::executor::block_on;
+
+use super::*;
+
 #[cfg(feature = "mmap")]
 mod mmap_directory_tests {
     use crate::directory::MmapDirectory;
@@ -118,15 +120,6 @@ mod ram_directory_tests {
     }
 }
 
-#[test]
-#[should_panic]
-fn ram_directory_panics_if_flush_forgotten() {
-    let test_path: &'static Path = Path::new("some_path_for_test");
-    let ram_directory = RamDirectory::create();
-    let mut write_file = ram_directory.open_write(test_path).unwrap();
-    assert!(write_file.write_all(&[4]).is_ok());
-}
-
 fn test_simple(directory: &dyn Directory) -> crate::Result<()> {
     let test_path: &'static Path = Path::new("some_path_for_test");
     let mut write_file = directory.open_write(test_path)?;
