@@ -1,53 +1,41 @@
-/*!
-Column oriented field storage for tantivy.
-
-It is the equivalent of `Lucene`'s `DocValues`.
-
-Fast fields is a column-oriented fashion storage of `tantivy`.
-
-It is designed for the fast random access of some document
-fields given a document id.
-
-`FastField` are useful when a field is required for all or most of
-the `DocSet` : for instance for scoring, grouping, filtering, or faceting.
-
-
-Fields have to be declared as `FAST` in the  schema.
-Currently only 64-bits integers (signed or unsigned) are
-supported.
-
-They are stored in a bit-packed fashion so that their
-memory usage is directly linear with the amplitude of the
-values stored.
-
-Read access performance is comparable to that of an array lookup.
-*/
+//! Column oriented field storage for tantivy.
+//!
+//! It is the equivalent of `Lucene`'s `DocValues`.
+//!
+//! A fast field is a column-oriented fashion storage for `tantivy`.
+//!
+//! It is designed for the fast random access of some document
+//! fields given a document id.
+//!
+//! `FastField` are useful when a field is required for all or most of
+//! the `DocSet` : for instance for scoring, grouping, filtering, or faceting.
+//!
+//!
+//! Fields have to be declared as `FAST` in the  schema.
+//! Currently supported fields are: u64, i64, f64 and bytes.
+//!
+//! u64, i64 and f64 fields are stored in a bit-packed fashion so that
+//! their memory usage is directly linear with the amplitude of the
+//! values stored.
+//!
+//! Read access performance is comparable to that of an array lookup.
 
+pub use self::alive_bitset::{intersect_alive_bitsets, write_alive_bitset, AliveBitSet};
 pub use self::bytes::{BytesFastFieldReader, BytesFastFieldWriter};
-pub use self::delete::write_delete_bitset;
-pub use self::delete::DeleteBitSet;
 pub use self::error::{FastFieldNotAvailableError, Result};
 pub use self::facet_reader::FacetReader;
 pub use self::multivalued::{MultiValuedFastFieldReader, MultiValuedFastFieldWriter};
-pub(crate) use self::reader::BitpackedFastFieldReader;
-pub use self::reader::DynamicFastFieldReader;
-pub use self::reader::FastFieldReader;
+pub use self::reader::{DynamicFastFieldReader, FastFieldReader};
 pub use self::readers::FastFieldReaders;
-pub use self::serializer::CompositeFastFieldSerializer;
-pub use self::serializer::FastFieldDataAccess;
-pub use self::serializer::FastFieldStats;
+pub(crate) use self::readers::{type_and_cardinality, FastType};
+pub use self::serializer::{CompositeFastFieldSerializer, FastFieldDataAccess, FastFieldStats};
 pub use self::writer::{FastFieldsWriter, IntFastFieldWriter};
-use crate::schema::Cardinality;
-use crate::schema::FieldType;
-use crate::schema::Value;
+use crate::chrono::{NaiveDateTime, Utc};
+use crate::schema::{Cardinality, FieldType, Type, Value};
 use crate::DocId;
-use crate::{
-    chrono::{NaiveDateTime, Utc},
-    schema::Type,
-};
 
+mod alive_bitset;
 mod bytes;
-mod delete;
 mod error;
 mod facet_reader;
 mod multivalued;
@@ -109,7 +97,7 @@ impl FastValue for u64 {
     fn fast_field_cardinality(field_type: &FieldType) -> Option<Cardinality> {
         match *field_type {
             FieldType::U64(ref integer_options) => integer_options.get_fastfield_cardinality(),
-            FieldType::HierarchicalFacet(_) => Some(Cardinality::MultiValues),
+            FieldType::Facet(_) => Some(Cardinality::MultiValues),
             _ => None,
         }
     }
@@ -212,22 +200,20 @@ fn value_to_u64(value: &Value) -> u64 {
 #[cfg(test)]
 mod tests {
 
-    use super::*;
-    use crate::directory::CompositeFile;
-    use crate::directory::{Directory, RamDirectory, WritePtr};
-    use crate::merge_policy::NoMergePolicy;
-    use crate::schema::Field;
-    use crate::schema::Schema;
-    use crate::schema::FAST;
-    use crate::schema::{Document, IntOptions};
-    use crate::{Index, SegmentId, SegmentReader};
+    use std::collections::HashMap;
+    use std::path::Path;
+
     use common::HasLen;
     use once_cell::sync::Lazy;
     use rand::prelude::SliceRandom;
     use rand::rngs::StdRng;
     use rand::SeedableRng;
-    use std::collections::HashMap;
-    use std::path::Path;
+
+    use super::*;
+    use crate::directory::{CompositeFile, Directory, RamDirectory, WritePtr};
+    use crate::merge_policy::NoMergePolicy;
+    use crate::schema::{Document, Field, NumericOptions, Schema, FAST};
+    use crate::{Index, SegmentId, SegmentReader};
 
     pub static SCHEMA: Lazy<Schema> = Lazy::new(|| {
         let mut schema_builder = Schema::builder();
@@ -406,7 +392,7 @@ mod tests {
             serializer.close().unwrap();
         }
         let file = directory.open_read(path).unwrap();
-        //assert_eq!(file.len(), 17710 as usize); //bitpacked size
+        // assert_eq!(file.len(), 17710 as usize); //bitpacked size
         assert_eq!(file.len(), 10175_usize); // linear interpol size
         {
             let fast_fields_composite = CompositeFile::open(&file)?;
@@ -496,18 +482,18 @@ mod tests {
     }
 
     #[test]
-    fn test_merge_missing_date_fast_field() {
+    fn test_merge_missing_date_fast_field() -> crate::Result<()> {
         let mut schema_builder = Schema::builder();
         let date_field = schema_builder.add_date_field("date", FAST);
         let schema = schema_builder.build();
         let index = Index::create_in_ram(schema);
         let mut index_writer = index.writer_for_tests().unwrap();
         index_writer.set_merge_policy(Box::new(NoMergePolicy));
-        index_writer.add_document(doc!(date_field =>crate::chrono::prelude::Utc::now()));
-        index_writer.commit().unwrap();
-        index_writer.add_document(doc!());
-        index_writer.commit().unwrap();
-        let reader = index.reader().unwrap();
+        index_writer.add_document(doc!(date_field =>crate::chrono::prelude::Utc::now()))?;
+        index_writer.commit()?;
+        index_writer.add_document(doc!())?;
+        index_writer.commit()?;
+        let reader = index.reader()?;
         let segment_ids: Vec<SegmentId> = reader
             .searcher()
             .segment_readers()
@@ -516,10 +502,10 @@ mod tests {
             .collect();
         assert_eq!(segment_ids.len(), 2);
         let merge_future = index_writer.merge(&segment_ids[..]);
-        let merge_res = futures::executor::block_on(merge_future);
-        assert!(merge_res.is_ok());
-        assert!(reader.reload().is_ok());
+        futures::executor::block_on(merge_future)?;
+        reader.reload()?;
         assert_eq!(reader.searcher().segment_readers().len(), 1);
+        Ok(())
     }
 
     #[test]
@@ -528,32 +514,32 @@ mod tests {
     }
 
     #[test]
-    fn test_datefastfield() {
+    fn test_datefastfield() -> crate::Result<()> {
         use crate::fastfield::FastValue;
         let mut schema_builder = Schema::builder();
         let date_field = schema_builder.add_date_field("date", FAST);
         let multi_date_field = schema_builder.add_date_field(
             "multi_date",
-            IntOptions::default().set_fast(Cardinality::MultiValues),
+            NumericOptions::default().set_fast(Cardinality::MultiValues),
         );
         let schema = schema_builder.build();
         let index = Index::create_in_ram(schema);
-        let mut index_writer = index.writer_for_tests().unwrap();
+        let mut index_writer = index.writer_for_tests()?;
         index_writer.set_merge_policy(Box::new(NoMergePolicy));
         index_writer.add_document(doc!(
             date_field => crate::DateTime::from_u64(1i64.to_u64()),
             multi_date_field => crate::DateTime::from_u64(2i64.to_u64()),
             multi_date_field => crate::DateTime::from_u64(3i64.to_u64())
-        ));
+        ))?;
         index_writer.add_document(doc!(
             date_field => crate::DateTime::from_u64(4i64.to_u64())
-        ));
+        ))?;
         index_writer.add_document(doc!(
             multi_date_field => crate::DateTime::from_u64(5i64.to_u64()),
             multi_date_field => crate::DateTime::from_u64(6i64.to_u64())
-        ));
-        index_writer.commit().unwrap();
-        let reader = index.reader().unwrap();
+        ))?;
+        index_writer.commit()?;
+        let reader = index.reader()?;
         let searcher = reader.searcher();
         assert_eq!(searcher.segment_readers().len(), 1);
         let segment_reader = searcher.segment_reader(0);
@@ -580,21 +566,22 @@ mod tests {
             assert_eq!(dates[0].timestamp(), 5i64);
             assert_eq!(dates[1].timestamp(), 6i64);
         }
+        Ok(())
     }
 }
 
 #[cfg(all(test, feature = "unstable"))]
 mod bench {
-    use super::tests::FIELD;
-    use super::tests::{generate_permutation, SCHEMA};
-    use super::*;
-    use crate::directory::CompositeFile;
-    use crate::directory::{Directory, RamDirectory, WritePtr};
-    use crate::fastfield::FastFieldReader;
     use std::collections::HashMap;
     use std::path::Path;
+
     use test::{self, Bencher};
 
+    use super::tests::{generate_permutation, FIELD, SCHEMA};
+    use super::*;
+    use crate::directory::{CompositeFile, Directory, RamDirectory, WritePtr};
+    use crate::fastfield::FastFieldReader;
+
     #[bench]
     fn bench_intfastfield_linear_veclookup(b: &mut Bencher) {
         let permutation = generate_permutation();
