@@ -1,14 +1,12 @@
-use crate::directory::CompositeFile;
-use crate::directory::FileSlice;
-use crate::fastfield::MultiValuedFastFieldReader;
-use crate::fastfield::{BitpackedFastFieldReader, FastFieldNotAvailableError};
-use crate::fastfield::{BytesFastFieldReader, FastValue};
+use super::reader::DynamicFastFieldReader;
+use crate::directory::{CompositeFile, FileSlice};
+use crate::fastfield::{
+    BytesFastFieldReader, FastFieldNotAvailableError, FastValue, MultiValuedFastFieldReader,
+};
 use crate::schema::{Cardinality, Field, FieldType, Schema};
 use crate::space_usage::PerFieldSpaceUsage;
 use crate::TantivyError;
 
-use super::reader::DynamicFastFieldReader;
-
 /// Provides access to all of the BitpackedFastFieldReader.
 ///
 /// Internally, `FastFieldReaders` have preloaded fast field readers,
@@ -19,14 +17,14 @@ pub struct FastFieldReaders {
     fast_fields_composite: CompositeFile,
 }
 #[derive(Eq, PartialEq, Debug)]
-enum FastType {
+pub(crate) enum FastType {
     I64,
     U64,
     F64,
     Date,
 }
 
-fn type_and_cardinality(field_type: &FieldType) -> Option<(FastType, Cardinality)> {
+pub(crate) fn type_and_cardinality(field_type: &FieldType) -> Option<(FastType, Cardinality)> {
     match field_type {
         FieldType::U64(options) => options
             .get_fastfield_cardinality()
@@ -40,7 +38,7 @@ fn type_and_cardinality(field_type: &FieldType) -> Option<(FastType, Cardinality
         FieldType::Date(options) => options
             .get_fastfield_cardinality()
             .map(|cardinality| (FastType::Date, cardinality)),
-        FieldType::HierarchicalFacet(_) => Some((FastType::U64, Cardinality::MultiValues)),
+        FieldType::Facet(_) => Some((FastType::U64, Cardinality::MultiValues)),
         _ => None,
     }
 }
@@ -57,7 +55,8 @@ impl FastFieldReaders {
         self.fast_fields_composite.space_usage()
     }
 
-    fn fast_field_data(&self, field: Field, idx: usize) -> crate::Result<FileSlice> {
+    #[doc(hidden)]
+    pub fn fast_field_data(&self, field: Field, idx: usize) -> crate::Result<FileSlice> {
         self.fast_fields_composite
             .open_read_with_idx(field, idx)
             .ok_or_else(|| {
@@ -131,10 +130,11 @@ impl FastFieldReaders {
         self.typed_fast_field_reader(field)
     }
 
-    /// Returns the `u64` fast field reader reader associated to `field`, regardless of whether the given
-    /// field is effectively of type `u64` or not.
+    /// Returns the `u64` fast field reader reader associated to `field`, regardless of whether the
+    /// given field is effectively of type `u64` or not.
     ///
-    /// If not, the fastfield reader will returns the u64-value associated to the original FastValue.
+    /// If not, the fastfield reader will returns the u64-value associated to the original
+    /// FastValue.
     pub fn u64_lenient(&self, field: Field) -> crate::Result<DynamicFastFieldReader<u64>> {
         self.typed_fast_field_reader(field)
     }
@@ -171,8 +171,8 @@ impl FastFieldReaders {
         self.typed_fast_field_multi_reader(field)
     }
 
-    /// Returns a `u64s` multi-valued fast field reader reader associated to `field`, regardless of whether the given
-    /// field is effectively of type `u64` or not.
+    /// Returns a `u64s` multi-valued fast field reader reader associated to `field`, regardless of
+    /// whether the given field is effectively of type `u64` or not.
     ///
     /// If `field` is not a u64 multi-valued fast field, this method returns an Error.
     pub fn u64s_lenient(&self, field: Field) -> crate::Result<MultiValuedFastFieldReader<u64>> {
@@ -219,7 +219,7 @@ impl FastFieldReaders {
                 )));
             }
             let fast_field_idx_file = self.fast_field_data(field, 0)?;
-            let idx_reader = BitpackedFastFieldReader::open(fast_field_idx_file)?;
+            let idx_reader = DynamicFastFieldReader::open(fast_field_idx_file)?;
             let data = self.fast_field_data(field, 1)?;
             BytesFastFieldReader::open(idx_reader, data)
         } else {
