@@ -1,14 +1,10 @@
-use crate::schema;
-use crate::Index;
-use crate::IndexSettings;
-use crate::IndexSortByField;
-use crate::Order;
-use crate::Searcher;
-use crate::{doc, schema::*};
-use rand::thread_rng;
-use rand::Rng;
 use std::collections::HashSet;
 
+use rand::{thread_rng, Rng};
+
+use crate::schema::*;
+use crate::{doc, schema, Index, IndexSettings, IndexSortByField, Order, Searcher};
+
 fn check_index_content(searcher: &Searcher, vals: &[u64]) -> crate::Result<()> {
     assert!(searcher.segment_readers().len() < 20);
     assert_eq!(searcher.num_docs() as usize, vals.len());
@@ -39,8 +35,7 @@ fn test_functional_store() -> crate::Result<()> {
     let mut doc_set: Vec<u64> = Vec::new();
 
     let mut doc_id = 0u64;
-    for iteration in 0..get_num_iterations() {
-        dbg!(iteration);
+    for _iteration in 0..get_num_iterations() {
         let num_docs: usize = rng.gen_range(0..4);
         if !doc_set.is_empty() {
             let doc_to_remove_id = rng.gen_range(0..doc_set.len());
@@ -49,7 +44,7 @@ fn test_functional_store() -> crate::Result<()> {
         }
         for _ in 0..num_docs {
             doc_set.push(doc_id);
-            index_writer.add_document(doc!(id_field=>doc_id));
+            index_writer.add_document(doc!(id_field=>doc_id))?;
             doc_id += 1;
         }
         index_writer.commit()?;
@@ -124,20 +119,18 @@ fn test_functional_indexing_sorted() -> crate::Result<()> {
                 doc.add_u64(multiples_field, random_val * i);
             }
             doc.add_text(text_field, get_text());
-            index_writer.add_document(doc);
+            index_writer.add_document(doc)?;
         }
     }
     Ok(())
 }
 
-const LOREM: &str = "Doc Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed \
-             do eiusmod tempor incididunt ut labore et dolore magna aliqua. \
-             Ut enim ad minim veniam, quis nostrud exercitation ullamco \
-             laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure \
-             dolor in reprehenderit in voluptate velit esse cillum dolore eu \
-             fugiat nulla pariatur. Excepteur sint occaecat cupidatat non \
-             proident, sunt in culpa qui officia deserunt mollit anim id est \
-             laborum.";
+const LOREM: &str = "Doc Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod \
+                     tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, \
+                     quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo \
+                     consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse \
+                     cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat \
+                     non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.";
 fn get_text() -> String {
     use rand::seq::SliceRandom;
     let mut rng = thread_rng();
@@ -201,7 +194,7 @@ fn test_functional_indexing_unsorted() -> crate::Result<()> {
                 doc.add_u64(multiples_field, random_val * i);
             }
             doc.add_text(text_field, get_text());
-            index_writer.add_document(doc);
+            index_writer.add_document(doc)?;
         }
     }
     Ok(())
