@@ -1,33 +1,25 @@
 #[cfg(test)]
 mod tests {
-    use crate::fastfield::{DeleteBitSet, FastFieldReader};
-    use crate::schema::IndexRecordOption;
-    use crate::{
-        collector::TopDocs,
-        schema::{Cardinality, TextFieldIndexing},
-    };
-    use crate::{core::Index, fastfield::MultiValuedFastFieldReader};
-    use crate::{
-        query::QueryParser,
-        schema::{IntOptions, TextOptions},
-    };
-    use crate::{schema::Facet, IndexSortByField};
-    use crate::{schema::INDEXED, Order};
-    use crate::{
-        schema::{self, BytesOptions},
-        DocAddress,
-    };
-    use crate::{DocSet, IndexSettings, Postings, Term};
     use futures::executor::block_on;
 
+    use crate::collector::TopDocs;
+    use crate::core::Index;
+    use crate::fastfield::{AliveBitSet, FastFieldReader, MultiValuedFastFieldReader};
+    use crate::query::QueryParser;
+    use crate::schema::{
+        self, BytesOptions, Cardinality, Facet, FacetOptions, IndexRecordOption, NumericOptions,
+        TextFieldIndexing, TextOptions,
+    };
+    use crate::{DocAddress, DocSet, IndexSettings, IndexSortByField, Order, Postings, Term};
+
     fn create_test_index_posting_list_issue(index_settings: Option<IndexSettings>) -> Index {
         let mut schema_builder = schema::Schema::builder();
-        let int_options = IntOptions::default()
+        let int_options = NumericOptions::default()
             .set_fast(Cardinality::SingleValue)
             .set_indexed();
         let int_field = schema_builder.add_u64_field("intval", int_options);
 
-        let facet_field = schema_builder.add_facet_field("facet", INDEXED);
+        let facet_field = schema_builder.add_facet_field("facet", FacetOptions::default());
 
         let schema = schema_builder.build();
 
@@ -39,14 +31,17 @@ mod tests {
 
         {
             let mut index_writer = index.writer_for_tests().unwrap();
-
-            index_writer.add_document(doc!(int_field=>3_u64, facet_field=> Facet::from("/crime")));
-            index_writer.add_document(doc!(int_field=>6_u64, facet_field=> Facet::from("/crime")));
-
-            assert!(index_writer.commit().is_ok());
-            index_writer.add_document(doc!(int_field=>5_u64, facet_field=> Facet::from("/fanta")));
-
-            assert!(index_writer.commit().is_ok());
+            index_writer
+                .add_document(doc!(int_field=>3_u64, facet_field=> Facet::from("/crime")))
+                .unwrap();
+            index_writer
+                .add_document(doc!(int_field=>6_u64, facet_field=> Facet::from("/crime")))
+                .unwrap();
+            index_writer.commit().unwrap();
+            index_writer
+                .add_document(doc!(int_field=>5_u64, facet_field=> Facet::from("/fanta")))
+                .unwrap();
+            index_writer.commit().unwrap();
         }
 
         // Merging the segments
@@ -61,14 +56,14 @@ mod tests {
         index
     }
 
-    // force_disjunct_segment_sort_values forces the field, by which the index is sorted have disjunct
-    // ranges between segments, e.g. values in segment [1-3] [10 - 20] [50 - 500]
+    // force_disjunct_segment_sort_values forces the field, by which the index is sorted have
+    // disjunct ranges between segments, e.g. values in segment [1-3] [10 - 20] [50 - 500]
     fn create_test_index(
         index_settings: Option<IndexSettings>,
         force_disjunct_segment_sort_values: bool,
-    ) -> Index {
+    ) -> crate::Result<Index> {
         let mut schema_builder = schema::Schema::builder();
-        let int_options = IntOptions::default()
+        let int_options = NumericOptions::default()
             .set_fast(Cardinality::SingleValue)
             .set_stored()
             .set_indexed();
@@ -76,11 +71,11 @@ mod tests {
 
         let bytes_options = BytesOptions::default().set_fast().set_indexed();
         let bytes_field = schema_builder.add_bytes_field("bytes", bytes_options);
-        let facet_field = schema_builder.add_facet_field("facet", INDEXED);
+        let facet_field = schema_builder.add_facet_field("facet", FacetOptions::default());
 
         let multi_numbers = schema_builder.add_u64_field(
             "multi_numbers",
-            IntOptions::default().set_fast(Cardinality::MultiValues),
+            NumericOptions::default().set_fast(Cardinality::MultiValues),
         );
         let text_field_options = TextOptions::default()
             .set_indexing_options(
@@ -95,34 +90,34 @@ mod tests {
         if let Some(settings) = index_settings {
             index_builder = index_builder.settings(settings);
         }
-        let index = index_builder.create_in_ram().unwrap();
+        let index = index_builder.create_in_ram()?;
 
         {
-            let mut index_writer = index.writer_for_tests().unwrap();
+            let mut index_writer = index.writer_for_tests()?;
 
             // segment 1 - range 1-3
-            index_writer.add_document(doc!(int_field=>1_u64));
+            index_writer.add_document(doc!(int_field=>1_u64))?;
             index_writer.add_document(
                 doc!(int_field=>3_u64, multi_numbers => 3_u64, multi_numbers => 4_u64, bytes_field => vec![1, 2, 3], text_field => "some text", facet_field=> Facet::from("/book/crime")),
-            );
+            )?;
             index_writer.add_document(
                 doc!(int_field=>1_u64, text_field=> "deleteme",  text_field => "ok text more text"),
-            );
+            )?;
             index_writer.add_document(
                 doc!(int_field=>2_u64, multi_numbers => 2_u64, multi_numbers => 3_u64, text_field => "ok text more text"),
-            );
+            )?;
 
-            assert!(index_writer.commit().is_ok());
+            index_writer.commit()?;
             // segment 2 - range 1-20 , with force_disjunct_segment_sort_values 10-20
-            index_writer.add_document(doc!(int_field=>20_u64, multi_numbers => 20_u64));
+            index_writer.add_document(doc!(int_field=>20_u64, multi_numbers => 20_u64))?;
 
             let in_val = if force_disjunct_segment_sort_values {
                 10_u64
             } else {
                 1
             };
-            index_writer.add_document(doc!(int_field=>in_val, text_field=> "deleteme" , text_field => "ok text more text", facet_field=> Facet::from("/book/crime")));
-            assert!(index_writer.commit().is_ok());
+            index_writer.add_document(doc!(int_field=>in_val, text_field=> "deleteme" , text_field => "ok text more text", facet_field=> Facet::from("/book/crime")))?;
+            index_writer.commit()?;
             // segment 3 - range 5-1000, with force_disjunct_segment_sort_values 50-1000
             let int_vals = if force_disjunct_segment_sort_values {
                 [100_u64, 50]
@@ -131,26 +126,24 @@ mod tests {
             };
             index_writer.add_document( // position of this doc after delete in desc sorting = [2], in disjunct case [1]
                 doc!(int_field=>int_vals[0], multi_numbers => 10_u64, multi_numbers => 11_u64, text_field=> "blubber", facet_field=> Facet::from("/book/fantasy")),
-            );
-            index_writer.add_document(doc!(int_field=>int_vals[1], text_field=> "deleteme"));
+            )?;
+            index_writer.add_document(doc!(int_field=>int_vals[1], text_field=> "deleteme"))?;
             index_writer.add_document(
                 doc!(int_field=>1_000u64, multi_numbers => 1001_u64, multi_numbers => 1002_u64, bytes_field => vec![5, 5],text_field => "the biggest num")
-            );
+            )?;
 
             index_writer.delete_term(Term::from_field_text(text_field, "deleteme"));
-            assert!(index_writer.commit().is_ok());
+            index_writer.commit()?;
         }
 
         // Merging the segments
         {
-            let segment_ids = index
-                .searchable_segment_ids()
-                .expect("Searchable segments failed.");
-            let mut index_writer = index.writer_for_tests().unwrap();
-            assert!(block_on(index_writer.merge(&segment_ids)).is_ok());
-            assert!(index_writer.wait_merging_threads().is_ok());
+            let segment_ids = index.searchable_segment_ids()?;
+            let mut index_writer = index.writer_for_tests()?;
+            block_on(index_writer.merge(&segment_ids))?;
+            index_writer.wait_merging_threads()?;
         }
-        index
+        Ok(index)
     }
 
     #[test]
@@ -183,7 +176,8 @@ mod tests {
                 ..Default::default()
             }),
             force_disjunct_segment_sort_values,
-        );
+        )
+        .unwrap();
 
         let int_field = index.schema().get_field("intval").unwrap();
         let reader = index.reader().unwrap();
@@ -257,10 +251,10 @@ mod tests {
                 .unwrap();
 
             assert_eq!(postings.doc_freq(), 2);
-            let fallback_bitset = DeleteBitSet::for_test(&[0], 100);
+            let fallback_bitset = AliveBitSet::for_test_from_deleted_docs(&[0], 100);
             assert_eq!(
                 postings.doc_freq_given_deletes(
-                    segment_reader.delete_bitset().unwrap_or(&fallback_bitset)
+                    segment_reader.alive_bitset().unwrap_or(&fallback_bitset)
                 ),
                 2
             );
@@ -285,11 +279,11 @@ mod tests {
             };
             let doc = searcher.doc(DocAddress::new(0, blubber_pos)).unwrap();
             assert_eq!(
-                doc.get_first(my_text_field).unwrap().text(),
+                doc.get_first(my_text_field).unwrap().as_text(),
                 Some("blubber")
             );
             let doc = searcher.doc(DocAddress::new(0, 0)).unwrap();
-            assert_eq!(doc.get_first(int_field).unwrap().u64_value(), Some(1000));
+            assert_eq!(doc.get_first(int_field).unwrap().as_u64(), Some(1000));
         }
     }
 
@@ -300,7 +294,8 @@ mod tests {
                 ..Default::default()
             }),
             false,
-        );
+        )
+        .unwrap();
 
         let reader = index.reader().unwrap();
         let searcher = reader.searcher();
@@ -336,10 +331,10 @@ mod tests {
                 .unwrap()
                 .unwrap();
             assert_eq!(postings.doc_freq(), 2);
-            let fallback_bitset = DeleteBitSet::for_test(&[0], 100);
+            let fallback_bitset = AliveBitSet::for_test_from_deleted_docs(&[0], 100);
             assert_eq!(
                 postings.doc_freq_given_deletes(
-                    segment_reader.delete_bitset().unwrap_or(&fallback_bitset)
+                    segment_reader.alive_bitset().unwrap_or(&fallback_bitset)
                 ),
                 2
             );
@@ -367,7 +362,8 @@ mod tests {
                 ..Default::default()
             }),
             false,
-        );
+        )
+        .unwrap();
 
         let int_field = index.schema().get_field("intval").unwrap();
         let multi_numbers = index.schema().get_field("multi_numbers").unwrap();
@@ -446,10 +442,10 @@ mod tests {
                 .unwrap();
 
             assert_eq!(postings.doc_freq(), 2);
-            let fallback_bitset = DeleteBitSet::for_test(&[0], 100);
+            let fallback_bitset = AliveBitSet::for_test_from_deleted_docs(&[0], 100);
             assert_eq!(
                 postings.doc_freq_given_deletes(
-                    segment_reader.delete_bitset().unwrap_or(&fallback_bitset)
+                    segment_reader.alive_bitset().unwrap_or(&fallback_bitset)
                 ),
                 2
             );
@@ -466,17 +462,17 @@ mod tests {
         // access doc store
         {
             let doc = searcher.doc(DocAddress::new(0, 0)).unwrap();
-            assert_eq!(doc.get_first(int_field).unwrap().u64_value(), Some(1));
+            assert_eq!(doc.get_first(int_field).unwrap().as_u64(), Some(1));
             let doc = searcher.doc(DocAddress::new(0, 1)).unwrap();
-            assert_eq!(doc.get_first(int_field).unwrap().u64_value(), Some(2));
+            assert_eq!(doc.get_first(int_field).unwrap().as_u64(), Some(2));
             let doc = searcher.doc(DocAddress::new(0, 2)).unwrap();
-            assert_eq!(doc.get_first(int_field).unwrap().u64_value(), Some(3));
+            assert_eq!(doc.get_first(int_field).unwrap().as_u64(), Some(3));
             let doc = searcher.doc(DocAddress::new(0, 3)).unwrap();
-            assert_eq!(doc.get_first(int_field).unwrap().u64_value(), Some(10));
+            assert_eq!(doc.get_first(int_field).unwrap().as_u64(), Some(10));
             let doc = searcher.doc(DocAddress::new(0, 4)).unwrap();
-            assert_eq!(doc.get_first(int_field).unwrap().u64_value(), Some(20));
+            assert_eq!(doc.get_first(int_field).unwrap().as_u64(), Some(20));
             let doc = searcher.doc(DocAddress::new(0, 5)).unwrap();
-            assert_eq!(doc.get_first(int_field).unwrap().u64_value(), Some(1_000));
+            assert_eq!(doc.get_first(int_field).unwrap().as_u64(), Some(1_000));
         }
     }
 }
@@ -484,23 +480,17 @@ mod tests {
 #[cfg(all(test, feature = "unstable"))]
 mod bench_sorted_index_merge {
 
-    use crate::core::Index;
-    //use cratedoc_id, readerdoc_id_mappinglet vals = reader.fate::schema;
-    use crate::fastfield::DynamicFastFieldReader;
-    use crate::fastfield::FastFieldReader;
-    use crate::indexer::merger::IndexMerger;
-    use crate::schema::Cardinality;
-    use crate::schema::Document;
-    use crate::schema::IntOptions;
-    use crate::schema::Schema;
-    use crate::IndexSettings;
-    use crate::IndexSortByField;
-    use crate::IndexWriter;
-    use crate::Order;
     use test::{self, Bencher};
+
+    use crate::core::Index;
+    // use cratedoc_id, readerdoc_id_mappinglet vals = reader.fate::schema;
+    use crate::fastfield::{DynamicFastFieldReader, FastFieldReader};
+    use crate::indexer::merger::IndexMerger;
+    use crate::schema::{Cardinality, Document, NumericOptions, Schema};
+    use crate::{IndexSettings, IndexSortByField, IndexWriter, Order};
     fn create_index(sort_by_field: Option<IndexSortByField>) -> Index {
         let mut schema_builder = Schema::builder();
-        let int_options = IntOptions::default()
+        let int_options = NumericOptions::default()
             .set_fast(Cardinality::SingleValue)
             .set_indexed();
         let int_field = schema_builder.add_u64_field("intval", int_options);
@@ -517,7 +507,7 @@ mod bench_sorted_index_merge {
             let index_doc = |index_writer: &mut IndexWriter, val: u64| {
                 let mut doc = Document::default();
                 doc.add_u64(int_field, val);
-                index_writer.add_document(doc);
+                index_writer.add_document(doc).unwrap();
             };
             // 3 segments with 10_000 values in the fast fields
             for _ in 0..3 {
@@ -545,34 +535,34 @@ mod bench_sorted_index_merge {
             IndexMerger::open(index.schema(), index.settings().clone(), &segments[..])?;
         let doc_id_mapping = merger.generate_doc_id_mapping(&sort_by_field).unwrap();
         b.iter(|| {
-
-            let sorted_doc_ids = doc_id_mapping.iter().map(|(doc_id, reader)|{
-            let u64_reader: DynamicFastFieldReader<u64> = reader.reader
-                .fast_fields()
-                .typed_fast_field_reader(field)
-                .expect("Failed to find a reader for single fast field. This is a tantivy bug and it should never happen.");
+            let sorted_doc_ids = doc_id_mapping.iter().map(|(doc_id, ordinal)| {
+                let reader = &merger.readers[*ordinal as usize];
+                let u64_reader: DynamicFastFieldReader<u64> =
+                    reader.fast_fields().typed_fast_field_reader(field).expect(
+                        "Failed to find a reader for single fast field. This is a tantivy bug and \
+                         it should never happen.",
+                    );
                 (doc_id, reader, u64_reader)
             });
-            // add values in order of the new docids
+            // add values in order of the new doc_ids
             let mut val = 0;
             for (doc_id, _reader, field_reader) in sorted_doc_ids {
                 val = field_reader.get(*doc_id);
             }
 
             val
-
         });
 
         Ok(())
     }
     #[bench]
-    fn create_sorted_index_create_docid_mapping(b: &mut Bencher) -> crate::Result<()> {
+    fn create_sorted_index_create_doc_id_mapping(b: &mut Bencher) -> crate::Result<()> {
         let sort_by_field = IndexSortByField {
             field: "intval".to_string(),
             order: Order::Desc,
         };
         let index = create_index(Some(sort_by_field.clone()));
-        //let field = index.schema().get_field("intval").unwrap();
+        // let field = index.schema().get_field("intval").unwrap();
         let segments = index.searchable_segments().unwrap();
         let merger: IndexMerger =
             IndexMerger::open(index.schema(), index.settings().clone(), &segments[..])?;
