@@ -1,11 +1,9 @@
-use crate::core::SegmentId;
-use crate::core::SegmentMeta;
+use std::collections::{HashMap, HashSet};
+use std::fmt::{self, Debug, Display, Formatter};
+
+use crate::core::{SegmentId, SegmentMeta};
 use crate::indexer::delete_queue::DeleteCursor;
 use crate::indexer::segment_entry::SegmentEntry;
-use std::collections::HashMap;
-use std::collections::HashSet;
-use std::fmt::Display;
-use std::fmt::{self, Debug, Formatter};
 
 /// The segment register keeps track
 /// of the list of segment, their size as well
@@ -66,13 +64,10 @@ impl SegmentRegister {
     }
 
     pub fn segment_metas(&self) -> Vec<SegmentMeta> {
-        let mut segment_ids: Vec<SegmentMeta> = self
-            .segment_states
+        self.segment_states
             .values()
             .map(|segment_entry| segment_entry.meta().clone())
-            .collect();
-        segment_ids.sort_by_key(SegmentMeta::id);
-        segment_ids
+            .collect()
     }
 
     pub fn contains_all(&self, segment_ids: &[SegmentId]) -> bool {
