@@ -10,6 +10,7 @@
 )]
 #![doc(test(attr(allow(unused_variables), deny(warnings))))]
 #![warn(missing_docs)]
+#![allow(clippy::len_without_is_empty)]
 
 //! # `tantivy`
 //!
@@ -62,7 +63,7 @@
 //!     body => "He was an old man who fished alone in a skiff in \
 //!             the Gulf Stream and he had gone eighty-four days \
 //!             now without taking a fish."
-//! ));
+//! ))?;
 //!
 //! // We need to call .commit() explicitly to force the
 //! // index_writer to finish processing the documents in the queue,
@@ -103,7 +104,7 @@
 //! A good place for you to get started is to check out
 //! the example code (
 //! [literate programming](https://tantivy-search.github.io/examples/basic_search.html) /
-//! [source code](https://github.com/tantivy-search/tantivy/blob/main/examples/basic_search.rs))
+//! [source code](https://github.com/quickwit-oss/tantivy/blob/main/examples/basic_search.rs))
 
 #[cfg_attr(test, macro_use)]
 extern crate serde_json;
@@ -123,15 +124,20 @@ mod functional_test;
 #[macro_use]
 mod macros;
 
-pub use crate::error::TantivyError;
 pub use chrono;
 
+pub use crate::error::TantivyError;
+
 /// Tantivy result.
 ///
 /// Within tantivy, please avoid importing `Result` using `use crate::Result`
 /// and instead, refer to this as `crate::Result<T>`.
 pub type Result<T> = std::result::Result<T, TantivyError>;
 
+/// Result for an Async io operation.
+#[cfg(feature = "quickwit")]
+pub type AsyncIoResult<T> = std::result::Result<T, crate::error::AsyncIoError>;
+
 /// Tantivy DateTime
 pub type DateTime = chrono::DateTime<chrono::Utc>;
 
@@ -142,6 +148,7 @@ mod indexer;
 pub mod error;
 pub mod tokenizer;
 
+pub mod aggregation;
 pub mod collector;
 pub mod directory;
 pub mod fastfield;
@@ -156,31 +163,31 @@ pub mod termdict;
 
 mod reader;
 
-pub use self::reader::{IndexReader, IndexReaderBuilder, ReloadPolicy};
+pub use self::reader::{IndexReader, IndexReaderBuilder, ReloadPolicy, Warmer};
 mod snippet;
 pub use self::snippet::{Snippet, SnippetGenerator};
 
 mod docset;
+use std::fmt;
+
+pub use census::{Inventory, TrackedObject};
+pub use common::{f64_to_u64, i64_to_u64, u64_to_f64, u64_to_i64, HasLen};
+use once_cell::sync::Lazy;
+use serde::{Deserialize, Serialize};
+
 pub use self::docset::{DocSet, TERMINATED};
-pub use crate::core::{Executor, SegmentComponent};
 pub use crate::core::{
-    Index, IndexBuilder, IndexMeta, IndexSettings, IndexSortByField, Order, Searcher, Segment,
-    SegmentId, SegmentMeta,
+    Executor, Index, IndexBuilder, IndexMeta, IndexSettings, IndexSortByField, InvertedIndexReader,
+    Order, Searcher, SearcherGeneration, Segment, SegmentComponent, SegmentId, SegmentMeta,
+    SegmentReader,
 };
-pub use crate::core::{InvertedIndexReader, SegmentReader};
 pub use crate::directory::Directory;
-pub use crate::indexer::merge_segments;
+pub use crate::indexer::demuxer::*;
 pub use crate::indexer::operation::UserOperation;
-pub use crate::indexer::IndexWriter;
+pub use crate::indexer::{merge_filtered_segments, merge_indices, IndexWriter, PreparedCommit};
 pub use crate::postings::Postings;
 pub use crate::reader::LeasedItem;
 pub use crate::schema::{Document, Term};
-pub use common::HasLen;
-pub use common::{f64_to_u64, i64_to_u64, u64_to_f64, u64_to_i64};
-use std::fmt;
-
-use once_cell::sync::Lazy;
-use serde::{Deserialize, Serialize};
 
 /// Index format version.
 const INDEX_FORMAT_VERSION: u32 = 4;
@@ -232,10 +239,9 @@ pub fn version_string() -> &'static str {
 
 /// Defines tantivy's merging strategy
 pub mod merge_policy {
-    pub use crate::indexer::DefaultMergePolicy;
-    pub use crate::indexer::LogMergePolicy;
-    pub use crate::indexer::MergePolicy;
-    pub use crate::indexer::NoMergePolicy;
+    pub use crate::indexer::{
+        DefaultMergePolicy, LogMergePolicy, MergeCandidate, MergePolicy, NoMergePolicy,
+    };
 }
 
 /// A `u32` identifying a document within a segment.
@@ -293,21 +299,18 @@ pub struct DocAddress {
 
 #[cfg(test)]
 pub mod tests {
+    use common::{BinarySerializable, FixedSize};
+    use rand::distributions::{Bernoulli, Uniform};
+    use rand::rngs::StdRng;
+    use rand::{Rng, SeedableRng};
+
     use crate::collector::tests::TEST_COLLECTOR_WITH_SCORE;
     use crate::core::SegmentReader;
     use crate::docset::{DocSet, TERMINATED};
     use crate::fastfield::FastFieldReader;
     use crate::query::BooleanQuery;
     use crate::schema::*;
-    use crate::DocAddress;
-    use crate::Index;
-    use crate::Postings;
-    use crate::ReloadPolicy;
-    use common::{BinarySerializable, FixedSize};
-    use rand::distributions::Bernoulli;
-    use rand::distributions::Uniform;
-    use rand::rngs::StdRng;
-    use rand::{Rng, SeedableRng};
+    use crate::{DocAddress, Index, Postings, ReloadPolicy};
 
     pub fn fixed_size_test<O: BinarySerializable + FixedSize + Default>() {
         let mut buffer = Vec::new();
@@ -376,24 +379,22 @@ pub mod tests {
         let mut schema_builder = Schema::builder();
         let text_field = schema_builder.add_text_field("text", TEXT);
         let schema = schema_builder.build();
-        let index = Index::create_from_tempdir(schema).unwrap();
+        let index = Index::create_from_tempdir(schema)?;
+        // writing the segment
+        let mut index_writer = index.writer_for_tests()?;
         {
-            // writing the segment
-            let mut index_writer = index.writer_for_tests()?;
-            {
-                let doc = doc!(text_field=>"af b");
-                index_writer.add_document(doc);
-            }
-            {
-                let doc = doc!(text_field=>"a b c");
-                index_writer.add_document(doc);
-            }
-            {
-                let doc = doc!(text_field=>"a b c d");
-                index_writer.add_document(doc);
-            }
-            assert!(index_writer.commit().is_ok());
+            let doc = doc!(text_field=>"af b");
+            index_writer.add_document(doc)?;
         }
+        {
+            let doc = doc!(text_field=>"a b c");
+            index_writer.add_document(doc)?;
+        }
+        {
+            let doc = doc!(text_field=>"a b c d");
+            index_writer.add_document(doc)?;
+        }
+        index_writer.commit()?;
         Ok(())
     }
 
@@ -403,12 +404,12 @@ pub mod tests {
         let text_field = schema_builder.add_text_field("text", TEXT);
         let index = Index::create_in_ram(schema_builder.build());
         let mut index_writer = index.writer_for_tests()?;
-        index_writer.add_document(doc!(text_field=>"a b c"));
+        index_writer.add_document(doc!(text_field=>"a b c"))?;
         index_writer.commit()?;
-        index_writer.add_document(doc!(text_field=>"a"));
-        index_writer.add_document(doc!(text_field=>"a a"));
+        index_writer.add_document(doc!(text_field=>"a"))?;
+        index_writer.add_document(doc!(text_field=>"a a"))?;
         index_writer.commit()?;
-        index_writer.add_document(doc!(text_field=>"c"));
+        index_writer.add_document(doc!(text_field=>"c"))?;
         index_writer.commit()?;
         let reader = index.reader()?;
         let searcher = reader.searcher();
@@ -430,7 +431,7 @@ pub mod tests {
         let text_field = schema_builder.add_text_field("text", TEXT);
         let index = Index::create_in_ram(schema_builder.build());
         let mut index_writer = index.writer_for_tests()?;
-        index_writer.add_document(doc!(text_field=>"a b c"));
+        index_writer.add_document(doc!(text_field=>"a b c"))?;
         index_writer.commit()?;
         let index_reader = index.reader()?;
         let searcher = index_reader.searcher();
@@ -452,9 +453,9 @@ pub mod tests {
         let text_field = schema_builder.add_text_field("text", TEXT);
         let index = Index::create_in_ram(schema_builder.build());
         let mut index_writer = index.writer_for_tests()?;
-        index_writer.add_document(doc!(text_field=>"a b c"));
-        index_writer.add_document(doc!());
-        index_writer.add_document(doc!(text_field=>"a b"));
+        index_writer.add_document(doc!(text_field=>"a b c"))?;
+        index_writer.add_document(doc!())?;
+        index_writer.add_document(doc!(text_field=>"a b"))?;
         index_writer.commit()?;
         let reader = index.reader()?;
         let searcher = reader.searcher();
@@ -496,20 +497,20 @@ pub mod tests {
             // writing the segment
             let mut index_writer = index.writer_for_tests()?;
             // 0
-            index_writer.add_document(doc!(text_field=>"a b"));
+            index_writer.add_document(doc!(text_field=>"a b"))?;
             // 1
-            index_writer.add_document(doc!(text_field=>" a c"));
+            index_writer.add_document(doc!(text_field=>" a c"))?;
             // 2
-            index_writer.add_document(doc!(text_field=>" b c"));
+            index_writer.add_document(doc!(text_field=>" b c"))?;
             // 3
-            index_writer.add_document(doc!(text_field=>" b d"));
+            index_writer.add_document(doc!(text_field=>" b d"))?;
 
             index_writer.delete_term(Term::from_field_text(text_field, "c"));
             index_writer.delete_term(Term::from_field_text(text_field, "a"));
             // 4
-            index_writer.add_document(doc!(text_field=>" b c"));
+            index_writer.add_document(doc!(text_field=>" b c"))?;
             // 5
-            index_writer.add_document(doc!(text_field=>" a"));
+            index_writer.add_document(doc!(text_field=>" a"))?;
             index_writer.commit()?;
         }
         {
@@ -543,7 +544,7 @@ pub mod tests {
             // writing the segment
             let mut index_writer = index.writer_for_tests()?;
             // 0
-            index_writer.add_document(doc!(text_field=>"a b"));
+            index_writer.add_document(doc!(text_field=>"a b"))?;
             // 1
             index_writer.delete_term(Term::from_field_text(text_field, "c"));
             index_writer.rollback()?;
@@ -579,7 +580,7 @@ pub mod tests {
         {
             // writing the segment
             let mut index_writer = index.writer_for_tests()?;
-            index_writer.add_document(doc!(text_field=>"a b"));
+            index_writer.add_document(doc!(text_field=>"a b"))?;
             index_writer.delete_term(Term::from_field_text(text_field, "c"));
             index_writer.rollback()?;
             index_writer.delete_term(Term::from_field_text(text_field, "a"));
@@ -629,7 +630,7 @@ pub mod tests {
 
         let index = Index::create_in_ram(schema);
         let mut index_writer = index.writer_for_tests()?;
-        index_writer.add_document(doc!(field=>1u64));
+        index_writer.add_document(doc!(field=>1u64))?;
         index_writer.commit()?;
         let reader = index.reader()?;
         let searcher = reader.searcher();
@@ -653,7 +654,7 @@ pub mod tests {
         let index = Index::create_in_ram(schema);
         let mut index_writer = index.writer_for_tests()?;
         let negative_val = -1i64;
-        index_writer.add_document(doc!(value_field => negative_val));
+        index_writer.add_document(doc!(value_field => negative_val))?;
         index_writer.commit()?;
         let reader = index.reader()?;
         let searcher = reader.searcher();
@@ -677,7 +678,7 @@ pub mod tests {
         let index = Index::create_in_ram(schema);
         let mut index_writer = index.writer_for_tests()?;
         let val = std::f64::consts::PI;
-        index_writer.add_document(doc!(value_field => val));
+        index_writer.add_document(doc!(value_field => val))?;
         index_writer.commit()?;
         let reader = index.reader()?;
         let searcher = reader.searcher();
@@ -700,7 +701,7 @@ pub mod tests {
         let schema = schema_builder.build();
         let index = Index::create_in_ram(schema);
         let mut index_writer = index.writer_for_tests()?;
-        index_writer.add_document(doc!(text_field=>"a"));
+        index_writer.add_document(doc!(text_field=>"a"))?;
         assert!(index_writer.commit().is_ok());
         let reader = index.reader()?;
         let searcher = reader.searcher();
@@ -723,14 +724,14 @@ pub mod tests {
 
         // writing the segment
         let mut index_writer = index.writer_for_tests()?;
-        index_writer.add_document(doc!(text_field=>"63"));
-        index_writer.add_document(doc!(text_field=>"70"));
-        index_writer.add_document(doc!(text_field=>"34"));
-        index_writer.add_document(doc!(text_field=>"1"));
-        index_writer.add_document(doc!(text_field=>"38"));
-        index_writer.add_document(doc!(text_field=>"33"));
-        index_writer.add_document(doc!(text_field=>"40"));
-        index_writer.add_document(doc!(text_field=>"17"));
+        index_writer.add_document(doc!(text_field=>"63"))?;
+        index_writer.add_document(doc!(text_field=>"70"))?;
+        index_writer.add_document(doc!(text_field=>"34"))?;
+        index_writer.add_document(doc!(text_field=>"1"))?;
+        index_writer.add_document(doc!(text_field=>"38"))?;
+        index_writer.add_document(doc!(text_field=>"33"))?;
+        index_writer.add_document(doc!(text_field=>"40"))?;
+        index_writer.add_document(doc!(text_field=>"17"))?;
         index_writer.delete_term(Term::from_field_text(text_field, "38"));
         index_writer.delete_term(Term::from_field_text(text_field, "34"));
         index_writer.commit()?;
@@ -748,7 +749,7 @@ pub mod tests {
         {
             // writing the segment
             let mut index_writer = index.writer_for_tests()?;
-            index_writer.add_document(doc!(text_field=>"af af af bc bc"));
+            index_writer.add_document(doc!(text_field=>"af af af bc bc"))?;
             index_writer.commit()?;
         }
         {
@@ -780,9 +781,9 @@ pub mod tests {
         let reader = index.reader()?;
         // writing the segment
         let mut index_writer = index.writer_for_tests()?;
-        index_writer.add_document(doc!(text_field=>"af af af b"));
-        index_writer.add_document(doc!(text_field=>"a b c"));
-        index_writer.add_document(doc!(text_field=>"a b c d"));
+        index_writer.add_document(doc!(text_field=>"af af af b"))?;
+        index_writer.add_document(doc!(text_field=>"a b c"))?;
+        index_writer.add_document(doc!(text_field=>"a b c d"))?;
         index_writer.commit()?;
 
         reader.reload()?;
@@ -844,9 +845,9 @@ pub mod tests {
         assert_eq!(reader.searcher().num_docs(), 0u64);
         // writing the segment
         let mut index_writer = index.writer_for_tests()?;
-        index_writer.add_document(doc!(text_field=>"af b"));
-        index_writer.add_document(doc!(text_field=>"a b c"));
-        index_writer.add_document(doc!(text_field=>"a b c d"));
+        index_writer.add_document(doc!(text_field=>"af b"))?;
+        index_writer.add_document(doc!(text_field=>"a b c"))?;
+        index_writer.add_document(doc!(text_field=>"a b c d"))?;
         index_writer.commit()?;
         reader.reload()?;
         assert_eq!(reader.searcher().num_docs(), 3u64);
@@ -864,11 +865,11 @@ pub mod tests {
         assert_eq!(document.len(), 3);
         let values: Vec<&Value> = document.get_all(text_field).collect();
         assert_eq!(values.len(), 2);
-        assert_eq!(values[0].text(), Some("tantivy"));
-        assert_eq!(values[1].text(), Some("some other value"));
+        assert_eq!(values[0].as_text(), Some("tantivy"));
+        assert_eq!(values[1].as_text(), Some("some other value"));
         let values: Vec<&Value> = document.get_all(other_text_field).collect();
         assert_eq!(values.len(), 1);
-        assert_eq!(values[0].text(), Some("short"));
+        assert_eq!(values[0].as_text(), Some("short"));
     }
 
     #[test]
@@ -886,7 +887,7 @@ pub mod tests {
         {
             let document =
                 doc!(fast_field_unsigned => 4u64, fast_field_signed=>4i64, fast_field_float=>4f64);
-            index_writer.add_document(document);
+            index_writer.add_document(document)?;
             index_writer.commit()?;
         }
         let reader = index.reader()?;
@@ -934,11 +935,12 @@ pub mod tests {
     // motivated by #729
     #[test]
     fn test_update_via_delete_insert() -> crate::Result<()> {
+        use futures::executor::block_on;
+
         use crate::collector::Count;
         use crate::indexer::NoMergePolicy;
         use crate::query::AllQuery;
         use crate::SegmentId;
-        use futures::executor::block_on;
 
         const DOC_COUNT: u64 = 2u64;
 
@@ -953,7 +955,7 @@ pub mod tests {
         index_writer.set_merge_policy(Box::new(NoMergePolicy));
 
         for doc_id in 0u64..DOC_COUNT {
-            index_writer.add_document(doc!(id => doc_id));
+            index_writer.add_document(doc!(id => doc_id))?;
         }
         index_writer.commit()?;
 
@@ -970,7 +972,7 @@ pub mod tests {
             index_writer.delete_term(Term::from_field_u64(id, doc_id));
             index_writer.commit()?;
             index_reader.reload()?;
-            index_writer.add_document(doc!(id =>  doc_id));
+            index_writer.add_document(doc!(id =>  doc_id))?;
             index_writer.commit()?;
             index_reader.reload()?;
             let searcher = index_reader.searcher();
@@ -1005,8 +1007,8 @@ pub mod tests {
         let index = Index::create_in_dir(&index_path, schema)?;
         let mut writer = index.writer(50_000_000)?;
         for _ in 0..5000 {
-            writer.add_document(doc!(body => "foo"));
-            writer.add_document(doc!(body => "boo"));
+            writer.add_document(doc!(body => "foo"))?;
+            writer.add_document(doc!(body => "boo"))?;
         }
         writer.commit()?;
         assert!(index.validate_checksum()?.is_empty());
