@@ -1,6 +1,4 @@
-/*!
-Postings module (also called inverted index)
-*/
+//! Postings module (also called inverted index)
 
 mod block_search;
 
@@ -8,6 +6,9 @@ pub(crate) use self::block_search::branchless_binary_search;
 
 mod block_segment_postings;
 pub(crate) mod compression;
+mod indexing_context;
+mod json_postings_writer;
+mod per_field_postings_writer;
 mod postings;
 mod postings_writer;
 mod recorder;
@@ -18,8 +19,10 @@ mod stacker;
 mod term_info;
 
 pub use self::block_segment_postings::BlockSegmentPostings;
+pub(crate) use self::indexing_context::IndexingContext;
+pub(crate) use self::per_field_postings_writer::PerFieldPostingsWriter;
 pub use self::postings::Postings;
-pub(crate) use self::postings_writer::MultiFieldPostingsWriter;
+pub(crate) use self::postings_writer::{serialize_postings, IndexingPosition, PostingsWriter};
 pub use self::segment_postings::SegmentPostings;
 pub use self::serializer::{FieldSerializer, InvertedIndexSerializer};
 pub(crate) use self::skip::{BlockInfo, SkipReader};
@@ -38,25 +41,20 @@ pub(crate) enum FreqReadingOption {
 
 #[cfg(test)]
 pub mod tests {
-    use super::InvertedIndexSerializer;
-    use super::Postings;
-    use crate::core::Index;
-    use crate::core::SegmentComponent;
-    use crate::core::SegmentReader;
+    use std::mem;
+
+    use super::{InvertedIndexSerializer, Postings};
+    use crate::core::{Index, SegmentComponent, SegmentReader};
     use crate::docset::{DocSet, TERMINATED};
     use crate::fieldnorm::FieldNormReader;
     use crate::indexer::operation::AddOperation;
     use crate::indexer::SegmentWriter;
-    use crate::merge_policy::NoMergePolicy;
     use crate::query::Scorer;
-    use crate::schema::{Field, TextOptions};
-    use crate::schema::{IndexRecordOption, TextFieldIndexing};
-    use crate::schema::{Schema, Term, INDEXED, TEXT};
+    use crate::schema::{
+        Field, IndexRecordOption, Schema, Term, TextFieldIndexing, TextOptions, INDEXED, TEXT,
+    };
     use crate::tokenizer::{SimpleTokenizer, MAX_TOKEN_LEN};
-    use crate::DocId;
-    use crate::HasLen;
-    use crate::Score;
-    use std::mem;
+    use crate::{DocId, HasLen, Score};
 
     #[test]
     pub fn test_position_write() -> crate::Result<()> {
@@ -87,12 +85,12 @@ pub mod tests {
         let schema = schema_builder.build();
         let index = Index::create_in_ram(schema);
         let mut index_writer = index.writer_for_tests()?;
-        index_writer.add_document(doc!(title => r#"abc abc abc"#));
-        index_writer.add_document(doc!(title => r#"abc be be be be abc"#));
+        index_writer.add_document(doc!(title => r#"abc abc abc"#))?;
+        index_writer.add_document(doc!(title => r#"abc be be be be abc"#))?;
         for _ in 0..1_000 {
-            index_writer.add_document(doc!(title => r#"abc abc abc"#));
+            index_writer.add_document(doc!(title => r#"abc abc abc"#))?;
         }
-        index_writer.add_document(doc!(title => r#"abc be be be be abc"#));
+        index_writer.add_document(doc!(title => r#"abc be be be be abc"#))?;
         index_writer.commit()?;
 
         let searcher = index.reader()?.searcher();
@@ -154,10 +152,7 @@ pub mod tests {
     }
 
     #[test]
-    pub fn test_drop_token_that_are_too_long() -> crate::Result<()> {
-        let ok_token_text: String = "A".repeat(MAX_TOKEN_LEN);
-        let mut exceeding_token_text: String = "A".repeat(MAX_TOKEN_LEN + 1);
-        exceeding_token_text.push_str(" hello");
+    pub fn test_index_max_length_token() -> crate::Result<()> {
         let mut schema_builder = Schema::builder();
         let text_options = TextOptions::default().set_indexing_options(
             TextFieldIndexing::default()
@@ -170,33 +165,54 @@ pub mod tests {
         index
             .tokenizers()
             .register("simple_no_truncation", SimpleTokenizer);
-        let reader = index.reader().unwrap();
-        let mut index_writer = index.writer_for_tests().unwrap();
-        index_writer.set_merge_policy(Box::new(NoMergePolicy));
-        {
-            index_writer.add_document(doc!(text_field=>exceeding_token_text));
-            index_writer.commit().unwrap();
-            reader.reload().unwrap();
-            let searcher = reader.searcher();
-            let segment_reader = searcher.segment_reader(0u32);
-            let inverted_index = segment_reader.inverted_index(text_field)?;
-            assert_eq!(inverted_index.terms().num_terms(), 1);
-            let mut bytes = vec![];
-            assert!(inverted_index.terms().ord_to_term(0, &mut bytes)?);
-            assert_eq!(&bytes, b"hello");
-        }
-        {
-            index_writer.add_document(doc!(text_field=>ok_token_text.clone()));
-            index_writer.commit().unwrap();
-            reader.reload().unwrap();
-            let searcher = reader.searcher();
-            let segment_reader = searcher.segment_reader(1u32);
-            let inverted_index = segment_reader.inverted_index(text_field)?;
-            assert_eq!(inverted_index.terms().num_terms(), 1);
-            let mut bytes = vec![];
-            assert!(inverted_index.terms().ord_to_term(0, &mut bytes)?);
-            assert_eq!(&bytes[..], ok_token_text.as_bytes());
-        }
+        let reader = index.reader()?;
+        let mut index_writer = index.writer_for_tests()?;
+
+        let ok_token_text: String = "A".repeat(MAX_TOKEN_LEN);
+        index_writer.add_document(doc!(text_field=>ok_token_text.clone()))?;
+        index_writer.commit()?;
+        reader.reload()?;
+        let searcher = reader.searcher();
+        let segment_reader = searcher.segment_reader(0u32);
+        let inverted_index = segment_reader.inverted_index(text_field)?;
+        assert_eq!(inverted_index.terms().num_terms(), 1);
+        let mut bytes = vec![];
+        assert!(inverted_index.terms().ord_to_term(0, &mut bytes)?);
+        assert_eq!(&bytes[..], ok_token_text.as_bytes());
+
+        Ok(())
+    }
+
+    #[test]
+    pub fn test_drop_token_that_are_too_long() -> crate::Result<()> {
+        let mut schema_builder = Schema::builder();
+        let text_options = TextOptions::default().set_indexing_options(
+            TextFieldIndexing::default()
+                .set_index_option(IndexRecordOption::WithFreqsAndPositions)
+                .set_tokenizer("simple_no_truncation"),
+        );
+        let text_field = schema_builder.add_text_field("text", text_options);
+        let schema = schema_builder.build();
+        let index = Index::create_in_ram(schema);
+        index
+            .tokenizers()
+            .register("simple_no_truncation", SimpleTokenizer);
+        let reader = index.reader()?;
+        let mut index_writer = index.writer_for_tests()?;
+
+        let mut exceeding_token_text: String = "A".repeat(MAX_TOKEN_LEN + 1);
+        exceeding_token_text.push_str(" hello");
+        index_writer.add_document(doc!(text_field=>exceeding_token_text))?;
+        index_writer.commit()?;
+        reader.reload()?;
+        let searcher = reader.searcher();
+        let segment_reader = searcher.segment_reader(0u32);
+        let inverted_index = segment_reader.inverted_index(text_field)?;
+        assert_eq!(inverted_index.terms().num_terms(), 1);
+        let mut bytes = vec![];
+        assert!(inverted_index.terms().ord_to_term(0, &mut bytes)?);
+        assert_eq!(&bytes, b"hello");
+
         Ok(())
     }
 
@@ -211,7 +227,7 @@ pub mod tests {
 
         {
             let mut segment_writer =
-                SegmentWriter::for_segment(3_000_000, segment.clone(), &schema).unwrap();
+                SegmentWriter::for_segment(3_000_000, segment.clone(), schema).unwrap();
             {
                 // checking that position works if the field has two values
                 let op = AddOperation {
@@ -221,14 +237,14 @@ pub mod tests {
                        text_field => "d d d d a"
                     ),
                 };
-                segment_writer.add_document(op, &schema)?;
+                segment_writer.add_document(op)?;
             }
             {
                 let op = AddOperation {
                     opstamp: 1u64,
                     document: doc!(text_field => "b a"),
                 };
-                segment_writer.add_document(op, &schema).unwrap();
+                segment_writer.add_document(op).unwrap();
             }
             for i in 2..1000 {
                 let mut text: String = "e ".repeat(i);
@@ -237,7 +253,7 @@ pub mod tests {
                     opstamp: 2u64,
                     document: doc!(text_field => text),
                 };
-                segment_writer.add_document(op, &schema).unwrap();
+                segment_writer.add_document(op).unwrap();
             }
             segment_writer.finalize()?;
         }
@@ -315,13 +331,13 @@ pub mod tests {
         let schema = schema_builder.build();
         let index = Index::create_in_ram(schema);
         {
-            let mut index_writer = index.writer_for_tests().unwrap();
-            index_writer.add_document(doc!(text_field => "g b b d c g c"));
-            index_writer.add_document(doc!(text_field => "g a b b a d c g c"));
-            assert!(index_writer.commit().is_ok());
+            let mut index_writer = index.writer_for_tests()?;
+            index_writer.add_document(doc!(text_field => "g b b d c g c"))?;
+            index_writer.add_document(doc!(text_field => "g a b b a d c g c"))?;
+            index_writer.commit()?;
         }
         let term_a = Term::from_field_text(text_field, "a");
-        let searcher = index.reader().unwrap().searcher();
+        let searcher = index.reader()?.searcher();
         let segment_reader = searcher.segment_reader(0);
         let mut postings = segment_reader
             .inverted_index(text_field)?
@@ -350,7 +366,7 @@ pub mod tests {
                 let mut index_writer = index.writer_for_tests()?;
                 for i in 0u64..num_docs as u64 {
                     let doc = doc!(value_field => 2u64, value_field => i % 2u64);
-                    index_writer.add_document(doc);
+                    index_writer.add_document(doc)?;
                 }
                 assert!(index_writer.commit().is_ok());
             }
@@ -548,18 +564,16 @@ pub mod tests {
 
 #[cfg(all(test, feature = "unstable"))]
 mod bench {
-    use crate::docset::TERMINATED;
-    use crate::query::Intersection;
-    use crate::schema::IndexRecordOption;
-    use crate::schema::{Document, Field, Schema, Term, STRING};
-    use crate::tests;
-    use crate::DocSet;
-    use crate::Index;
     use once_cell::sync::Lazy;
     use rand::rngs::StdRng;
     use rand::{Rng, SeedableRng};
     use test::{self, Bencher};
 
+    use crate::docset::TERMINATED;
+    use crate::query::Intersection;
+    use crate::schema::{Document, Field, IndexRecordOption, Schema, Term, STRING};
+    use crate::{tests, DocSet, Index};
+
     pub static TERM_A: Lazy<Term> = Lazy::new(|| {
         let field = Field::from_field_id(0);
         Term::from_field_text(field, "a")
@@ -600,7 +614,7 @@ mod bench {
                     doc.add_text(text_field, "c");
                 }
                 doc.add_text(text_field, "d");
-                index_writer.add_document(doc);
+                index_writer.add_document(doc).unwrap();
             }
             assert!(index_writer.commit().is_ok());
         }
