@@ -1,8 +1,9 @@
+use common::read_u32_vint;
+
 use super::stacker::{ExpUnrolledLinkedList, MemoryArena};
 use crate::indexer::doc_id_mapping::DocIdMapping;
 use crate::postings::FieldSerializer;
 use crate::DocId;
-use common::{read_u32_vint, write_u32_vint};
 
 const POSITION_END: u32 = 0;
 
@@ -55,26 +56,24 @@ impl<'a> Iterator for VInt32Reader<'a> {
 ///   * the document id
 ///   * the term frequency
 ///   * the term positions
-pub(crate) trait Recorder: Copy + 'static {
-    ///
-    fn new() -> Self;
+pub(crate) trait Recorder: Copy + Default + 'static {
     /// Returns the current document
     fn current_doc(&self) -> u32;
     /// Starts recording information about a new document
     /// This method shall only be called if the term is within the document.
-    fn new_doc(&mut self, doc: DocId, heap: &mut MemoryArena);
+    fn new_doc(&mut self, doc: DocId, arena: &mut MemoryArena);
     /// Record the position of a term. For each document,
     /// this method will be called `term_freq` times.
-    fn record_position(&mut self, position: u32, heap: &mut MemoryArena);
+    fn record_position(&mut self, position: u32, arena: &mut MemoryArena);
     /// Close the document. It will help record the term frequency.
-    fn close_doc(&mut self, heap: &mut MemoryArena);
+    fn close_doc(&mut self, arena: &mut MemoryArena);
     /// Pushes the postings information to the serializer.
     fn serialize(
         &self,
-        buffer_lender: &mut BufferLender,
-        serializer: &mut FieldSerializer<'_>,
-        heap: &MemoryArena,
+        arena: &MemoryArena,
         doc_id_map: Option<&DocIdMapping>,
+        serializer: &mut FieldSerializer<'_>,
+        buffer_lender: &mut BufferLender,
     );
     /// Returns the number of document containing this term.
     ///
@@ -89,37 +88,39 @@ pub struct NothingRecorder {
     current_doc: DocId,
 }
 
-impl Recorder for NothingRecorder {
-    fn new() -> Self {
+impl Default for NothingRecorder {
+    fn default() -> Self {
         NothingRecorder {
             stack: ExpUnrolledLinkedList::new(),
             current_doc: u32::max_value(),
         }
     }
+}
 
+impl Recorder for NothingRecorder {
     fn current_doc(&self) -> DocId {
         self.current_doc
     }
 
-    fn new_doc(&mut self, doc: DocId, heap: &mut MemoryArena) {
+    fn new_doc(&mut self, doc: DocId, arena: &mut MemoryArena) {
         self.current_doc = doc;
-        let _ = write_u32_vint(doc, &mut self.stack.writer(heap));
+        self.stack.writer(arena).write_u32_vint(doc);
     }
 
-    fn record_position(&mut self, _position: u32, _heap: &mut MemoryArena) {}
+    fn record_position(&mut self, _position: u32, _arena: &mut MemoryArena) {}
 
-    fn close_doc(&mut self, _heap: &mut MemoryArena) {}
+    fn close_doc(&mut self, _arena: &mut MemoryArena) {}
 
     fn serialize(
         &self,
-        buffer_lender: &mut BufferLender,
-        serializer: &mut FieldSerializer<'_>,
-        heap: &MemoryArena,
+        arena: &MemoryArena,
         doc_id_map: Option<&DocIdMapping>,
+        serializer: &mut FieldSerializer<'_>,
+        buffer_lender: &mut BufferLender,
     ) {
         let (buffer, doc_ids) = buffer_lender.lend_all();
-        self.stack.read_to_end(heap, buffer);
-        //TODO avoid reading twice.
+        self.stack.read_to_end(arena, buffer);
+        // TODO avoid reading twice.
         if let Some(doc_id_map) = doc_id_map {
             doc_ids.extend(
                 VInt32Reader::new(&buffer[..])
@@ -151,8 +152,8 @@ pub struct TermFrequencyRecorder {
     term_doc_freq: u32,
 }
 
-impl Recorder for TermFrequencyRecorder {
-    fn new() -> Self {
+impl Default for TermFrequencyRecorder {
+    fn default() -> Self {
         TermFrequencyRecorder {
             stack: ExpUnrolledLinkedList::new(),
             current_doc: 0,
@@ -160,36 +161,38 @@ impl Recorder for TermFrequencyRecorder {
             term_doc_freq: 0u32,
         }
     }
+}
 
+impl Recorder for TermFrequencyRecorder {
     fn current_doc(&self) -> DocId {
         self.current_doc
     }
 
-    fn new_doc(&mut self, doc: DocId, heap: &mut MemoryArena) {
+    fn new_doc(&mut self, doc: DocId, arena: &mut MemoryArena) {
         self.term_doc_freq += 1;
         self.current_doc = doc;
-        let _ = write_u32_vint(doc, &mut self.stack.writer(heap));
+        self.stack.writer(arena).write_u32_vint(doc);
     }
 
-    fn record_position(&mut self, _position: u32, _heap: &mut MemoryArena) {
+    fn record_position(&mut self, _position: u32, _arena: &mut MemoryArena) {
         self.current_tf += 1;
     }
 
-    fn close_doc(&mut self, heap: &mut MemoryArena) {
+    fn close_doc(&mut self, arena: &mut MemoryArena) {
         debug_assert!(self.current_tf > 0);
-        let _ = write_u32_vint(self.current_tf, &mut self.stack.writer(heap));
+        self.stack.writer(arena).write_u32_vint(self.current_tf);
         self.current_tf = 0;
     }
 
     fn serialize(
         &self,
-        buffer_lender: &mut BufferLender,
-        serializer: &mut FieldSerializer<'_>,
-        heap: &MemoryArena,
+        arena: &MemoryArena,
         doc_id_map: Option<&DocIdMapping>,
+        serializer: &mut FieldSerializer<'_>,
+        buffer_lender: &mut BufferLender,
     ) {
         let buffer = buffer_lender.lend_u8();
-        self.stack.read_to_end(heap, buffer);
+        self.stack.read_to_end(arena, buffer);
         let mut u32_it = VInt32Reader::new(&buffer[..]);
         if let Some(doc_id_map) = doc_id_map {
             let mut doc_id_and_tf = vec![];
@@ -222,42 +225,47 @@ pub struct TfAndPositionRecorder {
     current_doc: DocId,
     term_doc_freq: u32,
 }
-impl Recorder for TfAndPositionRecorder {
-    fn new() -> Self {
+
+impl Default for TfAndPositionRecorder {
+    fn default() -> Self {
         TfAndPositionRecorder {
             stack: ExpUnrolledLinkedList::new(),
             current_doc: u32::max_value(),
             term_doc_freq: 0u32,
         }
     }
+}
 
+impl Recorder for TfAndPositionRecorder {
     fn current_doc(&self) -> DocId {
         self.current_doc
     }
 
-    fn new_doc(&mut self, doc: DocId, heap: &mut MemoryArena) {
+    fn new_doc(&mut self, doc: DocId, arena: &mut MemoryArena) {
         self.current_doc = doc;
         self.term_doc_freq += 1u32;
-        let _ = write_u32_vint(doc, &mut self.stack.writer(heap));
+        self.stack.writer(arena).write_u32_vint(doc);
     }
 
-    fn record_position(&mut self, position: u32, heap: &mut MemoryArena) {
-        let _ = write_u32_vint(position + 1u32, &mut self.stack.writer(heap));
+    fn record_position(&mut self, position: u32, arena: &mut MemoryArena) {
+        self.stack
+            .writer(arena)
+            .write_u32_vint(position.wrapping_add(1u32));
     }
 
-    fn close_doc(&mut self, heap: &mut MemoryArena) {
-        let _ = write_u32_vint(POSITION_END, &mut self.stack.writer(heap));
+    fn close_doc(&mut self, arena: &mut MemoryArena) {
+        self.stack.writer(arena).write_u32_vint(POSITION_END);
     }
 
     fn serialize(
         &self,
-        buffer_lender: &mut BufferLender,
-        serializer: &mut FieldSerializer<'_>,
-        heap: &MemoryArena,
+        arena: &MemoryArena,
         doc_id_map: Option<&DocIdMapping>,
+        serializer: &mut FieldSerializer<'_>,
+        buffer_lender: &mut BufferLender,
     ) {
         let (buffer_u8, buffer_positions) = buffer_lender.lend_all();
-        self.stack.read_to_end(heap, buffer_u8);
+        self.stack.read_to_end(arena, buffer_u8);
         let mut u32_it = VInt32Reader::new(&buffer_u8[..]);
         let mut doc_id_and_positions = vec![];
         while let Some(doc) = u32_it.next() {
@@ -299,9 +307,9 @@ impl Recorder for TfAndPositionRecorder {
 #[cfg(test)]
 mod tests {
 
-    use super::write_u32_vint;
-    use super::BufferLender;
-    use super::VInt32Reader;
+    use common::write_u32_vint;
+
+    use super::{BufferLender, VInt32Reader};
 
     #[test]
     fn test_buffer_lender() {
