@@ -1,13 +1,12 @@
-use crate::docset::DocSet;
-use crate::fastfield::DeleteBitSet;
-use crate::positions::PositionReader;
-use crate::postings::branchless_binary_search;
-use crate::postings::compression::COMPRESSION_BLOCK_SIZE;
-use crate::postings::BlockSegmentPostings;
-use crate::postings::Postings;
-use crate::{DocId, TERMINATED};
 use common::HasLen;
 
+use crate::docset::DocSet;
+use crate::fastfield::AliveBitSet;
+use crate::positions::PositionReader;
+use crate::postings::compression::COMPRESSION_BLOCK_SIZE;
+use crate::postings::{branchless_binary_search, BlockSegmentPostings, Postings};
+use crate::{DocId, TERMINATED};
+
 /// `SegmentPostings` represents the inverted list or postings associated to
 /// a term in a `Segment`.
 ///
@@ -34,7 +33,7 @@ impl SegmentPostings {
     ///
     /// This method will clone and scan through the posting lists.
     /// (this is a rather expensive operation).
-    pub fn doc_freq_given_deletes(&self, delete_bitset: &DeleteBitSet) -> u32 {
+    pub fn doc_freq_given_deletes(&self, alive_bitset: &AliveBitSet) -> u32 {
         let mut docset = self.clone();
         let mut doc_freq = 0;
         loop {
@@ -42,7 +41,7 @@ impl SegmentPostings {
             if doc == TERMINATED {
                 return doc_freq;
             }
-            if delete_bitset.is_alive(doc) {
+            if alive_bitset.is_alive(doc) {
                 doc_freq += 1u32;
             }
             docset.advance();
@@ -142,8 +141,7 @@ impl SegmentPostings {
     ///
     /// * `len` - number of document in the posting lists.
     /// * `data` - data array. The complete data is not necessarily used.
-    /// * `freq_handler` - the freq handler is in charge of decoding
-    ///   frequencies and/or positions
+    /// * `freq_handler` - the freq handler is in charge of decoding frequencies and/or positions
     pub(crate) fn from_block_postings(
         segment_block_postings: BlockSegmentPostings,
         position_reader: Option<PositionReader>,
@@ -234,8 +232,7 @@ impl Postings for SegmentPostings {
             // In that case we hit the block just as if the frequency had been
             // decoded. The block is simply prefilled by the value 1.
             self.cur < COMPRESSION_BLOCK_SIZE,
-            "Have you forgotten to call `.advance()` at least once before calling \
-             `.term_freq()`."
+            "Have you forgotten to call `.advance()` at least once before calling `.term_freq()`."
         );
         self.block_cursor.freq(self.cur)
     }
@@ -264,11 +261,11 @@ impl Postings for SegmentPostings {
 #[cfg(test)]
 mod tests {
 
-    use super::SegmentPostings;
     use common::HasLen;
 
+    use super::SegmentPostings;
     use crate::docset::{DocSet, TERMINATED};
-    use crate::fastfield::DeleteBitSet;
+    use crate::fastfield::AliveBitSet;
     use crate::postings::postings::Postings;
 
     #[test]
@@ -296,9 +293,10 @@ mod tests {
     fn test_doc_freq() {
         let docs = SegmentPostings::create_from_docs(&[0, 2, 10]);
         assert_eq!(docs.doc_freq(), 3);
-        let delete_bitset = DeleteBitSet::for_test(&[2], 12);
-        assert_eq!(docs.doc_freq_given_deletes(&delete_bitset), 2);
-        let all_deleted = DeleteBitSet::for_test(&[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11], 12);
+        let alive_bitset = AliveBitSet::for_test_from_deleted_docs(&[2], 12);
+        assert_eq!(docs.doc_freq_given_deletes(&alive_bitset), 2);
+        let all_deleted =
+            AliveBitSet::for_test_from_deleted_docs(&[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11], 12);
         assert_eq!(docs.doc_freq_given_deletes(&all_deleted), 0);
     }
 }
