@@ -1,16 +1,15 @@
-use crate::core::SegmentReader;
-use crate::query::ConstScorer;
-use crate::query::{BitSetDocSet, Explanation};
-use crate::query::{Scorer, Weight};
-use crate::schema::{Field, IndexRecordOption};
-use crate::termdict::{TermDictionary, TermStreamer};
-use crate::TantivyError;
-use crate::{DocId, Score};
-use common::BitSet;
 use std::io;
 use std::sync::Arc;
+
+use common::BitSet;
 use tantivy_fst::Automaton;
 
+use crate::core::SegmentReader;
+use crate::query::{BitSetDocSet, ConstScorer, Explanation, Scorer, Weight};
+use crate::schema::{Field, IndexRecordOption};
+use crate::termdict::{TermDictionary, TermStreamer};
+use crate::{DocId, Score, TantivyError};
+
 /// A weight struct for Fuzzy Term and Regex Queries
 pub struct AutomatonWeight<A> {
     field: Field,
@@ -85,23 +84,24 @@ where
 
 #[cfg(test)]
 mod tests {
+    use tantivy_fst::Automaton;
+
     use super::AutomatonWeight;
     use crate::docset::TERMINATED;
     use crate::query::Weight;
     use crate::schema::{Schema, STRING};
     use crate::Index;
-    use tantivy_fst::Automaton;
 
-    fn create_index() -> Index {
+    fn create_index() -> crate::Result<Index> {
         let mut schema = Schema::builder();
         let title = schema.add_text_field("title", STRING);
         let index = Index::create_in_ram(schema.build());
-        let mut index_writer = index.writer_for_tests().unwrap();
-        index_writer.add_document(doc!(title=>"abc"));
-        index_writer.add_document(doc!(title=>"bcd"));
-        index_writer.add_document(doc!(title=>"abcd"));
-        assert!(index_writer.commit().is_ok());
-        index
+        let mut index_writer = index.writer_for_tests()?;
+        index_writer.add_document(doc!(title=>"abc"))?;
+        index_writer.add_document(doc!(title=>"bcd"))?;
+        index_writer.add_document(doc!(title=>"abcd"))?;
+        index_writer.commit()?;
+        Ok(index)
     }
 
     #[derive(Clone, Copy)]
@@ -140,34 +140,32 @@ mod tests {
     }
 
     #[test]
-    fn test_automaton_weight() {
-        let index = create_index();
+    fn test_automaton_weight() -> crate::Result<()> {
+        let index = create_index()?;
         let field = index.schema().get_field("title").unwrap();
         let automaton_weight = AutomatonWeight::new(field, PrefixedByA);
-        let reader = index.reader().unwrap();
+        let reader = index.reader()?;
         let searcher = reader.searcher();
-        let mut scorer = automaton_weight
-            .scorer(searcher.segment_reader(0u32), 1.0)
-            .unwrap();
+        let mut scorer = automaton_weight.scorer(searcher.segment_reader(0u32), 1.0)?;
         assert_eq!(scorer.doc(), 0u32);
         assert_eq!(scorer.score(), 1.0);
         assert_eq!(scorer.advance(), 2u32);
         assert_eq!(scorer.doc(), 2u32);
         assert_eq!(scorer.score(), 1.0);
         assert_eq!(scorer.advance(), TERMINATED);
+        Ok(())
     }
 
     #[test]
-    fn test_automaton_weight_boost() {
-        let index = create_index();
+    fn test_automaton_weight_boost() -> crate::Result<()> {
+        let index = create_index()?;
         let field = index.schema().get_field("title").unwrap();
         let automaton_weight = AutomatonWeight::new(field, PrefixedByA);
-        let reader = index.reader().unwrap();
+        let reader = index.reader()?;
         let searcher = reader.searcher();
-        let mut scorer = automaton_weight
-            .scorer(searcher.segment_reader(0u32), 1.32)
-            .unwrap();
+        let mut scorer = automaton_weight.scorer(searcher.segment_reader(0u32), 1.32)?;
         assert_eq!(scorer.doc(), 0u32);
         assert_eq!(scorer.score(), 1.32);
+        Ok(())
     }
 }
