@@ -2,95 +2,92 @@ mod block_wand;
 mod boolean_query;
 mod boolean_weight;
 
-pub(crate) use self::block_wand::block_wand;
+pub(crate) use self::block_wand::{block_wand, block_wand_single_scorer};
 pub use self::boolean_query::BooleanQuery;
 
 #[cfg(test)]
 mod tests {
 
     use super::*;
-    use crate::assert_nearly_equals;
     use crate::collector::tests::TEST_COLLECTOR_WITH_SCORE;
     use crate::collector::TopDocs;
     use crate::query::score_combiner::SumWithCoordsCombiner;
     use crate::query::term_query::TermScorer;
-    use crate::query::Intersection;
-    use crate::query::Occur;
-    use crate::query::Query;
-    use crate::query::QueryParser;
-    use crate::query::RequiredOptionalScorer;
-    use crate::query::Scorer;
-    use crate::query::TermQuery;
+    use crate::query::{
+        Intersection, Occur, Query, QueryParser, RequiredOptionalScorer, Scorer, TermQuery,
+    };
     use crate::schema::*;
-    use crate::Index;
-    use crate::{DocAddress, DocId, Score};
+    use crate::{assert_nearly_equals, DocAddress, DocId, Index, Score};
 
-    fn aux_test_helper() -> (Index, Field) {
+    fn aux_test_helper() -> crate::Result<(Index, Field)> {
         let mut schema_builder = Schema::builder();
         let text_field = schema_builder.add_text_field("text", TEXT);
         let schema = schema_builder.build();
         let index = Index::create_in_ram(schema);
         {
             // writing the segment
-            let mut index_writer = index.writer_for_tests().unwrap();
-            index_writer.add_document(doc!(text_field => "a b c"));
-            index_writer.add_document(doc!(text_field => "a c"));
-            index_writer.add_document(doc!(text_field => "b c"));
-            index_writer.add_document(doc!(text_field => "a b c d"));
-            index_writer.add_document(doc!(text_field => "d"));
-            assert!(index_writer.commit().is_ok());
+            let mut index_writer = index.writer_for_tests()?;
+            index_writer.add_document(doc!(text_field => "a b c"))?;
+            index_writer.add_document(doc!(text_field => "a c"))?;
+            index_writer.add_document(doc!(text_field => "b c"))?;
+            index_writer.add_document(doc!(text_field => "a b c d"))?;
+            index_writer.add_document(doc!(text_field => "d"))?;
+            index_writer.commit()?;
         }
-        (index, text_field)
+        Ok((index, text_field))
     }
 
     #[test]
-    pub fn test_boolean_non_all_term_disjunction() {
-        let (index, text_field) = aux_test_helper();
+    pub fn test_boolean_non_all_term_disjunction() -> crate::Result<()> {
+        let (index, text_field) = aux_test_helper()?;
         let query_parser = QueryParser::for_index(&index, vec![text_field]);
-        let query = query_parser.parse_query("(+a +b) d").unwrap();
-        let searcher = index.reader().unwrap().searcher();
-        assert_eq!(query.count(&searcher).unwrap(), 3);
+        let query = query_parser.parse_query("(+a +b) d")?;
+        let searcher = index.reader()?.searcher();
+        assert_eq!(query.count(&searcher)?, 3);
+        Ok(())
     }
 
     #[test]
-    pub fn test_boolean_single_must_clause() {
-        let (index, text_field) = aux_test_helper();
+    pub fn test_boolean_single_must_clause() -> crate::Result<()> {
+        let (index, text_field) = aux_test_helper()?;
         let query_parser = QueryParser::for_index(&index, vec![text_field]);
-        let query = query_parser.parse_query("+a").unwrap();
-        let searcher = index.reader().unwrap().searcher();
-        let weight = query.weight(&searcher, true).unwrap();
-        let scorer = weight.scorer(searcher.segment_reader(0u32), 1.0).unwrap();
+        let query = query_parser.parse_query("+a")?;
+        let searcher = index.reader()?.searcher();
+        let weight = query.weight(&searcher, true)?;
+        let scorer = weight.scorer(searcher.segment_reader(0u32), 1.0)?;
         assert!(scorer.is::<TermScorer>());
+        Ok(())
     }
 
     #[test]
-    pub fn test_boolean_termonly_intersection() {
-        let (index, text_field) = aux_test_helper();
+    pub fn test_boolean_termonly_intersection() -> crate::Result<()> {
+        let (index, text_field) = aux_test_helper()?;
         let query_parser = QueryParser::for_index(&index, vec![text_field]);
-        let searcher = index.reader().unwrap().searcher();
+        let searcher = index.reader()?.searcher();
         {
-            let query = query_parser.parse_query("+a +b +c").unwrap();
-            let weight = query.weight(&searcher, true).unwrap();
-            let scorer = weight.scorer(searcher.segment_reader(0u32), 1.0).unwrap();
+            let query = query_parser.parse_query("+a +b +c")?;
+            let weight = query.weight(&searcher, true)?;
+            let scorer = weight.scorer(searcher.segment_reader(0u32), 1.0)?;
             assert!(scorer.is::<Intersection<TermScorer>>());
         }
         {
-            let query = query_parser.parse_query("+a +(b c)").unwrap();
-            let weight = query.weight(&searcher, true).unwrap();
-            let scorer = weight.scorer(searcher.segment_reader(0u32), 1.0).unwrap();
+            let query = query_parser.parse_query("+a +(b c)")?;
+            let weight = query.weight(&searcher, true)?;
+            let scorer = weight.scorer(searcher.segment_reader(0u32), 1.0)?;
             assert!(scorer.is::<Intersection<Box<dyn Scorer>>>());
         }
+        Ok(())
     }
 
     #[test]
-    pub fn test_boolean_reqopt() {
-        let (index, text_field) = aux_test_helper();
+    pub fn test_boolean_reqopt() -> crate::Result<()> {
+        let (index, text_field) = aux_test_helper()?;
         let query_parser = QueryParser::for_index(&index, vec![text_field]);
-        let searcher = index.reader().unwrap().searcher();
+        let searcher = index.reader()?.searcher();
         {
-            let query = query_parser.parse_query("+a b").unwrap();
-            let weight = query.weight(&searcher, true).unwrap();
-            let scorer = weight.scorer(searcher.segment_reader(0u32), 1.0).unwrap();
+            let query = query_parser.parse_query("+a b")?;
+            let weight = query.weight(&searcher, true)?;
+            let scorer = weight.scorer(searcher.segment_reader(0u32), 1.0)?;
             assert!(scorer.is::<RequiredOptionalScorer<
                 Box<dyn Scorer>,
                 Box<dyn Scorer>,
@@ -98,16 +95,17 @@ mod tests {
             >>());
         }
         {
-            let query = query_parser.parse_query("+a b").unwrap();
-            let weight = query.weight(&searcher, false).unwrap();
-            let scorer = weight.scorer(searcher.segment_reader(0u32), 1.0).unwrap();
+            let query = query_parser.parse_query("+a b")?;
+            let weight = query.weight(&searcher, false)?;
+            let scorer = weight.scorer(searcher.segment_reader(0u32), 1.0)?;
             assert!(scorer.is::<TermScorer>());
         }
+        Ok(())
     }
 
     #[test]
-    pub fn test_boolean_query() {
-        let (index, text_field) = aux_test_helper();
+    pub fn test_boolean_query() -> crate::Result<()> {
+        let (index, text_field) = aux_test_helper()?;
 
         let make_term_query = |text: &str| {
             let term_query = TermQuery::new(
@@ -118,7 +116,7 @@ mod tests {
             query
         };
 
-        let reader = index.reader().unwrap();
+        let reader = index.reader()?;
 
         let matching_docs = |boolean_query: &dyn Query| {
             reader
@@ -165,11 +163,12 @@ mod tests {
             let boolean_query = BooleanQuery::new(vec![(Occur::MustNot, make_term_query("d"))]);
             assert_eq!(matching_docs(&boolean_query), Vec::<u32>::new());
         }
+        Ok(())
     }
 
     #[test]
-    pub fn test_boolean_query_two_excluded() {
-        let (index, text_field) = aux_test_helper();
+    pub fn test_boolean_query_two_excluded() -> crate::Result<()> {
+        let (index, text_field) = aux_test_helper()?;
 
         let make_term_query = |text: &str| {
             let term_query = TermQuery::new(
@@ -180,7 +179,7 @@ mod tests {
             query
         };
 
-        let reader = index.reader().unwrap();
+        let reader = index.reader()?;
 
         let matching_topdocs = |query: &dyn Query| {
             reader
@@ -213,20 +212,21 @@ mod tests {
             assert_eq!(top_doc, DocAddress::new(0, 4));
             assert_eq!(top_score, score_doc_4);
         }
+        Ok(())
     }
 
     #[test]
-    pub fn test_boolean_query_with_weight() {
+    pub fn test_boolean_query_with_weight() -> crate::Result<()> {
         let mut schema_builder = Schema::builder();
         let text_field = schema_builder.add_text_field("text", TEXT);
         let schema = schema_builder.build();
         let index = Index::create_in_ram(schema);
         {
-            let mut index_writer = index.writer_for_tests().unwrap();
-            index_writer.add_document(doc!(text_field => "a b c"));
-            index_writer.add_document(doc!(text_field => "a c"));
-            index_writer.add_document(doc!(text_field => "b c"));
-            assert!(index_writer.commit().is_ok());
+            let mut index_writer = index.writer_for_tests()?;
+            index_writer.add_document(doc!(text_field => "a b c"))?;
+            index_writer.add_document(doc!(text_field => "a c"))?;
+            index_writer.add_document(doc!(text_field => "b c"))?;
+            index_writer.commit()?;
         }
         let term_a: Box<dyn Query> = Box::new(TermQuery::new(
             Term::from_field_text(text_field, "a"),
@@ -242,24 +242,21 @@ mod tests {
             BooleanQuery::new(vec![(Occur::Should, term_a), (Occur::Should, term_b)]);
         let boolean_weight = boolean_query.weight(&searcher, true).unwrap();
         {
-            let mut boolean_scorer = boolean_weight
-                .scorer(searcher.segment_reader(0u32), 1.0)
-                .unwrap();
+            let mut boolean_scorer = boolean_weight.scorer(searcher.segment_reader(0u32), 1.0)?;
             assert_eq!(boolean_scorer.doc(), 0u32);
             assert_nearly_equals!(boolean_scorer.score(), 0.84163445);
         }
         {
-            let mut boolean_scorer = boolean_weight
-                .scorer(searcher.segment_reader(0u32), 2.0)
-                .unwrap();
+            let mut boolean_scorer = boolean_weight.scorer(searcher.segment_reader(0u32), 2.0)?;
             assert_eq!(boolean_scorer.doc(), 0u32);
             assert_nearly_equals!(boolean_scorer.score(), 1.6832689);
         }
+        Ok(())
     }
 
     #[test]
-    pub fn test_intersection_score() {
-        let (index, text_field) = aux_test_helper();
+    pub fn test_intersection_score() -> crate::Result<()> {
+        let (index, text_field) = aux_test_helper()?;
 
         let make_term_query = |text: &str| {
             let term_query = TermQuery::new(
@@ -269,7 +266,7 @@ mod tests {
             let query: Box<dyn Query> = Box::new(term_query);
             query
         };
-        let reader = index.reader().unwrap();
+        let reader = index.reader()?;
         let score_docs = |boolean_query: &dyn Query| {
             let fruit = reader
                 .searcher()
@@ -287,6 +284,7 @@ mod tests {
             assert_nearly_equals!(scores[0], 0.977973);
             assert_nearly_equals!(scores[1], 0.84699446);
         }
+        Ok(())
     }
 
     #[test]
@@ -296,8 +294,8 @@ mod tests {
         let schema = schema_builder.build();
         let index = Index::create_in_ram(schema);
         let mut index_writer = index.writer_with_num_threads(1, 5_000_000)?;
-        index_writer.add_document(doc!(text=>"a"));
-        index_writer.add_document(doc!(text=>"b"));
+        index_writer.add_document(doc!(text=>"a"))?;
+        index_writer.add_document(doc!(text=>"b"))?;
         index_writer.commit()?;
         let searcher = index.reader()?.searcher();
         let term_a: Box<dyn Query> = Box::new(TermQuery::new(
