@@ -1,10 +1,7 @@
 use super::MoreLikeThis;
-
-use crate::{
-    query::{Query, Weight},
-    schema::{Field, FieldValue},
-    DocAddress, Result, Searcher,
-};
+use crate::query::{Query, Weight};
+use crate::schema::{Field, Value};
+use crate::{DocAddress, Result, Searcher};
 
 /// A query that matches all of the documents similar to a document
 /// or a set of field values provided.
@@ -24,7 +21,6 @@ use crate::{
 ///     .with_boost_factor(1.0)
 ///     .with_stop_words(vec!["for".to_string()])
 ///     .with_document(DocAddress::new(2, 1));
-///
 /// ```
 #[derive(Debug, Clone)]
 pub struct MoreLikeThisQuery {
@@ -35,7 +31,7 @@ pub struct MoreLikeThisQuery {
 #[derive(Debug, PartialEq, Clone)]
 enum TargetDocument {
     DocumentAdress(DocAddress),
-    DocumentFields(Vec<(Field, Vec<FieldValue>)>),
+    DocumentFields(Vec<(Field, Vec<Value>)>),
 }
 
 impl MoreLikeThisQuery {
@@ -61,24 +57,17 @@ impl Query for MoreLikeThisQuery {
 }
 
 /// The builder for more-like-this query
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, Default)]
 pub struct MoreLikeThisQueryBuilder {
     mlt: MoreLikeThis,
 }
 
-impl Default for MoreLikeThisQueryBuilder {
-    fn default() -> Self {
-        Self {
-            mlt: MoreLikeThis::default(),
-        }
-    }
-}
-
 impl MoreLikeThisQueryBuilder {
     /// Sets the minimum document frequency.
     ///
     /// The resulting query will ignore words which do not occur
     /// in at least this many docs.
+    #[must_use]
     pub fn with_min_doc_frequency(mut self, value: u64) -> Self {
         self.mlt.min_doc_frequency = Some(value);
         self
@@ -88,6 +77,7 @@ impl MoreLikeThisQueryBuilder {
     ///
     /// The resulting query will ignore words which occur
     /// in more than this many docs.
+    #[must_use]
     pub fn with_max_doc_frequency(mut self, value: u64) -> Self {
         self.mlt.max_doc_frequency = Some(value);
         self
@@ -97,6 +87,7 @@ impl MoreLikeThisQueryBuilder {
     ///
     /// The resulting query will ignore words less
     /// frequent that this number.
+    #[must_use]
     pub fn with_min_term_frequency(mut self, value: usize) -> Self {
         self.mlt.min_term_frequency = Some(value);
         self
@@ -105,6 +96,7 @@ impl MoreLikeThisQueryBuilder {
     /// Sets the maximum query terms.
     ///
     /// The resulting query will not return a query with more clause than this.
+    #[must_use]
     pub fn with_max_query_terms(mut self, value: usize) -> Self {
         self.mlt.max_query_terms = Some(value);
         self
@@ -113,6 +105,7 @@ impl MoreLikeThisQueryBuilder {
     /// Sets the minimum word length.
     ///
     /// The resulting query will ignore words shorter than this length.
+    #[must_use]
     pub fn with_min_word_length(mut self, value: usize) -> Self {
         self.mlt.min_word_length = Some(value);
         self
@@ -121,6 +114,7 @@ impl MoreLikeThisQueryBuilder {
     /// Sets the maximum word length.
     ///
     /// The resulting query will ignore words longer than this length.
+    #[must_use]
     pub fn with_max_word_length(mut self, value: usize) -> Self {
         self.mlt.max_word_length = Some(value);
         self
@@ -129,6 +123,7 @@ impl MoreLikeThisQueryBuilder {
     /// Sets the boost factor
     ///
     /// The boost factor used by the resulting query for boosting terms.
+    #[must_use]
     pub fn with_boost_factor(mut self, value: f32) -> Self {
         self.mlt.boost_factor = Some(value);
         self
@@ -137,6 +132,7 @@ impl MoreLikeThisQueryBuilder {
     /// Sets the set of stop words
     ///
     /// The resulting query will ignore these set of words.
+    #[must_use]
     pub fn with_stop_words(mut self, value: Vec<String>) -> Self {
         self.mlt.stop_words = value;
         self
@@ -164,10 +160,7 @@ impl MoreLikeThisQueryBuilder {
     /// that will be used to compose the resulting query.
     /// This interface is meant to be used when you want to provide your own set of fields
     /// not necessarily from a specific document.
-    pub fn with_document_fields(
-        self,
-        doc_fields: Vec<(Field, Vec<FieldValue>)>,
-    ) -> MoreLikeThisQuery {
+    pub fn with_document_fields(self, doc_fields: Vec<(Field, Vec<Value>)>) -> MoreLikeThisQuery {
         MoreLikeThisQuery {
             mlt: self.mlt,
             target: TargetDocument::DocumentFields(doc_fields),
@@ -177,27 +170,25 @@ impl MoreLikeThisQueryBuilder {
 
 #[cfg(test)]
 mod tests {
-    use super::MoreLikeThisQuery;
-    use super::TargetDocument;
+    use super::{MoreLikeThisQuery, TargetDocument};
     use crate::collector::TopDocs;
     use crate::schema::{Schema, STORED, TEXT};
-    use crate::DocAddress;
-    use crate::Index;
+    use crate::{DocAddress, Index};
 
-    fn create_test_index() -> Index {
+    fn create_test_index() -> crate::Result<Index> {
         let mut schema_builder = Schema::builder();
         let title = schema_builder.add_text_field("title", TEXT);
         let body = schema_builder.add_text_field("body", TEXT | STORED);
         let schema = schema_builder.build();
         let index = Index::create_in_ram(schema);
         let mut index_writer = index.writer_for_tests().unwrap();
-        index_writer.add_document(doc!(title => "aaa", body => "the old man and the sea"));
-        index_writer.add_document(doc!(title => "bbb", body => "an old man sailing on the sea"));
-        index_writer.add_document(doc!(title => "ccc", body=> "send this message to alice"));
-        index_writer.add_document(doc!(title => "ddd", body=> "a lady was riding and old bike"));
-        index_writer.add_document(doc!(title => "eee", body=> "Yes, my lady."));
-        index_writer.commit().unwrap();
-        index
+        index_writer.add_document(doc!(title => "aaa", body => "the old man and the sea"))?;
+        index_writer.add_document(doc!(title => "bbb", body => "an old man sailing on the sea"))?;
+        index_writer.add_document(doc!(title => "ccc", body=> "send this message to alice"))?;
+        index_writer.add_document(doc!(title => "ddd", body=> "a lady was riding and old bike"))?;
+        index_writer.add_document(doc!(title => "eee", body=> "Yes, my lady."))?;
+        index_writer.commit()?;
+        Ok(index)
     }
 
     #[test]
@@ -243,9 +234,9 @@ mod tests {
     }
 
     #[test]
-    fn test_more_like_this_query() {
-        let index = create_test_index();
-        let reader = index.reader().unwrap();
+    fn test_more_like_this_query() -> crate::Result<()> {
+        let index = create_test_index()?;
+        let reader = index.reader()?;
         let searcher = reader.searcher();
 
         // search base 1st doc with words [sea, and] skipping [old]
@@ -258,7 +249,7 @@ mod tests {
             .with_boost_factor(1.0)
             .with_stop_words(vec!["old".to_string()])
             .with_document(DocAddress::new(0, 0));
-        let top_docs = searcher.search(&query, &TopDocs::with_limit(5)).unwrap();
+        let top_docs = searcher.search(&query, &TopDocs::with_limit(5))?;
         let mut doc_ids: Vec<_> = top_docs.iter().map(|item| item.1.doc_id).collect();
         doc_ids.sort_unstable();
 
@@ -274,11 +265,12 @@ mod tests {
             .with_max_word_length(5)
             .with_boost_factor(1.0)
             .with_document(DocAddress::new(0, 4));
-        let top_docs = searcher.search(&query, &TopDocs::with_limit(5)).unwrap();
+        let top_docs = searcher.search(&query, &TopDocs::with_limit(5))?;
         let mut doc_ids: Vec<_> = top_docs.iter().map(|item| item.1.doc_id).collect();
         doc_ids.sort_unstable();
 
         assert_eq!(doc_ids.len(), 2);
         assert_eq!(doc_ids, vec![3, 4]);
+        Ok(())
     }
 }
