@@ -1,10 +1,12 @@
+use std::clone::Clone;
+use std::sync::Arc;
+
+use tantivy_fst::Regex;
+
 use crate::error::TantivyError;
 use crate::query::{AutomatonWeight, Query, Weight};
 use crate::schema::Field;
 use crate::Searcher;
-use std::clone::Clone;
-use std::sync::Arc;
-use tantivy_fst::Regex;
 
 /// A Regex Query matches all of the documents
 /// containing a specific term that matches
@@ -28,17 +30,17 @@ use tantivy_fst::Regex;
 ///     let mut index_writer = index.writer(3_000_000)?;
 ///     index_writer.add_document(doc!(
 ///         title => "The Name of the Wind",
-///     ));
+///     ))?;
 ///     index_writer.add_document(doc!(
 ///         title => "The Diary of Muadib",
-///     ));
+///     ))?;
 ///     index_writer.add_document(doc!(
 ///         title => "A Dairy Cow",
-///     ));
+///     ))?;
 ///     index_writer.add_document(doc!(
 ///         title => "The Diary of a Young Girl",
-///     ));
-///     index_writer.commit().unwrap();
+///     ))?;
+///     index_writer.commit()?;
 /// }
 ///
 /// let reader = index.reader()?;
@@ -91,16 +93,16 @@ impl Query for RegexQuery {
 
 #[cfg(test)]
 mod test {
-    use super::RegexQuery;
-    use crate::assert_nearly_equals;
-    use crate::collector::TopDocs;
-    use crate::schema::TEXT;
-    use crate::schema::{Field, Schema};
-    use crate::{Index, IndexReader};
     use std::sync::Arc;
+
     use tantivy_fst::Regex;
 
-    fn build_test_index() -> (IndexReader, Field) {
+    use super::RegexQuery;
+    use crate::collector::TopDocs;
+    use crate::schema::{Field, Schema, TEXT};
+    use crate::{assert_nearly_equals, Index, IndexReader};
+
+    fn build_test_index() -> crate::Result<(IndexReader, Field)> {
         let mut schema_builder = Schema::builder();
         let country_field = schema_builder.add_text_field("country", TEXT);
         let schema = schema_builder.build();
@@ -109,15 +111,15 @@ mod test {
             let mut index_writer = index.writer_for_tests().unwrap();
             index_writer.add_document(doc!(
                 country_field => "japan",
-            ));
+            ))?;
             index_writer.add_document(doc!(
                 country_field => "korea",
-            ));
-            index_writer.commit().unwrap();
+            ))?;
+            index_writer.commit()?;
         }
-        let reader = index.reader().unwrap();
+        let reader = index.reader()?;
 
-        (reader, country_field)
+        Ok((reader, country_field))
     }
 
     fn verify_regex_query(
@@ -141,31 +143,32 @@ mod test {
     }
 
     #[test]
-    pub fn test_regex_query() {
-        let (reader, field) = build_test_index();
-
-        let matching_one = RegexQuery::from_pattern("jap[ao]n", field).unwrap();
-        let matching_zero = RegexQuery::from_pattern("jap[A-Z]n", field).unwrap();
+    pub fn test_regex_query() -> crate::Result<()> {
+        let (reader, field) = build_test_index()?;
 
+        let matching_one = RegexQuery::from_pattern("jap[ao]n", field)?;
+        let matching_zero = RegexQuery::from_pattern("jap[A-Z]n", field)?;
         verify_regex_query(matching_one, matching_zero, reader);
+        Ok(())
     }
 
     #[test]
-    pub fn test_construct_from_regex() {
-        let (reader, field) = build_test_index();
+    pub fn test_construct_from_regex() -> crate::Result<()> {
+        let (reader, field) = build_test_index()?;
 
         let matching_one = RegexQuery::from_regex(Regex::new("jap[ao]n").unwrap(), field);
         let matching_zero = RegexQuery::from_regex(Regex::new("jap[A-Z]n").unwrap(), field);
 
         verify_regex_query(matching_one, matching_zero, reader);
+        Ok(())
     }
 
     #[test]
-    pub fn test_construct_from_reused_regex() {
+    pub fn test_construct_from_reused_regex() -> crate::Result<()> {
         let r1 = Arc::new(Regex::new("jap[ao]n").unwrap());
         let r2 = Arc::new(Regex::new("jap[A-Z]n").unwrap());
 
-        let (reader, field) = build_test_index();
+        let (reader, field) = build_test_index()?;
 
         let matching_one = RegexQuery::from_regex(r1.clone(), field);
         let matching_zero = RegexQuery::from_regex(r2.clone(), field);
@@ -176,5 +179,6 @@ mod test {
         let matching_zero = RegexQuery::from_regex(r2, field);
 
         verify_regex_query(matching_one, matching_zero, reader);
+        Ok(())
     }
 }
