@@ -1,18 +1,18 @@
+use std::collections::{HashMap, HashSet};
+use std::io::{self, Read, Write};
+use std::mem;
+
+use common::{BinarySerializable, VInt};
+
 use super::*;
 use crate::tokenizer::PreTokenizedString;
 use crate::DateTime;
-use common::BinarySerializable;
-use common::VInt;
-use std::io::{self, Read, Write};
-use std::mem;
 
 /// Tantivy's Document is the object that can
 /// be indexed and then searched for.
 ///
 /// Documents are fundamentally a collection of unordered couple `(field, value)`.
 /// In this list, one field may appear more than once.
-///
-///
 
 /// Documents are really just a list of couple `(field, value)`.
 /// In this list, one field may appear more than once.
@@ -26,20 +26,40 @@ impl From<Vec<FieldValue>> for Document {
         Document { field_values }
     }
 }
-
 impl PartialEq for Document {
     fn eq(&self, other: &Document) -> bool {
         // super slow, but only here for tests
-        let mut self_field_values: Vec<&_> = self.field_values.iter().collect();
-        let mut other_field_values: Vec<&_> = other.field_values.iter().collect();
-        self_field_values.sort();
-        other_field_values.sort();
+        let convert_to_comparable_map = |field_values: &[FieldValue]| {
+            let mut field_value_set: HashMap<Field, HashSet<String>> = Default::default();
+            for field_value in field_values.iter() {
+                let json_val = serde_json::to_string(field_value.value()).unwrap();
+                field_value_set
+                    .entry(field_value.field())
+                    .or_default()
+                    .insert(json_val);
+            }
+            field_value_set
+        };
+        let self_field_values: HashMap<Field, HashSet<String>> =
+            convert_to_comparable_map(&self.field_values);
+        let other_field_values: HashMap<Field, HashSet<String>> =
+            convert_to_comparable_map(&other.field_values);
         self_field_values.eq(&other_field_values)
     }
 }
 
 impl Eq for Document {}
 
+impl IntoIterator for Document {
+    type Item = FieldValue;
+
+    type IntoIter = std::vec::IntoIter<FieldValue>;
+
+    fn into_iter(self) -> Self::IntoIter {
+        self.field_values.into_iter()
+    }
+}
+
 impl Document {
     /// Creates a new, empty document object
     pub fn new() -> Document {
@@ -51,70 +71,68 @@ impl Document {
         self.field_values.len()
     }
 
-    /// Returns true iff the document contains no fields.
+    /// Returns true if the document contains no fields.
     pub fn is_empty(&self) -> bool {
         self.field_values.is_empty()
     }
 
-    /// Retain only the field that are matching the
-    /// predicate given in argument.
-    pub fn filter_fields<P: Fn(Field) -> bool>(&mut self, predicate: P) {
-        self.field_values
-            .retain(|field_value| predicate(field_value.field()));
-    }
-
     /// Adding a facet to the document.
     pub fn add_facet<F>(&mut self, field: Field, path: F)
-    where
-        Facet: From<F>,
-    {
+    where Facet: From<F> {
         let facet = Facet::from(path);
         let value = Value::Facet(facet);
-        self.add(FieldValue::new(field, value));
+        self.add_field_value(field, value);
     }
 
     /// Add a text field.
     pub fn add_text<S: ToString>(&mut self, field: Field, text: S) {
-        self.add(FieldValue::new(field, Value::Str(text.to_string())));
+        let value = Value::Str(text.to_string());
+        self.add_field_value(field, value);
     }
 
     /// Add a pre-tokenized text field.
-    pub fn add_pre_tokenized_text(
-        &mut self,
-        field: Field,
-        pre_tokenized_text: &PreTokenizedString,
-    ) {
-        let value = Value::PreTokStr(pre_tokenized_text.clone());
-        self.add(FieldValue::new(field, value));
+    pub fn add_pre_tokenized_text(&mut self, field: Field, pre_tokenized_text: PreTokenizedString) {
+        self.add_field_value(field, pre_tokenized_text);
     }
 
     /// Add a u64 field
     pub fn add_u64(&mut self, field: Field, value: u64) {
-        self.add(FieldValue::new(field, Value::U64(value)));
+        self.add_field_value(field, value);
     }
 
     /// Add a i64 field
     pub fn add_i64(&mut self, field: Field, value: i64) {
-        self.add(FieldValue::new(field, Value::I64(value)));
+        self.add_field_value(field, value);
     }
 
     /// Add a f64 field
     pub fn add_f64(&mut self, field: Field, value: f64) {
-        self.add(FieldValue::new(field, Value::F64(value)));
+        self.add_field_value(field, value);
     }
 
     /// Add a date field
-    pub fn add_date(&mut self, field: Field, value: &DateTime) {
-        self.add(FieldValue::new(field, Value::Date(*value)));
+    pub fn add_date(&mut self, field: Field, value: DateTime) {
+        self.add_field_value(field, value);
     }
 
     /// Add a bytes field
     pub fn add_bytes<T: Into<Vec<u8>>>(&mut self, field: Field, value: T) {
-        self.add(FieldValue::new(field, Value::Bytes(value.into())))
+        self.add_field_value(field, value.into());
     }
 
-    /// Add a field value
-    pub fn add(&mut self, field_value: FieldValue) {
+    /// Add a bytes field
+    pub fn add_json_object(
+        &mut self,
+        field: Field,
+        json_object: serde_json::Map<String, serde_json::Value>,
+    ) {
+        self.add_field_value(field, json_object);
+    }
+
+    /// Add a (field, value) to the document.
+    pub fn add_field_value<T: Into<Value>>(&mut self, field: Field, typed_val: T) {
+        let value = typed_val.into();
+        let field_value = FieldValue { field, value };
         self.field_values.push(field_value);
     }
 
@@ -127,30 +145,29 @@ impl Document {
     ///
     /// The result of this method is not cached and is
     /// computed on the fly when this method is called.
-    pub fn get_sorted_field_values(&self) -> Vec<(Field, Vec<&FieldValue>)> {
+    pub fn get_sorted_field_values(&self) -> Vec<(Field, Vec<&Value>)> {
         let mut field_values: Vec<&FieldValue> = self.field_values().iter().collect();
         field_values.sort_by_key(|field_value| field_value.field());
 
-        let mut grouped_field_values = vec![];
-
-        let mut current_field;
-        let mut current_group;
-
         let mut field_values_it = field_values.into_iter();
-        if let Some(field_value) = field_values_it.next() {
-            current_field = field_value.field();
-            current_group = vec![field_value]
+
+        let first_field_value = if let Some(first_field_value) = field_values_it.next() {
+            first_field_value
         } else {
-            return grouped_field_values;
-        }
+            return Vec::new();
+        };
+
+        let mut grouped_field_values = vec![];
+        let mut current_field = first_field_value.field();
+        let mut current_group = vec![first_field_value.value()];
 
         for field_value in field_values_it {
             if field_value.field() == current_field {
-                current_group.push(field_value);
+                current_group.push(field_value.value());
             } else {
                 grouped_field_values.push((
                     current_field,
-                    mem::replace(&mut current_group, vec![field_value]),
+                    mem::replace(&mut current_group, vec![field_value.value()]),
                 ));
                 current_field = field_value.field();
             }
@@ -172,21 +189,6 @@ impl Document {
     pub fn get_first(&self, field: Field) -> Option<&Value> {
         self.get_all(field).next()
     }
-
-    /// Prepares Document for being stored in the document store
-    ///
-    /// Method transforms PreTokenizedString values into String
-    /// values.
-    pub fn prepare_for_store(&mut self) {
-        for field_value in &mut self.field_values {
-            if let Value::PreTokStr(pre_tokenized_text) = field_value.value() {
-                *field_value = FieldValue::new(
-                    field_value.field(),
-                    Value::Str(pre_tokenized_text.text.clone()), //< TODO somehow remove .clone()
-                );
-            }
-        }
-    }
 }
 
 impl BinarySerializable for Document {
@@ -212,7 +214,6 @@ impl BinarySerializable for Document {
 mod tests {
 
     use crate::schema::*;
-    use crate::tokenizer::{PreTokenizedString, Token};
 
     #[test]
     fn test_doc() {
@@ -222,38 +223,4 @@ mod tests {
         doc.add_text(text_field, "My title");
         assert_eq!(doc.field_values().len(), 1);
     }
-
-    #[test]
-    fn test_prepare_for_store() {
-        let mut schema_builder = Schema::builder();
-        let text_field = schema_builder.add_text_field("title", TEXT);
-        let mut doc = Document::default();
-
-        let pre_tokenized_text = PreTokenizedString {
-            text: String::from("A"),
-            tokens: vec![Token {
-                offset_from: 0,
-                offset_to: 1,
-                position: 0,
-                text: String::from("A"),
-                position_length: 1,
-            }],
-        };
-
-        doc.add_pre_tokenized_text(text_field, &pre_tokenized_text);
-        doc.add_text(text_field, "title");
-        doc.prepare_for_store();
-
-        assert_eq!(doc.field_values().len(), 2);
-
-        match doc.field_values()[0].value() {
-            Value::Str(ref text) => assert_eq!(text, "A"),
-            _ => panic!("Incorrect variant of Value"),
-        }
-
-        match doc.field_values()[1].value() {
-            Value::Str(ref text) => assert_eq!(text, "title"),
-            _ => panic!("Incorrect variant of Value"),
-        }
-    }
 }
