@@ -1,14 +1,14 @@
-use common::BinarySerializable;
-use once_cell::sync::Lazy;
-use regex::Regex;
-use serde::{Deserialize, Deserializer, Serialize, Serializer};
-use std::borrow::Borrow;
-use std::borrow::Cow;
+use std::borrow::{Borrow, Cow};
 use std::fmt::{self, Debug, Display, Formatter};
 use std::io::{self, Read, Write};
 use std::str;
 use std::string::FromUtf8Error;
 
+use common::BinarySerializable;
+use once_cell::sync::Lazy;
+use regex::Regex;
+use serde::{Deserialize, Deserializer, Serialize, Serializer};
+
 const SLASH_BYTE: u8 = b'/';
 const ESCAPE_BYTE: u8 = b'\\';
 
@@ -49,7 +49,7 @@ impl Facet {
         Facet("".to_string())
     }
 
-    /// Returns true iff the facet is the root facet `/`.
+    /// Returns true if the facet is the root facet `/`.
     pub fn is_root(&self) -> bool {
         self.encoded_str().is_empty()
     }
@@ -74,7 +74,7 @@ impl Facet {
     /// Creates a `Facet` from its binary representation.
     pub fn from_encoded(encoded_bytes: Vec<u8>) -> Result<Facet, FromUtf8Error> {
         // facet bytes validation. `0u8` is used a separator but that is still legal utf-8
-        //Ok(Facet(String::from_utf8(encoded_bytes)?))
+        // Ok(Facet(String::from_utf8(encoded_bytes)?))
         String::from_utf8(encoded_bytes).map(Facet)
     }
 
@@ -84,9 +84,7 @@ impl Facet {
     /// contains a `/` or a `\`, it should be escaped
     /// using an anti-slash `/`.
     pub fn from_text<T>(path: &T) -> Result<Facet, FacetParseError>
-    where
-        T: ?Sized + AsRef<str>,
-    {
+    where T: ?Sized + AsRef<str> {
         #[derive(Copy, Clone)]
         enum State {
             Escaped,
@@ -212,18 +210,14 @@ fn escape_slashes(s: &str) -> Cow<'_, str> {
 
 impl Serialize for Facet {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
+    where S: Serializer {
         serializer.serialize_str(&self.to_string())
     }
 }
 
 impl<'de> Deserialize<'de> for Facet {
     fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
-    where
-        D: Deserializer<'de>,
-    {
+    where D: Deserializer<'de> {
         <&'de str as Deserialize<'de>>::deserialize(deserializer).map(Facet::from)
     }
 }
