@@ -1,7 +1,7 @@
-use common::BinarySerializable;
 use std::io;
-use std::io::Read;
-use std::io::Write;
+use std::io::{Read, Write};
+
+use common::BinarySerializable;
 
 /// `Field` is represented by an unsigned 32-bit integer type
 /// The schema holds the mapping between field names and `Field` objects.
@@ -17,8 +17,7 @@ impl Field {
     }
 
     /// Returns a u32 identifying uniquely a field within a schema.
-    #[allow(clippy::trivially_copy_pass_by_ref)]
-    pub const fn field_id(&self) -> u32 {
+    pub const fn field_id(self) -> u32 {
         self.0
     }
 }
