@@ -1,15 +1,15 @@
-use crate::schema::field_type::ValueParsingError;
-use std::collections::BTreeMap;
-use std::collections::HashMap;
+use std::collections::{BTreeMap, HashMap};
+use std::fmt;
 use std::sync::Arc;
 
-use super::*;
-use crate::schema::bytes_options::BytesOptions;
 use serde::de::{SeqAccess, Visitor};
 use serde::ser::SerializeSeq;
 use serde::{Deserialize, Deserializer, Serialize, Serializer};
-use serde_json::{self, Map as JsonObject, Value as JsonValue};
-use std::fmt;
+use serde_json::{self, Value as JsonValue};
+
+use super::*;
+use crate::schema::bytes_options::BytesOptions;
+use crate::schema::field_type::ValueParsingError;
 
 /// Tantivy has a very strict schema.
 /// You need to specify in advance whether a field is indexed or not,
@@ -29,7 +29,6 @@ use std::fmt;
 /// let title_field = schema_builder.add_text_field("title", TEXT);
 /// let body_field = schema_builder.add_text_field("body", TEXT);
 /// let schema = schema_builder.build();
-///
 /// ```
 #[derive(Default)]
 pub struct SchemaBuilder {
@@ -53,7 +52,7 @@ impl SchemaBuilder {
     /// by the second one.
     /// The first field will get a field id
     /// but only the second one will be indexed
-    pub fn add_u64_field<T: Into<IntOptions>>(
+    pub fn add_u64_field<T: Into<NumericOptions>>(
         &mut self,
         field_name_str: &str,
         field_options: T,
@@ -73,7 +72,7 @@ impl SchemaBuilder {
     /// by the second one.
     /// The first field will get a field id
     /// but only the second one will be indexed
-    pub fn add_i64_field<T: Into<IntOptions>>(
+    pub fn add_i64_field<T: Into<NumericOptions>>(
         &mut self,
         field_name_str: &str,
         field_options: T,
@@ -93,7 +92,7 @@ impl SchemaBuilder {
     /// by the second one.
     /// The first field will get a field id
     /// but only the second one will be indexed
-    pub fn add_f64_field<T: Into<IntOptions>>(
+    pub fn add_f64_field<T: Into<NumericOptions>>(
         &mut self,
         field_name_str: &str,
         field_options: T,
@@ -115,7 +114,7 @@ impl SchemaBuilder {
     /// by the second one.
     /// The first field will get a field id
     /// but only the second one will be indexed
-    pub fn add_date_field<T: Into<IntOptions>>(
+    pub fn add_date_field<T: Into<NumericOptions>>(
         &mut self,
         field_name_str: &str,
         field_options: T,
@@ -174,6 +173,16 @@ impl SchemaBuilder {
         self.add_field(field_entry)
     }
 
+    /// Adds a json object field to the schema.
+    pub fn add_json_field<T: Into<JsonObjectOptions>>(
+        &mut self,
+        field_name: &str,
+        field_options: T,
+    ) -> Field {
+        let field_entry = FieldEntry::new_json(field_name.to_string(), field_options.into());
+        self.add_field(field_entry)
+    }
+
     /// Adds a field entry to the schema in build.
     pub fn add_field(&mut self, field_entry: FieldEntry) -> Field {
         let field = Field::from_field_id(self.fields.len() as u32);
@@ -224,7 +233,6 @@ impl Eq for InnerSchema {}
 /// let title_field = schema_builder.add_text_field("title", TEXT);
 /// let body_field = schema_builder.add_text_field("body", TEXT);
 /// let schema = schema_builder.build();
-///
 /// ```
 #[derive(Clone, Eq, PartialEq, Debug)]
 pub struct Schema(Arc<InnerSchema>);
@@ -240,6 +248,11 @@ impl Schema {
         self.get_field_entry(field).name()
     }
 
+    /// Returns the number of fields in the schema.
+    pub fn num_fields(&self) -> usize {
+        self.0.fields.len()
+    }
+
     /// Return the list of all the `Field`s.
     pub fn fields(&self) -> impl Iterator<Item = (Field, &FieldEntry)> {
         self.0
@@ -268,11 +281,8 @@ impl Schema {
         for (field_name, values) in named_doc.0 {
             if let Some(field) = self.get_field(&field_name) {
                 for value in values {
-                    let field_value = FieldValue::new(field, value);
-                    document.add(field_value);
+                    document.add_field_value(field, value);
                 }
-            } else {
-                return Err(DocParsingError::NoSuchFieldInSchema(field_name));
             }
         }
         Ok(document)
@@ -283,11 +293,7 @@ impl Schema {
         let mut field_map = BTreeMap::new();
         for (field, field_values) in doc.get_sorted_field_values() {
             let field_name = self.get_field_name(field);
-            let values: Vec<Value> = field_values
-                .into_iter()
-                .map(FieldValue::value)
-                .cloned()
-                .collect();
+            let values: Vec<Value> = field_values.into_iter().cloned().collect();
             field_map.insert(field_name.to_string(), values);
         }
         NamedFieldDocument(field_map)
@@ -302,37 +308,36 @@ impl Schema {
 
     /// Build a document object from a json-object.
     pub fn parse_document(&self, doc_json: &str) -> Result<Document, DocParsingError> {
-        let json_obj: JsonObject<String, JsonValue> =
-            serde_json::from_str(doc_json).map_err(|_| {
-                let doc_json_sample: String = if doc_json.len() < 20 {
-                    String::from(doc_json)
-                } else {
-                    format!("{:?}...", &doc_json[0..20])
-                };
-                DocParsingError::NotJson(doc_json_sample)
-            })?;
+        let json_obj: serde_json::Map<String, JsonValue> =
+            serde_json::from_str(doc_json).map_err(|_| DocParsingError::invalid_json(doc_json))?;
+        self.json_object_to_doc(json_obj)
+    }
 
+    /// Build a document object from a json-object.
+    pub fn json_object_to_doc(
+        &self,
+        json_obj: serde_json::Map<String, JsonValue>,
+    ) -> Result<Document, DocParsingError> {
         let mut doc = Document::default();
-        for (field_name, json_value) in json_obj.iter() {
-            let field = self
-                .get_field(field_name)
-                .ok_or_else(|| DocParsingError::NoSuchFieldInSchema(field_name.clone()))?;
-            let field_entry = self.get_field_entry(field);
-            let field_type = field_entry.field_type();
-            match *json_value {
-                JsonValue::Array(ref json_items) => {
-                    for json_item in json_items {
+        for (field_name, json_value) in json_obj {
+            if let Some(field) = self.get_field(&field_name) {
+                let field_entry = self.get_field_entry(field);
+                let field_type = field_entry.field_type();
+                match json_value {
+                    JsonValue::Array(json_items) => {
+                        for json_item in json_items {
+                            let value = field_type
+                                .value_from_json(json_item)
+                                .map_err(|e| DocParsingError::ValueError(field_name.clone(), e))?;
+                            doc.add_field_value(field, value);
+                        }
+                    }
+                    _ => {
                         let value = field_type
-                            .value_from_json(json_item)
+                            .value_from_json(json_value)
                             .map_err(|e| DocParsingError::ValueError(field_name.clone(), e))?;
-                        doc.add(FieldValue::new(field, value));
+                        doc.add_field_value(field, value);
                     }
-                }
-                _ => {
-                    let value = field_type
-                        .value_from_json(json_value)
-                        .map_err(|e| DocParsingError::ValueError(field_name.clone(), e))?;
-                    doc.add(FieldValue::new(field, value));
                 }
             }
         }
@@ -342,9 +347,7 @@ impl Schema {
 
 impl Serialize for Schema {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
+    where S: Serializer {
         let mut seq = serializer.serialize_seq(Some(self.0.fields.len()))?;
         for e in &self.0.fields {
             seq.serialize_element(e)?;
@@ -355,9 +358,7 @@ impl Serialize for Schema {
 
 impl<'de> Deserialize<'de> for Schema {
     fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
-    where
-        D: Deserializer<'de>,
-    {
+    where D: Deserializer<'de> {
         struct SchemaVisitor;
 
         impl<'de> Visitor<'de> for SchemaVisitor {
@@ -368,9 +369,7 @@ impl<'de> Deserialize<'de> for Schema {
             }
 
             fn visit_seq<A>(self, mut seq: A) -> Result<Self::Value, A::Error>
-            where
-                A: SeqAccess<'de>,
-            {
+            where A: SeqAccess<'de> {
                 let mut schema = SchemaBuilder {
                     fields: Vec::with_capacity(seq.size_hint().unwrap_or(0)),
                     fields_map: HashMap::with_capacity(seq.size_hint().unwrap_or(0)),
@@ -394,25 +393,36 @@ impl<'de> Deserialize<'de> for Schema {
 pub enum DocParsingError {
     /// The payload given is not valid JSON.
     #[error("The provided string is not valid JSON")]
-    NotJson(String),
+    InvalidJson(String),
     /// One of the value node could not be parsed.
     #[error("The field '{0:?}' could not be parsed: {1:?}")]
     ValueError(String, ValueParsingError),
-    /// The json-document contains a field that is not declared in the schema.
-    #[error("The document contains a field that is not declared in the schema: {0:?}")]
-    NoSuchFieldInSchema(String),
+}
+
+impl DocParsingError {
+    /// Builds a NotJson DocParsingError
+    fn invalid_json(invalid_json: &str) -> Self {
+        let sample_json: String = if invalid_json.len() < 20 {
+            invalid_json.to_string()
+        } else {
+            format!("{:?}...", &invalid_json[0..20])
+        };
+        DocParsingError::InvalidJson(sample_json)
+    }
 }
 
 #[cfg(test)]
 mod tests {
 
-    use crate::schema::field_type::ValueParsingError;
-    use crate::schema::int_options::Cardinality::SingleValue;
-    use crate::schema::schema::DocParsingError::NotJson;
-    use crate::schema::*;
+    use std::collections::BTreeMap;
+
     use matches::{assert_matches, matches};
     use serde_json;
-    use std::collections::BTreeMap;
+
+    use crate::schema::field_type::ValueParsingError;
+    use crate::schema::numeric_options::Cardinality::SingleValue;
+    use crate::schema::schema::DocParsingError::InvalidJson;
+    use crate::schema::*;
 
     #[test]
     pub fn is_indexed_test() {
@@ -425,17 +435,25 @@ mod tests {
     #[test]
     pub fn test_schema_serialization() {
         let mut schema_builder = Schema::builder();
-        let count_options = IntOptions::default()
+        let count_options = NumericOptions::default()
             .set_stored()
             .set_fast(Cardinality::SingleValue);
-        let popularity_options = IntOptions::default()
+        let popularity_options = NumericOptions::default()
             .set_stored()
             .set_fast(Cardinality::SingleValue);
-        let score_options = IntOptions::default()
+        let score_options = NumericOptions::default()
             .set_indexed()
+            .set_fieldnorm()
             .set_fast(Cardinality::SingleValue);
         schema_builder.add_text_field("title", TEXT);
-        schema_builder.add_text_field("author", STRING);
+        schema_builder.add_text_field(
+            "author",
+            TextOptions::default().set_indexing_options(
+                TextFieldIndexing::default()
+                    .set_tokenizer("raw")
+                    .set_fieldnorms(false),
+            ),
+        );
         schema_builder.add_u64_field("count", count_options);
         schema_builder.add_i64_field("popularity", popularity_options);
         schema_builder.add_f64_field("score", score_options);
@@ -448,6 +466,7 @@ mod tests {
     "options": {
       "indexing": {
         "record": "position",
+        "fieldnorms": true,
         "tokenizer": "default"
       },
       "stored": false
@@ -459,6 +478,7 @@ mod tests {
     "options": {
       "indexing": {
         "record": "basic",
+        "fieldnorms": false,
         "tokenizer": "raw"
       },
       "stored": false
@@ -469,6 +489,7 @@ mod tests {
     "type": "u64",
     "options": {
       "indexed": false,
+      "fieldnorms": false,
       "fast": "single",
       "stored": true
     }
@@ -478,6 +499,7 @@ mod tests {
     "type": "i64",
     "options": {
       "indexed": false,
+      "fieldnorms": false,
       "fast": "single",
       "stored": true
     }
@@ -487,6 +509,7 @@ mod tests {
     "type": "f64",
     "options": {
       "indexed": true,
+      "fieldnorms": true,
       "fast": "single",
       "stored": false
     }
@@ -528,7 +551,7 @@ mod tests {
     #[test]
     pub fn test_document_to_json() {
         let mut schema_builder = Schema::builder();
-        let count_options = IntOptions::default()
+        let count_options = NumericOptions::default()
             .set_stored()
             .set_fast(Cardinality::SingleValue);
         schema_builder.add_text_field("title", TEXT);
@@ -578,32 +601,28 @@ mod tests {
     }
 
     #[test]
-    pub fn test_document_from_nameddoc_error() {
+    pub fn test_document_missing_field_no_error() {
         let schema = Schema::builder().build();
         let mut named_doc_map = BTreeMap::default();
         named_doc_map.insert(
             "title".to_string(),
             vec![Value::from("title1"), Value::from("title2")],
         );
-        let err = schema
+        schema
             .convert_named_doc(NamedFieldDocument(named_doc_map))
-            .unwrap_err();
-        assert_eq!(
-            err,
-            DocParsingError::NoSuchFieldInSchema("title".to_string())
-        );
+            .unwrap();
     }
 
     #[test]
     pub fn test_parse_document() {
         let mut schema_builder = Schema::builder();
-        let count_options = IntOptions::default()
+        let count_options = NumericOptions::default()
             .set_stored()
             .set_fast(Cardinality::SingleValue);
-        let popularity_options = IntOptions::default()
+        let popularity_options = NumericOptions::default()
             .set_stored()
             .set_fast(Cardinality::SingleValue);
-        let score_options = IntOptions::default()
+        let score_options = NumericOptions::default()
             .set_indexed()
             .set_fast(Cardinality::SingleValue);
         let title_field = schema_builder.add_text_field("title", TEXT);
@@ -628,24 +647,22 @@ mod tests {
             }"#,
                 )
                 .unwrap();
-            assert_eq!(doc.get_first(title_field).unwrap().text(), Some("my title"));
             assert_eq!(
-                doc.get_first(author_field).unwrap().text(),
+                doc.get_first(title_field).unwrap().as_text(),
+                Some("my title")
+            );
+            assert_eq!(
+                doc.get_first(author_field).unwrap().as_text(),
                 Some("fulmicoton")
             );
-            assert_eq!(doc.get_first(count_field).unwrap().u64_value(), Some(4));
-            assert_eq!(
-                doc.get_first(popularity_field).unwrap().i64_value(),
-                Some(10)
-            );
-            assert_eq!(
-                doc.get_first(score_field).unwrap().f64_value(),
-                Some(80.5f64)
-            );
+            assert_eq!(doc.get_first(count_field).unwrap().as_u64(), Some(4));
+            assert_eq!(doc.get_first(popularity_field).unwrap().as_i64(), Some(10));
+            assert_eq!(doc.get_first(score_field).unwrap().as_f64(), Some(80.5f64));
         }
         {
-            let json_err = schema.parse_document(
+            let res = schema.parse_document(
                 r#"{
+                "thisfieldisnotdefinedintheschema": "my title",
                 "title": "my title",
                 "author": "fulmicoton",
                 "count": 4,
@@ -654,7 +671,7 @@ mod tests {
                 "jambon": "bayonne"
             }"#,
             );
-            assert_matches!(json_err, Err(DocParsingError::NoSuchFieldInSchema(_)));
+            assert!(res.is_ok());
         }
         {
             let json_err = schema.parse_document(
@@ -671,7 +688,7 @@ mod tests {
                 json_err,
                 Err(DocParsingError::ValueError(
                     _,
-                    ValueParsingError::TypeError(_)
+                    ValueParsingError::TypeError { .. }
                 ))
             );
         }
@@ -689,7 +706,7 @@ mod tests {
                 json_err,
                 Err(DocParsingError::ValueError(
                     _,
-                    ValueParsingError::OverflowError(_)
+                    ValueParsingError::OverflowError { .. }
                 ))
             );
         }
@@ -707,7 +724,7 @@ mod tests {
                 json_err,
                 Err(DocParsingError::ValueError(
                     _,
-                    ValueParsingError::OverflowError(_)
+                    ValueParsingError::OverflowError { .. }
                 ))
             ));
         }
@@ -725,7 +742,7 @@ mod tests {
                 json_err,
                 Err(DocParsingError::ValueError(
                     _,
-                    ValueParsingError::OverflowError(_)
+                    ValueParsingError::OverflowError { .. }
                 ))
             );
         }
@@ -737,7 +754,7 @@ mod tests {
                 "count": 50,
             }"#,
             );
-            assert_matches!(json_err, Err(NotJson(_)));
+            assert_matches!(json_err, Err(InvalidJson(_)));
         }
     }
 
@@ -749,9 +766,10 @@ mod tests {
                 .set_tokenizer("raw")
                 .set_index_option(IndexRecordOption::Basic),
         );
-        let timestamp_options = IntOptions::default()
+        let timestamp_options = NumericOptions::default()
             .set_stored()
             .set_indexed()
+            .set_fieldnorm()
             .set_fast(SingleValue);
         schema_builder.add_text_field("_id", id_options);
         schema_builder.add_date_field("_timestamp", timestamp_options);
@@ -763,6 +781,7 @@ mod tests {
     "options": {
       "indexing": {
         "record": "position",
+        "fieldnorms": true,
         "tokenizer": "default"
       },
       "stored": false
@@ -773,6 +792,7 @@ mod tests {
     "type": "i64",
     "options": {
       "indexed": false,
+      "fieldnorms": false,
       "fast": "single",
       "stored": true
     }
@@ -793,6 +813,7 @@ mod tests {
     "options": {
       "indexing": {
         "record": "basic",
+        "fieldnorms": true,
         "tokenizer": "raw"
       },
       "stored": true
@@ -803,6 +824,7 @@ mod tests {
     "type": "date",
     "options": {
       "indexed": true,
+      "fieldnorms": true,
       "fast": "single",
       "stored": true
     }
@@ -813,6 +835,7 @@ mod tests {
     "options": {
       "indexing": {
         "record": "position",
+        "fieldnorms": true,
         "tokenizer": "default"
       },
       "stored": false
@@ -823,6 +846,7 @@ mod tests {
     "type": "i64",
     "options": {
       "indexed": false,
+      "fieldnorms": false,
       "fast": "single",
       "stored": true
     }
