@@ -1,12 +1,13 @@
-use crate::schema::flags::SchemaFlagList;
-use crate::schema::flags::StoredFlag;
-use crate::schema::IndexRecordOption;
-use serde::{Deserialize, Serialize};
 use std::borrow::Cow;
 use std::ops::BitOr;
 
+use serde::{Deserialize, Serialize};
+
+use crate::schema::flags::{SchemaFlagList, StoredFlag};
+use crate::schema::IndexRecordOption;
+
 /// Define how a text field should be handled by tantivy.
-#[derive(Clone, Debug, PartialEq, Serialize, Deserialize)]
+#[derive(Clone, Debug, PartialEq, Serialize, Deserialize, Default)]
 pub struct TextOptions {
     indexing: Option<TextFieldIndexing>,
     stored: bool,
@@ -18,42 +19,37 @@ impl TextOptions {
         self.indexing.as_ref()
     }
 
-    /// Returns true iff the text is to be stored.
+    /// Returns true if the text is to be stored.
     pub fn is_stored(&self) -> bool {
         self.stored
     }
 
     /// Sets the field as stored
+    #[must_use]
     pub fn set_stored(mut self) -> TextOptions {
         self.stored = true;
         self
     }
 
     /// Sets the field as indexed, with the specific indexing options.
+    #[must_use]
     pub fn set_indexing_options(mut self, indexing: TextFieldIndexing) -> TextOptions {
         self.indexing = Some(indexing);
         self
     }
 }
 
-impl Default for TextOptions {
-    fn default() -> TextOptions {
-        TextOptions {
-            indexing: None,
-            stored: false,
-        }
-    }
-}
-
 /// Configuration defining indexing for a text field.
 ///
 /// It defines
 /// - the amount of information that should be stored about the presence of a term in a document.
-/// Essentially, should we store the term frequency and/or the positions (See [`IndexRecordOption`](./enum.IndexRecordOption.html)).
+/// Essentially, should we store the term frequency and/or the positions (See
+/// [`IndexRecordOption`](./enum.IndexRecordOption.html)).
 /// - the name of the `Tokenizer` that should be used to process the field.
-#[derive(Clone, PartialEq, Debug, Serialize, Deserialize)]
+#[derive(Clone, PartialEq, Debug, Eq, Serialize, Deserialize)]
 pub struct TextFieldIndexing {
     record: IndexRecordOption,
+    fieldnorms: bool,
     tokenizer: Cow<'static, str>,
 }
 
@@ -62,12 +58,14 @@ impl Default for TextFieldIndexing {
         TextFieldIndexing {
             tokenizer: Cow::Borrowed("default"),
             record: IndexRecordOption::Basic,
+            fieldnorms: true,
         }
     }
 }
 
 impl TextFieldIndexing {
     /// Sets the tokenizer to be used for a given field.
+    #[must_use]
     pub fn set_tokenizer(mut self, tokenizer_name: &str) -> TextFieldIndexing {
         self.tokenizer = Cow::Owned(tokenizer_name.to_string());
         self
@@ -78,9 +76,22 @@ impl TextFieldIndexing {
         &self.tokenizer
     }
 
+    /// Sets fieldnorms
+    #[must_use]
+    pub fn set_fieldnorms(mut self, fieldnorms: bool) -> TextFieldIndexing {
+        self.fieldnorms = fieldnorms;
+        self
+    }
+
+    /// Returns true if and only if fieldnorms are stored.
+    pub fn fieldnorms(&self) -> bool {
+        self.fieldnorms
+    }
+
     /// Sets which information should be indexed with the tokens.
     ///
     /// See [IndexRecordOption](./enum.IndexRecordOption.html) for more detail.
+    #[must_use]
     pub fn set_index_option(mut self, index_option: IndexRecordOption) -> TextFieldIndexing {
         self.record = index_option;
         self
@@ -98,6 +109,7 @@ impl TextFieldIndexing {
 pub const STRING: TextOptions = TextOptions {
     indexing: Some(TextFieldIndexing {
         tokenizer: Cow::Borrowed("raw"),
+        fieldnorms: true,
         record: IndexRecordOption::Basic,
     }),
     stored: false,
@@ -107,6 +119,7 @@ pub const STRING: TextOptions = TextOptions {
 pub const TEXT: TextOptions = TextOptions {
     indexing: Some(TextFieldIndexing {
         tokenizer: Cow::Borrowed("default"),
+        fieldnorms: true,
         record: IndexRecordOption::WithFreqsAndPositions,
     }),
     stored: false,
