@@ -1,9 +1,12 @@
+use std::fmt;
+
+use serde::de::Visitor;
+use serde::{Deserialize, Deserializer, Serialize, Serializer};
+use serde_json::Map;
+
 use crate::schema::Facet;
 use crate::tokenizer::PreTokenizedString;
 use crate::DateTime;
-use serde::de::Visitor;
-use serde::{Deserialize, Deserializer, Serialize, Serializer};
-use std::{cmp::Ordering, fmt};
 
 /// Value represents the value of a any field.
 /// It is an enum over all over all of the possible field type.
@@ -21,59 +24,19 @@ pub enum Value {
     F64(f64),
     /// Signed 64-bits Date time stamp `date`
     Date(DateTime),
-    /// Hierarchical Facet
+    /// Facet
     Facet(Facet),
     /// Arbitrarily sized byte array
     Bytes(Vec<u8>),
+    /// Json object value.
+    JsonObject(serde_json::Map<String, serde_json::Value>),
 }
 
 impl Eq for Value {}
-impl PartialOrd for Value {
-    fn partial_cmp(&self, other: &Value) -> Option<Ordering> {
-        Some(self.cmp(other))
-    }
-}
-impl Ord for Value {
-    fn cmp(&self, other: &Self) -> Ordering {
-        match (self, other) {
-            (Value::Str(l), Value::Str(r)) => l.cmp(r),
-            (Value::PreTokStr(l), Value::PreTokStr(r)) => l.cmp(r),
-            (Value::U64(l), Value::U64(r)) => l.cmp(r),
-            (Value::I64(l), Value::I64(r)) => l.cmp(r),
-            (Value::Date(l), Value::Date(r)) => l.cmp(r),
-            (Value::Facet(l), Value::Facet(r)) => l.cmp(r),
-            (Value::Bytes(l), Value::Bytes(r)) => l.cmp(r),
-            (Value::F64(l), Value::F64(r)) => {
-                match (l.is_nan(), r.is_nan()) {
-                    (false, false) => l.partial_cmp(r).unwrap(), // only fail on NaN
-                    (true, true) => Ordering::Equal,
-                    (true, false) => Ordering::Less, // we define NaN as less than -∞
-                    (false, true) => Ordering::Greater,
-                }
-            }
-            (Value::Str(_), _) => Ordering::Less,
-            (_, Value::Str(_)) => Ordering::Greater,
-            (Value::PreTokStr(_), _) => Ordering::Less,
-            (_, Value::PreTokStr(_)) => Ordering::Greater,
-            (Value::U64(_), _) => Ordering::Less,
-            (_, Value::U64(_)) => Ordering::Greater,
-            (Value::I64(_), _) => Ordering::Less,
-            (_, Value::I64(_)) => Ordering::Greater,
-            (Value::F64(_), _) => Ordering::Less,
-            (_, Value::F64(_)) => Ordering::Greater,
-            (Value::Date(_), _) => Ordering::Less,
-            (_, Value::Date(_)) => Ordering::Greater,
-            (Value::Facet(_), _) => Ordering::Less,
-            (_, Value::Facet(_)) => Ordering::Greater,
-        }
-    }
-}
 
 impl Serialize for Value {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
+    where S: Serializer {
         match *self {
             Value::Str(ref v) => serializer.serialize_str(v),
             Value::PreTokStr(ref v) => v.serialize(serializer),
@@ -83,15 +46,14 @@ impl Serialize for Value {
             Value::Date(ref date) => serializer.serialize_str(&date.to_rfc3339()),
             Value::Facet(ref facet) => facet.serialize(serializer),
             Value::Bytes(ref bytes) => serializer.serialize_bytes(bytes),
+            Value::JsonObject(ref obj) => obj.serialize(serializer),
         }
     }
 }
 
 impl<'de> Deserialize<'de> for Value {
     fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
-    where
-        D: Deserializer<'de>,
-    {
+    where D: Deserializer<'de> {
         struct ValueVisitor;
 
         impl<'de> Visitor<'de> for ValueVisitor {
@@ -129,7 +91,7 @@ impl<'de> Deserialize<'de> for Value {
 impl Value {
     /// Returns the text value, provided the value is of the `Str` type.
     /// (Returns None if the value is not of the `Str` type).
-    pub fn text(&self) -> Option<&str> {
+    pub fn as_text(&self) -> Option<&str> {
         if let Value::Str(text) = self {
             Some(text)
         } else {
@@ -137,19 +99,18 @@ impl Value {
         }
     }
 
-    /// Returns the path value, provided the value is of the `Facet` type.
+    /// Returns the facet value, provided the value is of the `Facet` type.
     /// (Returns None if the value is not of the `Facet` type).
-    pub fn path(&self) -> Option<String> {
+    pub fn as_facet(&self) -> Option<&Facet> {
         if let Value::Facet(facet) = self {
-            Some(facet.to_path_string())
+            Some(facet)
         } else {
             None
         }
     }
 
     /// Returns the tokenized text, provided the value is of the `PreTokStr` type.
-    ///
-    /// Returns None if the value is not of the `PreTokStr` type.
+    /// (Returns None if the value is not of the `PreTokStr` type.)
     pub fn tokenized_text(&self) -> Option<&PreTokenizedString> {
         if let Value::PreTokStr(tokenized_text) = self {
             Some(tokenized_text)
@@ -159,9 +120,8 @@ impl Value {
     }
 
     /// Returns the u64-value, provided the value is of the `U64` type.
-    ///
-    /// Returns None if the value is not of the `U64` type.
-    pub fn u64_value(&self) -> Option<u64> {
+    /// (Returns None if the value is not of the `U64` type)
+    pub fn as_u64(&self) -> Option<u64> {
         if let Value::U64(val) = self {
             Some(*val)
         } else {
@@ -172,7 +132,7 @@ impl Value {
     /// Returns the i64-value, provided the value is of the `I64` type.
     ///
     /// Return None if the value is not of type `I64`.
-    pub fn i64_value(&self) -> Option<i64> {
+    pub fn as_i64(&self) -> Option<i64> {
         if let Value::I64(val) = self {
             Some(*val)
         } else {
@@ -183,7 +143,7 @@ impl Value {
     /// Returns the f64-value, provided the value is of the `F64` type.
     ///
     /// Return None if the value is not of type `F64`.
-    pub fn f64_value(&self) -> Option<f64> {
+    pub fn as_f64(&self) -> Option<f64> {
         if let Value::F64(value) = self {
             Some(*value)
         } else {
@@ -194,7 +154,7 @@ impl Value {
     /// Returns the Date-value, provided the value is of the `Date` type.
     ///
     /// Returns None if the value is not of type `Date`.
-    pub fn date_value(&self) -> Option<&DateTime> {
+    pub fn as_date(&self) -> Option<&DateTime> {
         if let Value::Date(date) = self {
             Some(date)
         } else {
@@ -205,13 +165,24 @@ impl Value {
     /// Returns the Bytes-value, provided the value is of the `Bytes` type.
     ///
     /// Returns None if the value is not of type `Bytes`.
-    pub fn bytes_value(&self) -> Option<&[u8]> {
+    pub fn as_bytes(&self) -> Option<&[u8]> {
         if let Value::Bytes(bytes) = self {
             Some(bytes)
         } else {
             None
         }
     }
+
+    /// Returns the json object, provided the value is of the JsonObject type.
+    ///
+    /// Returns None if the value is not of type JsonObject.
+    pub fn as_json(&self) -> Option<&Map<String, serde_json::Value>> {
+        if let Value::JsonObject(json) = self {
+            Some(json)
+        } else {
+            None
+        }
+    }
 }
 
 impl From<String> for Value {
@@ -274,13 +245,32 @@ impl From<PreTokenizedString> for Value {
     }
 }
 
+impl From<serde_json::Map<String, serde_json::Value>> for Value {
+    fn from(json_object: serde_json::Map<String, serde_json::Value>) -> Value {
+        Value::JsonObject(json_object)
+    }
+}
+
+impl From<serde_json::Value> for Value {
+    fn from(json_value: serde_json::Value) -> Value {
+        match json_value {
+            serde_json::Value::Object(json_object) => Value::JsonObject(json_object),
+            _ => {
+                panic!("Expected a json object.");
+            }
+        }
+    }
+}
+
 mod binary_serialize {
+    use std::io::{self, Read, Write};
+
+    use chrono::{TimeZone, Utc};
+    use common::{f64_to_u64, u64_to_f64, BinarySerializable};
+
     use super::Value;
     use crate::schema::Facet;
     use crate::tokenizer::PreTokenizedString;
-    use chrono::{TimeZone, Utc};
-    use common::{f64_to_u64, u64_to_f64, BinarySerializable};
-    use std::io::{self, Read, Write};
 
     const TEXT_CODE: u8 = 0;
     const U64_CODE: u8 = 1;
@@ -290,6 +280,7 @@ mod binary_serialize {
     const DATE_CODE: u8 = 5;
     const F64_CODE: u8 = 6;
     const EXT_CODE: u8 = 7;
+    const JSON_OBJ_CODE: u8 = 8;
 
     // extended types
 
@@ -338,8 +329,14 @@ mod binary_serialize {
                     BYTES_CODE.serialize(writer)?;
                     bytes.serialize(writer)
                 }
+                Value::JsonObject(ref map) => {
+                    JSON_OBJ_CODE.serialize(writer)?;
+                    serde_json::to_writer(writer, &map)?;
+                    Ok(())
+                }
             }
         }
+
         fn deserialize<R: Read>(reader: &mut R) -> io::Result<Self> {
             let type_code = u8::deserialize(reader)?;
             match type_code {
@@ -389,6 +386,10 @@ mod binary_serialize {
                         )),
                     }
                 }
+                JSON_OBJ_CODE => {
+                    let map = serde_json::from_reader(reader)?;
+                    Ok(Value::JsonObject(map))
+                }
                 _ => Err(io::Error::new(
                     io::ErrorKind::InvalidData,
                     format!("No field type is associated with code {:?}", type_code),
@@ -400,9 +401,10 @@ mod binary_serialize {
 
 #[cfg(test)]
 mod tests {
+    use std::str::FromStr;
+
     use super::Value;
     use crate::DateTime;
-    use std::str::FromStr;
 
     #[test]
     fn test_serialize_date() {
