@@ -1,14 +1,14 @@
-use crate::query::Query;
-use crate::schema::Field;
-use crate::schema::Value;
-use crate::tokenizer::{TextAnalyzer, Token};
-use crate::Searcher;
-use crate::{Document, Score};
-use htmlescape::encode_minimal;
 use std::cmp::Ordering;
 use std::collections::BTreeMap;
 use std::ops::Range;
 
+use htmlescape::encode_minimal;
+
+use crate::query::Query;
+use crate::schema::{Field, Value};
+use crate::tokenizer::{TextAnalyzer, Token};
+use crate::{Document, Score, Searcher};
+
 const DEFAULT_MAX_NUM_CHARS: usize = 150;
 
 #[derive(Debug)]
@@ -16,7 +16,6 @@ pub struct FragmentCandidate {
     score: Score,
     start_offset: usize,
     stop_offset: usize,
-    num_chars: usize,
     highlighted: Vec<Range<usize>>,
 }
 
@@ -31,7 +30,6 @@ impl FragmentCandidate {
             score: 0.0,
             start_offset,
             stop_offset: start_offset,
-            num_chars: 0,
             highlighted: vec![],
         }
     }
@@ -55,7 +53,7 @@ impl FragmentCandidate {
 /// Contains a fragment of a document, and some highlighed parts inside it.
 #[derive(Debug)]
 pub struct Snippet {
-    fragments: String,
+    fragment: String,
     highlighted: Vec<Range<usize>>,
 }
 
@@ -66,7 +64,7 @@ impl Snippet {
     /// Create a new, empty, `Snippet`
     pub fn empty() -> Snippet {
         Snippet {
-            fragments: String::new(),
+            fragment: String::new(),
             highlighted: Vec::new(),
         }
     }
@@ -77,21 +75,21 @@ impl Snippet {
         let mut start_from: usize = 0;
 
         for item in self.highlighted.iter() {
-            html.push_str(&encode_minimal(&self.fragments[start_from..item.start]));
+            html.push_str(&encode_minimal(&self.fragment[start_from..item.start]));
             html.push_str(HIGHLIGHTEN_PREFIX);
-            html.push_str(&encode_minimal(&self.fragments[item.clone()]));
+            html.push_str(&encode_minimal(&self.fragment[item.clone()]));
             html.push_str(HIGHLIGHTEN_POSTFIX);
             start_from = item.end;
         }
         html.push_str(&encode_minimal(
-            &self.fragments[start_from..self.fragments.len()],
+            &self.fragment[start_from..self.fragment.len()],
         ));
         html
     }
 
-    /// Returns a fragment from the `Snippet`.
-    pub fn fragments(&self) -> &str {
-        &self.fragments
+    /// Returns the fragment of text used in the  snippet.
+    pub fn fragment(&self) -> &str {
+        &self.fragment
     }
 
     /// Returns a list of higlighted positions from the `Snippet`.
@@ -170,14 +168,14 @@ fn select_best_fragment_combination(fragments: &[FragmentCandidate], text: &str)
             .map(|item| item.start - fragment.start_offset..item.end - fragment.start_offset)
             .collect();
         Snippet {
-            fragments: fragment_text.to_string(),
+            fragment: fragment_text.to_string(),
             highlighted,
         }
     } else {
         // when there no fragments to chose from,
         // for now create a empty snippet
         Snippet {
-            fragments: String::new(),
+            fragment: String::new(),
             highlighted: vec![],
         }
     }
@@ -209,7 +207,7 @@ fn select_best_fragment_combination(fragments: &[FragmentCandidate], text: &str)
 /// #  Quand avec mes haleurs ont fini ces tapages,
 /// #  Les Fleuves m'ont laissé descendre où je voulais.
 /// #  "#);
-/// #    index_writer.add_document(doc.clone());
+/// #    index_writer.add_document(doc.clone())?;
 /// #    index_writer.commit()?;
 /// #    let query_parser = QueryParser::for_index(&index, vec![text_field]);
 /// // ...
@@ -245,10 +243,15 @@ impl SnippetGenerator {
             if term.field() != field {
                 continue;
             }
+            let term_str = if let Some(term_str) = term.as_str() {
+                term_str
+            } else {
+                continue;
+            };
             let doc_freq = searcher.doc_freq(&term)?;
             if doc_freq > 0 {
                 let score = 1.0 / (1.0 + doc_freq as Score);
-                terms_text.insert(term.text().to_string(), score);
+                terms_text.insert(term_str.to_string(), score);
             }
         }
         let tokenizer = searcher.index().tokenizer_for_field(field)?;
@@ -277,7 +280,7 @@ impl SnippetGenerator {
     pub fn snippet_from_doc(&self, doc: &Document) -> Snippet {
         let text: String = doc
             .get_all(self.field)
-            .flat_map(Value::text)
+            .flat_map(Value::as_text)
             .collect::<Vec<&str>>()
             .join(" ");
         self.snippet(&text)
@@ -293,14 +296,15 @@ impl SnippetGenerator {
 
 #[cfg(test)]
 mod tests {
+    use std::collections::BTreeMap;
+
+    use maplit::btreemap;
+
     use super::{search_fragments, select_best_fragment_combination};
     use crate::query::QueryParser;
     use crate::schema::{IndexRecordOption, Schema, TextFieldIndexing, TextOptions, TEXT};
     use crate::tokenizer::SimpleTokenizer;
-    use crate::Index;
-    use crate::SnippetGenerator;
-    use maplit::btreemap;
-    use std::collections::BTreeMap;
+    use crate::{Index, SnippetGenerator};
 
     const TEST_TEXT: &str = r#"Rust is a systems programming language sponsored by
 Mozilla which describes it as a "safe, concurrent, practical language", supporting functional and
@@ -331,14 +335,14 @@ Survey in 2016, 2017, and 2018."#;
         }
         let snippet = select_best_fragment_combination(&fragments[..], TEST_TEXT);
         assert_eq!(
-            snippet.fragments,
-            "Rust is a systems programming language sponsored by\n\
-             Mozilla which describes it as a \"safe"
+            snippet.fragment,
+            "Rust is a systems programming language sponsored by\nMozilla which describes it as a \
+             \"safe"
         );
         assert_eq!(
             snippet.to_html(),
-            "<b>Rust</b> is a systems programming <b>language</b> \
-             sponsored by\nMozilla which describes it as a &quot;safe"
+            "<b>Rust</b> is a systems programming <b>language</b> sponsored by\nMozilla which \
+             describes it as a &quot;safe"
         )
     }
 
@@ -364,7 +368,7 @@ Survey in 2016, 2017, and 2018."#;
                 String::from("language") => 1.0
             };
             let fragments = search_fragments(&From::from(SimpleTokenizer), TEST_TEXT, &terms, 20);
-            //assert_eq!(fragments.len(), 7);
+            // assert_eq!(fragments.len(), 7);
             {
                 let first = &fragments[0];
                 assert_eq!(first.score, 0.9);
@@ -393,7 +397,7 @@ Survey in 2016, 2017, and 2018."#;
         }
 
         let snippet = select_best_fragment_combination(&fragments[..], text);
-        assert_eq!(snippet.fragments, "c d");
+        assert_eq!(snippet.fragment, "c d");
         assert_eq!(snippet.to_html(), "<b>c</b> d");
     }
 
@@ -415,7 +419,7 @@ Survey in 2016, 2017, and 2018."#;
         }
 
         let snippet = select_best_fragment_combination(&fragments[..], text);
-        assert_eq!(snippet.fragments, "e f");
+        assert_eq!(snippet.fragment, "e f");
         assert_eq!(snippet.to_html(), "e <b>f</b>");
     }
 
@@ -438,7 +442,7 @@ Survey in 2016, 2017, and 2018."#;
         }
 
         let snippet = select_best_fragment_combination(&fragments[..], text);
-        assert_eq!(snippet.fragments, "e f g");
+        assert_eq!(snippet.fragment, "e f g");
         assert_eq!(snippet.to_html(), "e <b>f</b> g");
     }
 
@@ -454,7 +458,7 @@ Survey in 2016, 2017, and 2018."#;
         assert_eq!(fragments.len(), 0);
 
         let snippet = select_best_fragment_combination(&fragments[..], text);
-        assert_eq!(snippet.fragments, "");
+        assert_eq!(snippet.fragment, "");
         assert_eq!(snippet.to_html(), "");
     }
 
@@ -467,63 +471,60 @@ Survey in 2016, 2017, and 2018."#;
         assert_eq!(fragments.len(), 0);
 
         let snippet = select_best_fragment_combination(&fragments[..], text);
-        assert_eq!(snippet.fragments, "");
+        assert_eq!(snippet.fragment, "");
         assert_eq!(snippet.to_html(), "");
     }
 
     #[test]
-    fn test_snippet_generator_term_score() {
+    fn test_snippet_generator_term_score() -> crate::Result<()> {
         let mut schema_builder = Schema::builder();
         let text_field = schema_builder.add_text_field("text", TEXT);
         let schema = schema_builder.build();
         let index = Index::create_in_ram(schema);
         {
             // writing the segment
-            let mut index_writer = index.writer_for_tests().unwrap();
-            index_writer.add_document(doc!(text_field => "a"));
-            index_writer.add_document(doc!(text_field => "a"));
-            index_writer.add_document(doc!(text_field => "a b"));
-            index_writer.commit().unwrap();
+            let mut index_writer = index.writer_for_tests()?;
+            index_writer.add_document(doc!(text_field => "a"))?;
+            index_writer.add_document(doc!(text_field => "a"))?;
+            index_writer.add_document(doc!(text_field => "a b"))?;
+            index_writer.commit()?;
         }
-        let searcher = index.reader().unwrap().searcher();
+        let searcher = index.reader()?.searcher();
         let query_parser = QueryParser::for_index(&index, vec![text_field]);
         {
-            let query = query_parser.parse_query("e").unwrap();
-            let snippet_generator =
-                SnippetGenerator::create(&searcher, &*query, text_field).unwrap();
+            let query = query_parser.parse_query("e")?;
+            let snippet_generator = SnippetGenerator::create(&searcher, &*query, text_field)?;
             assert!(snippet_generator.terms_text().is_empty());
         }
         {
-            let query = query_parser.parse_query("a").unwrap();
-            let snippet_generator =
-                SnippetGenerator::create(&searcher, &*query, text_field).unwrap();
+            let query = query_parser.parse_query("a")?;
+            let snippet_generator = SnippetGenerator::create(&searcher, &*query, text_field)?;
             assert_eq!(
                 &btreemap!("a".to_string() => 0.25),
                 snippet_generator.terms_text()
             );
         }
         {
-            let query = query_parser.parse_query("a b").unwrap();
-            let snippet_generator =
-                SnippetGenerator::create(&searcher, &*query, text_field).unwrap();
+            let query = query_parser.parse_query("a b")?;
+            let snippet_generator = SnippetGenerator::create(&searcher, &*query, text_field)?;
             assert_eq!(
                 &btreemap!("a".to_string() => 0.25, "b".to_string() => 0.5),
                 snippet_generator.terms_text()
             );
         }
         {
-            let query = query_parser.parse_query("a b c").unwrap();
-            let snippet_generator =
-                SnippetGenerator::create(&searcher, &*query, text_field).unwrap();
+            let query = query_parser.parse_query("a b c")?;
+            let snippet_generator = SnippetGenerator::create(&searcher, &*query, text_field)?;
             assert_eq!(
                 &btreemap!("a".to_string() => 0.25, "b".to_string() => 0.5),
                 snippet_generator.terms_text()
             );
         }
+        Ok(())
     }
 
     #[test]
-    fn test_snippet_generator() {
+    fn test_snippet_generator() -> crate::Result<()> {
         let mut schema_builder = Schema::builder();
         let text_options = TextOptions::default().set_indexing_options(
             TextFieldIndexing::default()
@@ -535,12 +536,10 @@ Survey in 2016, 2017, and 2018."#;
         let index = Index::create_in_ram(schema);
         {
             // writing the segment
-            let mut index_writer = index.writer_for_tests().unwrap();
-            {
-                let doc = doc ! (text_field => TEST_TEXT);
-                index_writer.add_document(doc);
-            }
-            index_writer.commit().unwrap();
+            let mut index_writer = index.writer_for_tests()?;
+            let doc = doc!(text_field => TEST_TEXT);
+            index_writer.add_document(doc)?;
+            index_writer.commit()?;
         }
         let searcher = index.reader().unwrap().searcher();
         let query_parser = QueryParser::for_index(&index, vec![text_field]);
@@ -549,12 +548,22 @@ Survey in 2016, 2017, and 2018."#;
             SnippetGenerator::create(&searcher, &*query, text_field).unwrap();
         {
             let snippet = snippet_generator.snippet(TEST_TEXT);
-            assert_eq!(snippet.to_html(), "imperative-procedural paradigms. <b>Rust</b> is syntactically similar to C++[according to whom?],\nbut its <b>designers</b> intend it to provide better memory safety");
+            assert_eq!(
+                snippet.to_html(),
+                "imperative-procedural paradigms. <b>Rust</b> is syntactically similar to \
+                 C++[according to whom?],\nbut its <b>designers</b> intend it to provide better \
+                 memory safety"
+            );
         }
         {
             snippet_generator.set_max_num_chars(90);
             let snippet = snippet_generator.snippet(TEST_TEXT);
-            assert_eq!(snippet.to_html(), "<b>Rust</b> is syntactically similar to C++[according to whom?],\nbut its <b>designers</b> intend it to");
+            assert_eq!(
+                snippet.to_html(),
+                "<b>Rust</b> is syntactically similar to C++[according to whom?],\nbut its \
+                 <b>designers</b> intend it to"
+            );
         }
+        Ok(())
     }
 }
