@@ -1,18 +1,18 @@
-/*!
-Representations for the space usage of various parts of a Tantivy index.
+//! Representations for the space usage of various parts of a Tantivy index.
+//!
+//! This can be used programmatically, and will also be exposed in a human readable fashion in
+//! tantivy-cli.
+//!
+//! One important caveat for all of this functionality is that none of it currently takes
+//! storage-level details into consideration. For example, if your file system block size is 4096
+//! bytes, we can under-count actual resultant space usage by up to 4095 bytes per file.
 
-This can be used programmatically, and will also be exposed in a human readable fashion in
-tantivy-cli.
+use std::collections::HashMap;
 
-One important caveat for all of this functionality is that none of it currently takes storage-level
-details into consideration. For example, if your file system block size is 4096 bytes, we can
-under-count actual resultant space usage by up to 4095 bytes per file.
-*/
+use serde::{Deserialize, Serialize};
 
 use crate::schema::Field;
 use crate::SegmentComponent;
-use serde::{Deserialize, Serialize};
-use std::collections::HashMap;
 
 /// Indicates space usage in bytes
 pub type ByteCount = usize;
@@ -286,11 +286,8 @@ impl FieldUsage {
 #[cfg(test)]
 mod test {
     use crate::core::Index;
-    use crate::schema::Field;
-    use crate::schema::Schema;
-    use crate::schema::{FAST, INDEXED, STORED, TEXT};
-    use crate::space_usage::ByteCount;
-    use crate::space_usage::PerFieldSpaceUsage;
+    use crate::schema::{Field, Schema, FAST, INDEXED, STORED, TEXT};
+    use crate::space_usage::{ByteCount, PerFieldSpaceUsage};
     use crate::Term;
 
     #[test]
@@ -321,24 +318,24 @@ mod test {
     }
 
     #[test]
-    fn test_fast_indexed() {
+    fn test_fast_indexed() -> crate::Result<()> {
         let mut schema_builder = Schema::builder();
         let name = schema_builder.add_u64_field("name", FAST | INDEXED);
         let schema = schema_builder.build();
         let index = Index::create_in_ram(schema);
 
         {
-            let mut index_writer = index.writer_for_tests().unwrap();
-            index_writer.add_document(doc!(name => 1u64));
-            index_writer.add_document(doc!(name => 2u64));
-            index_writer.add_document(doc!(name => 10u64));
-            index_writer.add_document(doc!(name => 20u64));
-            index_writer.commit().unwrap();
+            let mut index_writer = index.writer_for_tests()?;
+            index_writer.add_document(doc!(name => 1u64))?;
+            index_writer.add_document(doc!(name => 2u64))?;
+            index_writer.add_document(doc!(name => 10u64))?;
+            index_writer.add_document(doc!(name => 20u64))?;
+            index_writer.commit()?;
         }
 
-        let reader = index.reader().unwrap();
+        let reader = index.reader()?;
         let searcher = reader.searcher();
-        let searcher_space_usage = searcher.space_usage().unwrap();
+        let searcher_space_usage = searcher.space_usage()?;
         assert!(searcher_space_usage.total() > 0);
         assert_eq!(1, searcher_space_usage.segments().len());
 
@@ -355,29 +352,30 @@ mod test {
         // TODO: understand why the following fails
         //        assert_eq!(0, segment.store().total());
         assert_eq!(0, segment.deletes());
+        Ok(())
     }
 
     #[test]
-    fn test_text() {
+    fn test_text() -> crate::Result<()> {
         let mut schema_builder = Schema::builder();
         let name = schema_builder.add_text_field("name", TEXT);
         let schema = schema_builder.build();
         let index = Index::create_in_ram(schema);
 
         {
-            let mut index_writer = index.writer_for_tests().unwrap();
-            index_writer.add_document(doc!(name => "hi"));
-            index_writer.add_document(doc!(name => "this is a test"));
+            let mut index_writer = index.writer_for_tests()?;
+            index_writer.add_document(doc!(name => "hi"))?;
+            index_writer.add_document(doc!(name => "this is a test"))?;
             index_writer.add_document(
                 doc!(name => "some more documents with some word overlap with the other test"),
-            );
-            index_writer.add_document(doc!(name => "hello hi goodbye"));
-            index_writer.commit().unwrap();
+            )?;
+            index_writer.add_document(doc!(name => "hello hi goodbye"))?;
+            index_writer.commit()?;
         }
 
-        let reader = index.reader().unwrap();
+        let reader = index.reader()?;
         let searcher = reader.searcher();
-        let searcher_space_usage = searcher.space_usage().unwrap();
+        let searcher_space_usage = searcher.space_usage()?;
         assert!(searcher_space_usage.total() > 0);
         assert_eq!(1, searcher_space_usage.segments().len());
 
@@ -394,28 +392,29 @@ mod test {
         // TODO: understand why the following fails
         //        assert_eq!(0, segment.store().total());
         assert_eq!(0, segment.deletes());
+        Ok(())
     }
 
     #[test]
-    fn test_store() {
+    fn test_store() -> crate::Result<()> {
         let mut schema_builder = Schema::builder();
         let name = schema_builder.add_text_field("name", STORED);
         let schema = schema_builder.build();
         let index = Index::create_in_ram(schema);
 
         {
-            let mut index_writer = index.writer_for_tests().unwrap();
-            index_writer.add_document(doc!(name => "hi"));
-            index_writer.add_document(doc!(name => "this is a test"));
+            let mut index_writer = index.writer_for_tests()?;
+            index_writer.add_document(doc!(name => "hi"))?;
+            index_writer.add_document(doc!(name => "this is a test"))?;
             index_writer.add_document(
                 doc!(name => "some more documents with some word overlap with the other test"),
-            );
-            index_writer.add_document(doc!(name => "hello hi goodbye"));
-            index_writer.commit().unwrap();
+            )?;
+            index_writer.add_document(doc!(name => "hello hi goodbye"))?;
+            index_writer.commit()?;
         }
-        let reader = index.reader().unwrap();
+        let reader = index.reader()?;
         let searcher = reader.searcher();
-        let searcher_space_usage = searcher.space_usage().unwrap();
+        let searcher_space_usage = searcher.space_usage()?;
         assert!(searcher_space_usage.total() > 0);
         assert_eq!(1, searcher_space_usage.segments().len());
 
@@ -432,6 +431,7 @@ mod test {
         assert!(segment.store().total() > 0);
         assert!(segment.store().total() < 512);
         assert_eq!(0, segment.deletes());
+        Ok(())
     }
 
     #[test]
@@ -443,10 +443,10 @@ mod test {
 
         {
             let mut index_writer = index.writer_for_tests()?;
-            index_writer.add_document(doc!(name => 1u64));
-            index_writer.add_document(doc!(name => 2u64));
-            index_writer.add_document(doc!(name => 3u64));
-            index_writer.add_document(doc!(name => 4u64));
+            index_writer.add_document(doc!(name => 1u64))?;
+            index_writer.add_document(doc!(name => 2u64))?;
+            index_writer.add_document(doc!(name => 3u64))?;
+            index_writer.add_document(doc!(name => 4u64))?;
             index_writer.commit()?;
         }
 
