@@ -1,8 +1,9 @@
+use common::{BinarySerializable, VInt};
+
 use crate::directory::OwnedBytes;
 use crate::store::index::block::CheckpointBlock;
 use crate::store::index::Checkpoint;
 use crate::DocId;
-use common::{BinarySerializable, VInt};
 
 pub struct LayerCursor<'a> {
     remaining: &'a [u8],
@@ -19,9 +20,7 @@ impl<'a> Iterator for LayerCursor<'a> {
                 return None;
             }
             let (block_mut, remaining_mut) = (&mut self.block, &mut self.remaining);
-            if block_mut.deserialize(remaining_mut).is_err() {
-                return None;
-            }
+            block_mut.deserialize(remaining_mut).ok()?;
             self.cursor = 0;
         }
         let res = Some(self.block.get(self.cursor));
