@@ -1,16 +1,21 @@
+use std::io;
+use std::sync::atomic::{AtomicUsize, Ordering};
+use std::sync::{Arc, Mutex};
+
+use common::{BinarySerializable, HasLen, VInt};
+use lru::LruCache;
+use ownedbytes::OwnedBytes;
+
+use super::footer::DocStoreFooter;
+use super::index::SkipIndex;
 use super::Compressor;
-use super::{footer::DocStoreFooter, index::SkipIndex};
-use crate::directory::{FileSlice, OwnedBytes};
+use crate::directory::FileSlice;
+use crate::error::DataCorruption;
+use crate::fastfield::AliveBitSet;
 use crate::schema::Document;
 use crate::space_usage::StoreSpaceUsage;
 use crate::store::index::Checkpoint;
 use crate::DocId;
-use crate::{error::DataCorruption, fastfield::DeleteBitSet};
-use common::{BinarySerializable, HasLen, VInt};
-use lru::LruCache;
-use std::io;
-use std::sync::atomic::{AtomicUsize, Ordering};
-use std::sync::{Arc, Mutex};
 
 const LRU_CACHE_CAPACITY: usize = 100;
 
@@ -105,14 +110,13 @@ impl StoreReader {
         Ok(Document::deserialize(&mut doc_bytes)?)
     }
 
-    /// Reads raw bytes of a given document. Returns `RawDocument`, which contains the block of a document and its start and end
-    /// position within the block.
+    /// Reads raw bytes of a given document. Returns `RawDocument`, which contains the block of a
+    /// document and its start and end position within the block.
     ///
     /// Calling `.get(doc)` is relatively costly as it requires
     /// decompressing a compressed block. The store utilizes a LRU cache,
     /// so accessing docs from the same compressed block should be faster.
     /// For that reason a store reader should be kept and reused.
-    ///
     pub fn get_document_bytes(&self, doc_id: DocId) -> crate::Result<OwnedBytes> {
         let checkpoint = self.block_checkpoint(doc_id).ok_or_else(|| {
             crate::TantivyError::InvalidArgument(format!("Failed to lookup Doc #{}.", doc_id))
@@ -133,12 +137,12 @@ impl StoreReader {
 
     /// Iterator over all Documents in their order as they are stored in the doc store.
     /// Use this, if you want to extract all Documents from the doc store.
-    /// The delete_bitset has to be forwarded from the `SegmentReader` or the results maybe wrong.
+    /// The alive_bitset has to be forwarded from the `SegmentReader` or the results maybe wrong.
     pub fn iter<'a: 'b, 'b>(
         &'b self,
-        delete_bitset: Option<&'a DeleteBitSet>,
+        alive_bitset: Option<&'a AliveBitSet>,
     ) -> impl Iterator<Item = crate::Result<Document>> + 'b {
-        self.iter_raw(delete_bitset).map(|doc_bytes_res| {
+        self.iter_raw(alive_bitset).map(|doc_bytes_res| {
             let mut doc_bytes = doc_bytes_res?;
             Ok(Document::deserialize(&mut doc_bytes)?)
         })
@@ -146,12 +150,12 @@ impl StoreReader {
 
     /// Iterator over all RawDocuments in their order as they are stored in the doc store.
     /// Use this, if you want to extract all Documents from the doc store.
-    /// The delete_bitset has to be forwarded from the `SegmentReader` or the results maybe wrong.
+    /// The alive_bitset has to be forwarded from the `SegmentReader` or the results maybe wrong.
     pub(crate) fn iter_raw<'a: 'b, 'b>(
         &'b self,
-        delete_bitset: Option<&'a DeleteBitSet>,
+        alive_bitset: Option<&'a AliveBitSet>,
     ) -> impl Iterator<Item = crate::Result<OwnedBytes>> + 'b {
-        let last_docid = self
+        let last_doc_id = self
             .block_checkpoints()
             .last()
             .map(|checkpoint| checkpoint.doc_range.end)
@@ -164,7 +168,7 @@ impl StoreReader {
         let mut block_start_pos = 0;
         let mut num_skipped = 0;
         let mut reset_block_pos = false;
-        (0..last_docid)
+        (0..last_doc_id)
             .filter_map(move |doc_id| {
                 // filter_map is only used to resolve lifetime issues between the two closures on
                 // the outer variables
@@ -179,7 +183,7 @@ impl StoreReader {
                     num_skipped = 0;
                 }
 
-                let alive = delete_bitset.map_or(true, |bitset| bitset.is_alive(doc_id));
+                let alive = alive_bitset.map_or(true, |bitset| bitset.is_alive(doc_id));
                 if alive {
                     let ret = Some((curr_block.clone(), num_skipped, reset_block_pos));
                     // the map block will move over the num_skipped, so we reset to 0
@@ -196,7 +200,8 @@ impl StoreReader {
                 let block = block
                     .ok_or_else(|| {
                         DataCorruption::comment_only(
-                            "the current checkpoint in the doc store iterator is none, this should never happen",
+                            "the current checkpoint in the doc store iterator is none, this \
+                             should never happen",
                         )
                     })?
                     .map_err(|error_kind| {
@@ -235,16 +240,72 @@ impl StoreReader {
     }
 }
 
+#[cfg(feature = "quickwit")]
+impl StoreReader {
+    async fn read_block_async(&self, checkpoint: &Checkpoint) -> crate::AsyncIoResult<Block> {
+        if let Some(block) = self.cache.lock().unwrap().get(&checkpoint.byte_range.start) {
+            self.cache_hits.fetch_add(1, Ordering::SeqCst);
+            return Ok(block.clone());
+        }
+
+        self.cache_misses.fetch_add(1, Ordering::SeqCst);
+
+        let compressed_block = self
+            .data
+            .slice(checkpoint.byte_range.clone())
+            .read_bytes_async()
+            .await?;
+        let mut decompressed_block = vec![];
+        self.compressor
+            .decompress(compressed_block.as_slice(), &mut decompressed_block)?;
+
+        let block = OwnedBytes::new(decompressed_block);
+        self.cache
+            .lock()
+            .unwrap()
+            .put(checkpoint.byte_range.start, block.clone());
+
+        Ok(block)
+    }
+
+    /// Fetches a document asynchronously.
+    async fn get_document_bytes_async(&self, doc_id: DocId) -> crate::Result<OwnedBytes> {
+        let checkpoint = self.block_checkpoint(doc_id).ok_or_else(|| {
+            crate::TantivyError::InvalidArgument(format!("Failed to lookup Doc #{}.", doc_id))
+        })?;
+        let block = self.read_block_async(&checkpoint).await?;
+        let mut cursor = &block[..];
+        let cursor_len_before = cursor.len();
+        for _ in checkpoint.doc_range.start..doc_id {
+            let doc_length = VInt::deserialize(&mut cursor)?.val() as usize;
+            cursor = &cursor[doc_length..];
+        }
+        let doc_length = VInt::deserialize(&mut cursor)?.val() as usize;
+        let start_pos = cursor_len_before - cursor.len();
+        let end_pos = cursor_len_before - cursor.len() + doc_length;
+        Ok(block.slice(start_pos..end_pos))
+    }
+
+    /// Reads raw bytes of a given document. Returns `RawDocument`, which contains the block of a
+    /// document and its start and end position within the block.
+    pub(crate) async fn get_async(&self, doc_id: DocId) -> crate::Result<Document> {
+        let mut doc_bytes = self.get_document_bytes_async(doc_id).await?;
+        Ok(Document::deserialize(&mut doc_bytes)?)
+    }
+}
+
 #[cfg(test)]
 mod tests {
-    use super::*;
-    use crate::schema::Document;
-    use crate::schema::Field;
-    use crate::{directory::RamDirectory, store::tests::write_lorem_ipsum_store, Directory};
     use std::path::Path;
 
+    use super::*;
+    use crate::directory::RamDirectory;
+    use crate::schema::{Document, Field};
+    use crate::store::tests::write_lorem_ipsum_store;
+    use crate::Directory;
+
     fn get_text_field<'a>(doc: &'a Document, field: &'a Field) -> Option<&'a str> {
-        doc.get_first(*field).and_then(|f| f.text())
+        doc.get_first(*field).and_then(|f| f.as_text())
     }
 
     #[test]
