@@ -88,7 +88,8 @@
 //! If you built your schema programmatically, a complete example
 //! could like this for instance.
 //!
-//! Note that tokens with a len greater or equal to [`MAX_TOKEN_LEN`](./constant.MAX_TOKEN_LEN.html).
+//! Note that tokens with a len greater or equal to
+//! [`MAX_TOKEN_LEN`](./constant.MAX_TOKEN_LEN.html).
 //!
 //! # Example
 //!
@@ -116,9 +117,9 @@
 //!     .tokenizers()
 //!     .register("custom_en", custom_en_tokenizer);
 //! ```
-//!
 mod alphanum_only;
 mod ascii_folding_filter;
+mod empty_tokenizer;
 mod facet_tokenizer;
 mod lower_caser;
 mod ngram_tokenizer;
@@ -127,7 +128,6 @@ mod remove_long;
 mod simple_tokenizer;
 mod stemmer;
 mod stop_word_filter;
-mod token_stream_chain;
 mod tokenized_string;
 mod tokenizer;
 mod tokenizer_manager;
@@ -143,22 +143,19 @@ pub use self::remove_long::RemoveLongFilter;
 pub use self::simple_tokenizer::SimpleTokenizer;
 pub use self::stemmer::{Language, Stemmer};
 pub use self::stop_word_filter::StopWordFilter;
-pub(crate) use self::token_stream_chain::TokenStreamChain;
-pub use self::whitespace_tokenizer::WhitespaceTokenizer;
-
 pub use self::tokenized_string::{PreTokenizedStream, PreTokenizedString};
 pub use self::tokenizer::{
     BoxTokenFilter, BoxTokenStream, TextAnalyzer, Token, TokenFilter, TokenStream, Tokenizer,
 };
-
 pub use self::tokenizer_manager::TokenizerManager;
+pub use self::whitespace_tokenizer::WhitespaceTokenizer;
 
 /// Maximum authorized len (in bytes) for a token.
 ///
 /// Tokenizer are in charge of not emitting tokens larger than this value.
 /// Currently, if a faulty tokenizer implementation emits tokens with a length larger than
-/// `2^16 - 1 - 4`, the token will simply be ignored downstream.
-pub const MAX_TOKEN_LEN: usize = u16::max_value() as usize - 4;
+/// `2^16 - 1 - 5`, the token will simply be ignored downstream.
+pub const MAX_TOKEN_LEN: usize = u16::max_value() as usize - 5;
 
 #[cfg(test)]
 pub mod tests {
